@@ -41,6 +41,9 @@
 #include <rcsc/player/player_agent.h>
 #include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
+#include <rcsc/math_util.h>
+
+//#define DEBUG
 
 namespace rcsc {
 
@@ -50,7 +53,7 @@ namespace rcsc {
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Body_TurnToAngle::execute( PlayerAgent * agent )
 {
@@ -74,7 +77,7 @@ Body_TurnToAngle::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Body_TurnToPoint::execute( PlayerAgent * agent )
 {
@@ -89,10 +92,7 @@ Body_TurnToPoint::execute( PlayerAgent * agent )
     }
 
     // relative angle from my predicted pos & body angle to point
-    Vector2D my_point
-        = self.playerType().inertiaPoint( self.pos(),
-                                          self.vel(),
-                                          M_cycle );
+    Vector2D my_point = self.inertiaPoint( M_cycle );
     AngleDeg target_rel_angle
         = ( M_point - my_point ).th() - self.body();
 
@@ -114,7 +114,7 @@ Body_TurnToPoint::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Body_TurnToBall::execute( PlayerAgent * agent )
 {
@@ -136,38 +136,78 @@ Body_TurnToBall::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Body_TackleToPoint::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
                   __FILE__": Body_TackleToPoint" );
 
-    if ( agent->world().self().tackleProbability() < 0.01 )
+    const WorldModel & wm = agent->world();
+    const ServerParam & sp = ServerParam::i();
+
+    if ( wm.self().tackleProbability() < M_min_prob )
     {
         return false;
     }
 
-    AngleDeg target_rel_angle
-        = ( M_point - agent->world().ball().pos() ).th()
-        - agent->world().self().body();
+    AngleDeg target_angle = ( M_point - wm.ball().pos() ).th();
+    AngleDeg target_rel_angle = target_angle - wm.self().body();
 
     if ( agent->config().version() < 12.0 )
     {
         if ( target_rel_angle.abs() < 90.0 )
         {
-            return agent->doTackle( ServerParam::i().maxTacklePower() );
+            dlog.addText( Logger::ACTION,
+                          __FILE__": Body_TackleToPoint. foward tackle. target=(%.1f %.1f) angle_diff=%.2f",
+                          M_point.x, M_point.y,
+                          target_rel_angle.degree() );
+            return agent->doTackle( sp.maxTacklePower() );
         }
-        else if ( ServerParam::i().maxBackTacklePower() > 0.0 )
+        else if ( sp.maxBackTacklePower() > 0.0 )
         {
             // backward case
-            return agent->doTackle( - ServerParam::i().maxBackTacklePower() );
+            dlog.addText( Logger::ACTION,
+                          __FILE__": Body_TackleToPoint. backword tackle. target=(%.1f %.1f) angle_diff=%.2f",
+                          M_point.x, M_point.y,
+                          target_rel_angle.degree() );
+            return agent->doTackle( - sp.maxBackTacklePower() );
         }
+
+        dlog.addText( Logger::ACTION,
+                      __FILE__": Body_TackleToPoint. failed. target=(%.1f %.1f) angle_diff=%.2f",
+                      target_rel_angle.degree() );
         return false;
     }
 
-    // TODO: calculate the effective tackle power
+    AngleDeg ball_rel_angle = wm.ball().rpos().th() - wm.self().body();
 
+    double eff_power
+        = sp.maxBackTacklePower()
+        + ( ( sp.maxTacklePower() - sp.maxBackTacklePower() )
+            * ( 1.0 - target_rel_angle.abs() / 180.0 ) );
+    eff_power *= sp.tacklePowerRate();
+    eff_power *= 1.0 - 0.5*( ball_rel_angle.abs() / 180.0 );
+
+    Vector2D vel = wm.ball().vel()
+        + Vector2D::polar2vector( eff_power, target_angle );
+
+    dlog.addText( Logger::ACTION,
+                  __FILE__": Body_TackleToPoint. target=(%.1f %.1f) angle_diff=%.2f accel_r=%.3f vel=(%.2f %.2f)",
+                  M_point.x, M_point.y,
+                  target_rel_angle.degree(),
+                  eff_power,
+                  vel.x, vel.y );
+
+    if ( ( vel.th() - target_angle ).abs() > 90.0 // never accelerate to the target direction
+         || vel.r() < M_min_speed ) // too small speed
+    {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": Body_TackleToPoint. failed. min_speed=%.2f reached_speed=%.2f",
+                      M_min_speed,
+                      vel.r() );
+        return false;
+    }
 
     return agent->doTackle( target_rel_angle.degree() );
 }
@@ -177,7 +217,7 @@ Body_TackleToPoint::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Neck_TurnToRelative::execute( PlayerAgent * agent )
 {
@@ -193,15 +233,15 @@ Neck_TurnToRelative::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Neck_TurnToPoint::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
                   __FILE__": Neck_TurnToPoint" );
 
-    const Vector2D next_pos = agent->effector().queuedNextMyPos();
-    const AngleDeg next_body = agent->effector().queuedNextMyBody();
+    const Vector2D next_pos = agent->effector().queuedNextSelfPos();
+    const AngleDeg next_body = agent->effector().queuedNextSelfBody();
     const double next_view_width = agent->effector().queuedNextViewWidth().width() * 0.5;
 
     for ( std::vector< Vector2D >::const_iterator p = M_points.begin();
@@ -232,21 +272,23 @@ Neck_TurnToPoint::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Neck_TurnToBall::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
                   __FILE__": Neck_TurnToBall" );
 
-    if ( ! agent->world().ball().posValid() )
+    const WorldModel & wm = agent->world();
+
+    if ( ! wm.ball().posValid() )
     {
         Neck_ScanField().execute( agent );
         return true;
     }
 
-    const Vector2D my_next = agent->effector().queuedNextMyPos();
-    const AngleDeg my_body_next = agent->effector().queuedNextMyBody();
+    const Vector2D my_next = agent->effector().queuedNextSelfPos();
+    const AngleDeg my_body_next = agent->effector().queuedNextSelfBody();
 
     const Vector2D ball_next = agent->effector().queuedNextBallPos();
     const AngleDeg ball_angle_next = ( ball_next - my_next ).th();
@@ -255,9 +297,9 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
     const double next_view_width = agent->effector().queuedNextViewWidth().width();
 
     dlog.addText( Logger::ACTION,
-                  __FILE__":%d: ball_next=(%.2f, %.2f), ball_rel_angle=%.1f, next_view=%.1f"
-                  , __LINE__,
+                  __FILE__": (Neck_TurnToBall) ball_next=(%.2f, %.2f) ball_angle=%.1f rel_angle=%.1f, next_view=%.1f",
                   ball_next.x, ball_next.y,
+                  ball_angle_next.degree(),
                   ball_rel_angle_next.degree(),
                   next_view_width );
 
@@ -265,19 +307,152 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
     if ( ball_rel_angle_next.abs()
          > ServerParam::i().maxNeckAngle() + next_view_width * 0.5 )
     {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": (Neck_TurnToBall) never look. scan field" );
         Neck_ScanField().execute( agent );
         return true;
     }
 
-    // ball is near, directly look the ball
-    if ( my_next.dist( ball_next ) < 15.0 ) // [200803]
+    double angle_buf = std::max( 0.0, next_view_width * 0.5 - 10.0 ); //[200803] 15.0
+
+    const PlayerObject * opp = wm.getOpponentNearestToBall( 10 );
+    const PlayerObject * mate = wm.getTeammateNearestToBall( 10 );
+
+    const double ball_dist = my_next.dist( ball_next );
+
+    if ( ServerParam::i().visibleDistance() * 0.7 < ball_dist // 2008-07-11
+         && ball_dist < 15.0 // 200803
+         && ( wm.existKickableTeammate() // 2008-07-11
+              || wm.existKickableOpponent()
+              || ( opp && opp->distFromBall() < ( opp->playerTypePtr()->kickableArea() + 0.3 ) )
+              || ( mate && mate->distFromBall() < ( mate->playerTypePtr()->kickableArea() + 0.3 ) )
+              )
+         )
     {
-        agent->doTurnNeck( ball_rel_angle_next
-                           - agent->world().self().neck() );
-        return true;
+        dlog.addText( Logger::ACTION,
+                      __FILE__": (Neck_TurnToBall) ball is near." );
+        //agent->doTurnNeck( ball_rel_angle_next - wm.self().neck() );
+        //return true;
+        angle_buf = std::max( 0.0, next_view_width * 0.5 - 20.0 ); // 2008-07-11
     }
 
-    double angle_buf = std::max( 0.0, next_view_width * 0.5 - 10.0 ); //[200803] 15.0
+    if ( ( wm.ball().pos().x > 0.0
+           || wm.ball().pos().absY() > ServerParam::i().pitchHalfWidth() - 8.0
+           )
+         && wm.opponentsFromSelf().size() >= 11 )
+    {
+        //
+        // find the angle that contans the largest number of players.
+        //
+
+        double best_dir = -360.0;
+        int best_score = 0;
+        double best_angle_diff = 0.0;
+
+        const double dir_step = std::max( 5.0, ( angle_buf * 2.0 ) / 10.0 );
+        const double max_dir = ball_angle_next.degree() + angle_buf + dir_step * 0.5;
+
+        dlog.addText( Logger::ACTION,
+                      __FILE__": (Neck_TurnToBall) angle_buf=%.1f max_dir=%f dir_step=%f",
+                      angle_buf,
+                      max_dir, dir_step );
+
+        const AbstractPlayerCont::const_iterator end = wm.allPlayers().end();
+        for ( double dir = ball_angle_next.degree() - angle_buf;
+              dir < max_dir;
+              dir += dir_step )
+        {
+            AngleDeg left_angle = dir - next_view_width * 0.5;
+            AngleDeg right_angle = dir + next_view_width * 0.5;
+
+            int score = 0;
+            double angle_diff = 180.0;
+
+            for ( AbstractPlayerCont::const_iterator p = wm.allPlayers().begin();
+                  p != end;
+                  ++p )
+            {
+                if ( (*p)->isSelf() ) continue;
+                //if ( (*p)->isGhost() ) continue;
+                Vector2D pos = (*p)->pos() + (*p)->vel();
+                if ( pos.dist( my_next ) > 35.0 ) continue; // XXX magic number XXX
+
+                AngleDeg angle = ( pos - my_next ).th();
+                if ( angle.isRightOf( left_angle )
+                     && angle.isLeftOf( right_angle ) )
+                {
+                    if ( (*p)->goalie()
+                         && my_next.x > 33.0 )
+                    {
+                        score += (*p)->posCount() * 2 + 20;
+                    }
+                    else if ( pos.x > my_next.x - 5.0 )
+                    {
+                        score += (*p)->posCount() * 2 + 1;
+                    }
+                    else
+                    {
+                        score += (*p)->posCount();
+                    }
+
+                    double tmp_diff = std::min( ( angle - left_angle ).abs(),
+                                                ( angle - right_angle ).abs() );
+                    if ( tmp_diff < angle_diff )
+                    {
+                        angle_diff = tmp_diff;
+                    }
+                }
+            }
+#ifdef DEBUG
+            dlog.addText( Logger::ACTION,
+                          "__ dir=%.0f  score=%d  angle_diff=%.1f",
+                          dir,
+                          score,
+                          angle_diff );
+#endif
+            if ( score > best_score )
+            {
+                best_dir = dir;
+                best_score = score;
+                best_angle_diff = angle_diff;
+#ifdef DEBUG
+                dlog.addText( Logger::ACTION,
+                              "__ update best dir (1)" );
+#endif
+            }
+            else if ( score == best_score )
+            {
+                if ( best_angle_diff < angle_diff )
+                {
+                    best_dir = dir;
+                    best_score = score;
+                    best_angle_diff = angle_diff;
+#ifdef DEBUG
+                    dlog.addText( Logger::ACTION,
+                                  "__ update best dir (2)" );
+#endif
+                }
+            }
+        }
+
+        if ( best_dir != -360.0 )
+        {
+            AngleDeg best_angle = best_dir;
+            AngleDeg neck_moment = best_angle - my_body_next - wm.self().neck();
+            dlog.addText( Logger::ACTION,
+                          __FILE__": (Neck_TurnToBall)  best_dir=%.1f best_angle=%.1f  moment=%.1f",
+                          best_dir, best_angle.degree(), neck_moment.degree() );
+            agent->doTurnNeck( neck_moment );
+            return true;
+        }
+
+        dlog.addText( Logger::ACTION,
+                      __FILE__": (Neck_TurnToBall) could not find the player in the next view range" );
+    }
+
+    //
+    // find the lowest accuracy angle
+    //
 
     AngleDeg left_rel_angle = ball_rel_angle_next - angle_buf;
     AngleDeg right_rel_angle = ball_rel_angle_next + angle_buf;
@@ -295,17 +470,16 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
     int left_sum_count = 0;
     int right_sum_count = 0;
 
-    agent->world().dirRangeCount( my_body_next + left_rel_angle,
-                                  next_view_width,
-                                  NULL, &left_sum_count, NULL );
-    agent->world().dirRangeCount( my_body_next + right_rel_angle,
-                                  next_view_width,
-                                  NULL, &right_sum_count, NULL );
+    wm.dirRangeCount( my_body_next + left_rel_angle,
+                      next_view_width,
+                      NULL, &left_sum_count, NULL );
+    wm.dirRangeCount( my_body_next + right_rel_angle,
+                      next_view_width,
+                      NULL, &right_sum_count, NULL );
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: angle_buf=%.0f  left_rel=%.0f right_rel=%.0f"
-                  " left_sum=%d  right_sum=%d"
-                  ,__FILE__, __LINE__,
+                  __FILE__": (Neck_TurnToBall) angle_buf=%.0f  left_rel=%.0f right_rel=%.0f"
+                  " left_sum=%d  right_sum=%d",
                   angle_buf,
                   left_rel_angle.degree(), right_rel_angle.degree(),
                   left_sum_count, right_sum_count );
@@ -313,11 +487,11 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
 
     if ( left_sum_count > right_sum_count )
     {
-        agent->doTurnNeck( left_rel_angle - agent->world().self().neck() );
+        agent->doTurnNeck( left_rel_angle - wm.self().neck() );
     }
     else
     {
-        agent->doTurnNeck( right_rel_angle - agent->world().self().neck() );
+        agent->doTurnNeck( right_rel_angle - wm.self().neck() );
     }
 
     return true;
@@ -328,7 +502,7 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Bhv_BodyNeckToPoint::execute( PlayerAgent * agent )
 {
@@ -400,7 +574,7 @@ Bhv_BodyNeckToPoint::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Bhv_BodyNeckToBall::execute( PlayerAgent * agent )
 {
@@ -425,22 +599,23 @@ Bhv_BodyNeckToBall::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  __FILE__": Bhv_NeckBodyToPoint.(%.1f, %.1f)",
-                  M_point.x, M_point.y );
+                  __FILE__": Bhv_NeckBodyToPoint.(%.1f, %.1f) angle_buf=%.1f",
+                  M_point.x, M_point.y,
+                  M_angle_buf );
+
+    const WorldModel & wm = agent->world();
+
+    const double angle_buf = bound( 0.0, M_angle_buf, 180.0 );
 
     // relative angle from my predicted pos & body angle to point
-    Vector2D mynext
-        = agent->world().self().pos()
-        + agent->world().self().vel();
+    Vector2D my_next = wm.self().pos() + wm.self().vel();
 
-    AngleDeg target_rel_angle
-        = ( M_point - mynext ).th()
-        - agent->world().self().body();
+    AngleDeg target_rel_angle = ( M_point - my_next ).th() - wm.self().body();
 
     //if ( ( agent->world().self().neck() - target_rel_angle ).abs() < 1.0 )
     //{
@@ -451,25 +626,24 @@ Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
     //}
 
     // can face to point only by turn_neck.
-    if ( ServerParam::i().minNeckAngle() + 5.0 < target_rel_angle.degree()
-         && target_rel_angle.degree() < ServerParam::i().maxNeckAngle() - 5.0 )
+    if ( ServerParam::i().minNeckAngle() + angle_buf < target_rel_angle.degree()
+         && target_rel_angle.degree() < ServerParam::i().maxNeckAngle() - angle_buf )
     {
         agent->doTurn( 0.0 );
         agent->setNeckAction( new Neck_TurnToRelative( target_rel_angle ) );
         return true;
     }
-    // calc the max turn angle with current my speed
+
+    // max turn angle with current my speed
     double max_turn
-        = effective_turn( ServerParam::i().maxMoment(),
-                          agent->world().self().vel().r(),
-                          agent->world().self().playerType().inertiaMoment() );
+        = wm.self().playerType().effectiveTurn( ServerParam::i().maxMoment(),
+                                                wm.self().vel().r() );
 
     // can face only by turn
     if ( target_rel_angle.abs() < max_turn )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: can face only turn"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": Bhv_NeckBodyToPoint: can face only turn" );
         agent->doTurn( target_rel_angle );
         agent->setNeckAction( new Neck_TurnToRelative( 0.0 ) );
         return true;
@@ -489,8 +663,7 @@ Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
     }
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: turn & turn_neck"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Bhv_NeckBodyToPoin: turn & turn_neck" );
     // moment is justified automatically.
     agent->setNeckAction( new Neck_TurnToRelative( target_rel_angle ) );
     return true;
@@ -501,7 +674,7 @@ Bhv_NeckBodyToPoint::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Bhv_NeckBodyToBall::execute( PlayerAgent * agent )
 {
@@ -527,7 +700,7 @@ Bhv_NeckBodyToBall::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 View_Wide::execute( PlayerAgent * agent )
 {
@@ -542,7 +715,7 @@ View_Wide::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 View_Normal::execute( PlayerAgent * agent )
 {
@@ -557,7 +730,23 @@ View_Normal::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
+bool
+View_ChangeWidth::execute( PlayerAgent * agent )
+{
+    dlog.addText( Logger::ACTION,
+                  __FILE__": View_ChangeWidthTo %d",
+                  M_width.type() );
+
+    return agent->doChangeView( M_width );
+}
+
+/////////////////////////////////////////////////////////////////////
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
 bool
 Arm_PointToPoint::execute( PlayerAgent * agent )
 {
@@ -579,7 +768,7 @@ Arm_PointToPoint::execute( PlayerAgent * agent )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 Arm_Off::execute( PlayerAgent * agent )
 {
