@@ -38,7 +38,7 @@
 #include "neck_turn_to_relative.h"
 
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -72,7 +72,8 @@ Bhv_BeforeKickOff::execute( PlayerAgent * agent )
     }
 
 
-    if ( M_move_point.x >= 0.0 )
+    if ( ServerParam::i().kickoffOffside()
+         && M_move_point.x >= 0.0 )
     {
         M_move_point.x = -1.0;
     }
