@@ -39,7 +39,7 @@
 #include "body_go_to_point.h"
 
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/math_util.h>
 
 namespace rcsc {
@@ -53,12 +53,11 @@ Bhv_GoToPointLookBall::execute( PlayerAgent * agent )
 {
     const WorldModel & wm = agent->world();
 
-    const Vector2D my_point
-        = wm.self().playerType().inertiaFinalPoint( wm.self().pos(),
-                                                    wm.self().vel() );
+    const Vector2D my_final_pos = wm.self().inertiaFinalPoint();
 
     // already there
-    if ( my_point.dist2( M_target_point ) < square( M_dist_thr ) )
+    if ( my_final_pos.isValid()
+         && my_final_pos.dist2( M_target_point ) < square( M_dist_thr ) )
     {
         if ( wm.ball().posCount() > 5 )
         {
@@ -72,22 +71,17 @@ Bhv_GoToPointLookBall::execute( PlayerAgent * agent )
         }
     }
 
-    const Vector2D my_next
-        = wm.self().pos()
-        + wm.self().vel();
-    const Vector2D ball_next
-        = wm.ball().pos()
-        + wm.ball().vel();
-
+    const Vector2D my_next = wm.self().pos() + wm.self().vel();
+    const Vector2D ball_next = wm.ball().pos() + wm.ball().vel();
 
     // check angle difference between target and ball
     // if defference angle is out of my view cone width
     //   -> back dash is required
     const AngleDeg ball_target_angle_diff
-        = ( M_target_point - my_next ).th() - ( ball_next - my_next ).th();
+        = ( M_target_point - my_next ).th()  - ( ball_next - my_next ).th();
 
     /*--------------------------------------------------------*/
-    // nomal move
+    // normal move
     if ( ball_target_angle_diff.abs()
          < ServerParam::i().maxNeckAngle() - 5.0 )
     {
@@ -103,7 +97,6 @@ Bhv_GoToPointLookBall::execute( PlayerAgent * agent )
         return false;
     }
 
-
     /*--------------------------------------------------------*/
     // back dash
     {
