@@ -47,6 +47,8 @@ private:
 
     //! face target point
     const Vector2D M_point;
+    //! angle buffer threshold
+    const double M_angle_buf;
 
 public:
 
@@ -54,8 +56,10 @@ public:
       \brief construct with face target point
      */
     explicit
-    Bhv_NeckBodyToPoint( const Vector2D & point )
+    Bhv_NeckBodyToPoint( const Vector2D & point,
+                         const double & angle_buf = 5.0 )
         : M_point( point )
+        , M_angle_buf( angle_buf )
       { }
 
     /*!
