@@ -57,6 +57,12 @@ public:
       \return true if action is performed
      */
     bool execute( PlayerAgent * agent );
+
+private:
+
+    void findBall( PlayerAgent * agent );
+    void scanAllField( PlayerAgent * agent );
+
 };
 
 }
