@@ -36,11 +36,11 @@
 
 #include "body_advance_ball.h"
 
-#include "intention_kick.h"
+//#include "intention_kick.h"
 #include "body_clear_ball.h"
 #include "body_kick_one_step.h"
-#include "body_kick_two_step.h"
-#include "body_kick_multi_step.h"
+//#include "body_kick_two_step.h"
+#include "body_smart_kick.h"
 
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
@@ -62,8 +62,7 @@ Body_AdvanceBall::execute( PlayerAgent * agent )
     const WorldModel & wm = agent->world();
 
     dlog.addText( Logger::TEAM,
-                  "%s:%d: Body_AdvanceBall"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Body_AdvanceBall" );
 
     if ( ! wm.self().isKickable() )
     {
@@ -71,16 +70,14 @@ Body_AdvanceBall::execute( PlayerAgent * agent )
                   << " not ball kickable!"
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                      "%s:%d: Body_AdvanceBall. not kickable"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": Body_AdvanceBall. not kickable" );
         return false;
     }
 
     if ( S_last_calc_time != wm.time() )
     {
         dlog.addText( Logger::CLEAR,
-                      "%s:%d: update"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": update" );
         S_cached_best_angle = getBestAngle( agent );
         S_last_calc_time = wm.time();
     }
@@ -88,55 +85,44 @@ Body_AdvanceBall::execute( PlayerAgent * agent )
 
     const Vector2D target_point
         = wm.self().pos()
-        + Vector2D::polar2vector(30.0, S_cached_best_angle);
+        + Vector2D::polar2vector( 30.0, S_cached_best_angle );
 
     dlog.addText( Logger::CLEAR,
-                  "%s:%d: target_angle= %f"
-                  ,__FILE__, __LINE__,
+                  __FILE__": target_angle=%.1f",
                   S_cached_best_angle.degree() );
     agent->debugClient().setTarget( target_point );
     agent->debugClient().addLine( wm.ball().pos(), target_point );
 
     if ( wm.gameMode().type() != GameMode::PlayOn )
     {
-        agent->debugClient().addMessage( "Advance" );
-        // enforce one step kick
+        agent->debugClient().addMessage( "Advance1K" );
         Body_KickOneStep( target_point,
                           ServerParam::i().ballSpeedMax()
                           ).execute( agent );
+        return true;
     }
-    else
+
+    Vector2D one_step_vel
+        = Body_KickOneStep::get_max_possible_vel
+        ( ( target_point - wm.ball().pos() ).th(),
+          wm.self().kickRate(),
+          wm.ball().vel() );
+
+    if ( one_step_vel.r() > 2.1 )
     {
-        Vector2D one_step_vel
-            = Body_KickOneStep::get_max_possible_vel
-            ( ( target_point - wm.ball().pos() ).th(),
-              wm.self().kickRate(),
-              wm.ball().vel() );
-        if ( one_step_vel.r() > 2.0 )
-        {
-            Body_KickOneStep( target_point,
-                              ServerParam::i().ballSpeedMax()
-                              ).execute( agent );
-            agent->debugClient().addMessage( "Advance1K" );
-            return true;
-        }
-
-        agent->debugClient().addMessage( "Advance2K" );
-        Body_KickTwoStep( target_point,
-                          ServerParam::i().ballSpeedMax(),
-                          true // enforce
+        Body_KickOneStep( target_point,
+                          ServerParam::i().ballSpeedMax()
                           ).execute( agent );
-
-        agent->setIntention
-            ( new IntentionKick( target_point,
-                                 ServerParam::i().ballSpeedMax(),
-                                 1,
-                                 true, // enforde
-                                 wm.time() ) );
-        dlog.addText( Logger::TEAM,
-                      "%s:%d: register advance kick intention."
-                      ,__FILE__, __LINE__ );
+        agent->debugClient().addMessage( "Advance1K" );
+        return true;
     }
+
+    agent->debugClient().addMessage( "Advance" );
+
+    Body_SmartKick( target_point,
+                    ServerParam::i().ballSpeedMax(),
+                    ServerParam::i().ballSpeedMax() * 0.9,
+                    3 ).execute( agent );
     return true;
 }
 
@@ -185,15 +171,13 @@ Body_AdvanceBall::getBestAngle( const PlayerAgent* agent )
     if ( lower_angle > upper_angle )
     {
         dlog.addText( Logger::CLEAR,
-                      "%s:%d: getBestAngle. angle_error. lower=%f, uppser=%f"
-                      ,__FILE__, __LINE__,
+                      __FILE__": getBestAngle. angle_error. lower=%.1f, uppser=%.1f",
                       lower_angle, upper_angle );
         return AngleDeg( 0.0 );
     }
 
     dlog.addText( Logger::CLEAR,
-                  "%s:%d: getBestAngle. left(%.1f %.1f)lower_angle=%.1f right(%.1f %.1f)upper_angle=%.1f"
-                  ,__FILE__, __LINE__,
+                  __FILE__": getBestAngle. left(%.1f %.1f)lower_angle=%.1f right(%.1f %.1f)upper_angle=%.1f",
                   left_limit.x, left_limit.y, lower_angle,
                   right_limit.x, right_limit.y, upper_angle );
 
