@@ -35,15 +35,16 @@
 
 #include "body_clear_ball.h"
 
-#include "intention_kick.h"
+//#include "intention_kick.h"
 #include "body_kick_one_step.h"
-#include "body_kick_two_step.h"
-#include "body_kick_multi_step.h"
+//#include "body_kick_two_step.h"
+//#include "body_kick_multi_step.h"
+#include "body_smart_kick.h"
 
-#include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/logger.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/geom/line_2d.h>
 
 namespace rcsc {
@@ -146,21 +147,11 @@ Body_ClearBall::execute( PlayerAgent * agent )
             return true;
         }
 
-        agent->debugClient().addMessage( "Clear2K" );
-        Body_KickTwoStep( target_point,
-                          ServerParam::i().ballSpeedMax(),
-                          true // enforce
-                          ).execute( agent );
-
-        agent->setIntention
-            ( new IntentionKick( target_point,
-                                 ServerParam::i().ballSpeedMax(),
-                                 1,
-                                 true, // enforde
-                                 agent->world().time() ) );
-        dlog.addText( Logger::TEAM,
-                      "%s:%d: register clear kick intention"
-                      ,__FILE__, __LINE__ );
+        agent->debugClient().addMessage( "ClearS" );
+        Body_SmartKick( target_point,
+                        ServerParam::i().ballSpeedMax(),
+                        ServerParam::i().ballSpeedMax() * 0.85,
+                        2 ).execute( agent );
     }
     return true;
 }
@@ -193,7 +184,7 @@ Body_ClearBall::get_best_angle( const PlayerAgent * agent,
                            * ( AngleDeg::sin_deg(1.5 * tmp_angle.abs() + 45.0 )
                                + 1.0) );
         }
-        int dir_count = agent->world().getDirCount( tmp_angle ) - 3;
+        int dir_count = agent->world().dirCount( tmp_angle ) - 3;
         if ( dir_count < 0 ) dir_count = 0;
         tmp_score *= std::pow( 0.95, static_cast< double >( dir_count ) );
 
