@@ -36,7 +36,8 @@
 #include "body_kick_one_step.h"
 
 #include "body_stop_ball.h"
-#include "body_kick_to_relative.h"
+//#include "body_kick_to_relative.h"
+#include "body_hold_ball.h"
 
 #include <rcsc/player/player_agent.h>
 #include <rcsc/common/logger.h>
@@ -56,8 +57,7 @@ bool
 Body_KickOneStep::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::KICK,
-                  "%s:%d: Body_KickOneStep"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Body_KickOneStep" );
 
     const WorldModel & wm = agent->world();
 
@@ -67,27 +67,31 @@ Body_KickOneStep::execute( PlayerAgent * agent )
                   << " not ball kickable!"
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                      "%s:%d:  not kickable"
-                      ,__FILE__, __LINE__ );
+                      __FILE__":  not kickable" );
         return false;
     }
 
+    Vector2D ball_vel = wm.ball().vel();
+
     if ( ! wm.ball().velValid() )
     {
-        dlog.addText( Logger::KICK,
-                      "%s:%d. unknown ball vel"
-                      ,__FILE__, __LINE__ );
-        return Body_StopBall().execute( agent );
+        if ( ! M_force_mode )
+        {
+            dlog.addText( Logger::KICK,
+                          __FILE__". unknown ball vel" );
+            return Body_StopBall().execute( agent );
+        }
+
+        ball_vel.assign( 0.0, 0.0 );
     }
 
-    M_first_speed = std::min(M_first_speed, ServerParam::i().ballSpeedMax());
+    M_first_speed = std::min( M_first_speed, ServerParam::i().ballSpeedMax() );
 
-    const AngleDeg target_angle
-        = ( M_target_point - wm.ball().pos() ).th();
+    const AngleDeg target_angle = ( M_target_point - wm.ball().pos() ).th();
 
     Vector2D first_vel = get_max_possible_vel( target_angle,
-                                              wm.self().kickRate(),
-                                              wm.ball().vel() );
+                                               wm.self().kickRate(),
+                                               ball_vel );
     if ( first_vel.r() > M_first_speed )
     {
         first_vel.setLength( M_first_speed );
@@ -95,38 +99,37 @@ Body_KickOneStep::execute( PlayerAgent * agent )
     else
     {
         dlog.addText( Logger::KICK,
-                      "%s:%d: cannot get required vel. only angle adjusted"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": cannot get required vel. only angle adjusted" );
     }
 
     // first_vel.r() may be less than M_first_speed ...
 
 
-    const Vector2D real_accel = first_vel - wm.ball().vel();
+    const Vector2D kick_accel = first_vel - ball_vel;
 
-    const double kick_power = real_accel.r() / wm.self().kickRate();
-    const AngleDeg kick_dir = real_accel.th() - wm.self().body();
+    double kick_power = kick_accel.r() / wm.self().kickRate();
+    const AngleDeg kick_dir = kick_accel.th() - wm.self().body();
 
     if ( kick_power > ServerParam::i().maxPower() + 0.01 )
     {
+        std::cerr << agent->config().teamName() << ' '
+                  << wm.self().unum() << ": "
+                  << wm.time() << " Body_KickOneStep unexpected reach."
+                  << std::endl;
         dlog.addText( Logger::KICK,
-                      "%s:%d: why power over??"
-                      ,__FILE__, __LINE__ );
-        double keep_dist = wm.self().playerType().playerSize()
-            + wm.self().playerType().kickableMargin() * 0.6;
-        // keep current angle
-        AngleDeg keep_angle = wm.ball().angleFromSelf()
-            - wm.self().body();
-
-        return Body_KickToRelative( keep_dist,
-                                    keep_angle,
-                                    false
-                                    ).execute( agent );
+                      __FILE__": why power over??  power=%f", kick_power );
+        if ( ! M_force_mode )
+        {
+            return Body_HoldBall( true,
+                                  M_target_point,
+                                  M_target_point
+                                  ).execute( agent );
+        }
+        kick_power = ServerParam::i().maxPower();
     }
 
     dlog.addText( Logger::KICK,
-                  "%s:%d: first_speed= %.3f, angle= %.1f, power= %.1f, dir= %.1f "
-                  ,__FILE__, __LINE__,
+                  __FILE__": first_speed=%.3f, angle=%.1f, power=%.1f, dir=%.1f ",
                   first_vel.r(), first_vel.th().degree(),
                   kick_power, kick_dir.degree() );
 
@@ -165,8 +168,7 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
     if ( num == 0 )
     {
         dlog.addText( Logger::KICK,
-                      "%s:%d: get_max_possible_vel: angle=%.1f. No solution"
-                      ,__FILE__, __LINE__,
+                      __FILE__": get_max_possible_vel: angle=%.1f. No solution",
                       target_angle.degree() );
         return Vector2D( 0.0, 0.0 );
     }
@@ -182,9 +184,8 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
                 sol1.setLength( ServerParam::i().ballSpeedMax() );
 
                 dlog.addText( Logger::KICK,
-                              "%s:%d: get_max_possible_vel: angle=%.1f."
-                              " 1 solution  adjust."
-                              ,__FILE__, __LINE__,
+                              __FILE__": get_max_possible_vel: angle=%.1f."
+                              " 1 solution  adjust.",
                               target_angle.degree() );
             }
             else
@@ -193,9 +194,8 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
                 sol1.assign( 0.0, 0.0 );
 
                 dlog.addText( Logger::KICK,
-                              "%s:%d: get_max_possible_vel: angle=%.1f."
-                              " 1 solution. failed."
-                              ,__FILE__, __LINE__,
+                              __FILE__": get_max_possible_vel: angle=%.1f."
+                              " 1 solution. failed.",
                               target_angle.degree() );
             }
         }
@@ -219,8 +219,7 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
         std::swap( sol1, sol2 );
         std::swap( length1, length2 );
         dlog.addText( Logger::KICK,
-                      "%s:%d: get_max_possible_vel: swap"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": get_max_possible_vel: swap" );
     }
 
     if ( length1 > ServerParam::i().ballSpeedMax() )
@@ -230,9 +229,8 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
             sol1.assign( 0.0, 0.0 );
 
             dlog.addText( Logger::KICK,
-                          "%s:%d: get_max_possible_vel: angle=%.1f."
-                          " 2 solutions. but never reach"
-                          ,__FILE__, __LINE__,
+                          __FILE__": get_max_possible_vel: angle=%.1f."
+                          " 2 solutions. but never reach",
                           target_angle.degree() );
         }
         else
@@ -240,15 +238,14 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
             sol1.setLength( ServerParam::i().ballSpeedMax() );
 
             dlog.addText( Logger::KICK,
-                          "%s:%d: get_max_possible_vel: angle=%.1f."
-                          " 2 solutions. adjust to ballSpeedMax"
-                          ,__FILE__, __LINE__,
+                          __FILE__": get_max_possible_vel: angle=%.1f."
+                          " 2 solutions. adjust to ballSpeedMax",
                           target_angle.degree() );
         }
     }
 
     dlog.addText( Logger::KICK,
-                  "one kick: 2 solutions: angle=%.1f max_vel=(%.2f, %.2f)r=%.2f",
+                  __FILE__": one kick: 2 solutions: angle=%.1f max_vel=(%.2f, %.2f)r=%.2f",
                   target_angle.degree(),
                   sol1.x, sol1.y, sol1.r() );
 
