@@ -35,9 +35,11 @@
 
 #include "body_pass.h"
 
-#include "intention_kick.h"
-#include "body_kick_two_step.h"
-#include "body_kick_multi_step.h"
+//#include "intention_kick.h"
+#include "body_kick_one_step.h"
+//#include "body_kick_two_step.h"
+//#include "body_kick_multi_step.h"
+#include "body_smart_kick.h"
 #include "body_stop_ball.h"
 #include "body_hold_ball.h"
 
@@ -100,6 +102,7 @@ Body_Pass::execute( PlayerAgent * agent )
     agent->debugClient().addMessage( "pass" );
     agent->debugClient().setTarget( target_point );
 
+#if 0
     Body_KickMultiStep( target_point,
                         first_speed,
                         false
@@ -118,13 +121,41 @@ Body_Pass::execute( PlayerAgent * agent )
                                  false, // not enforce
                                  agent->world().time() ) );
     }
+#else
+    int kick_step = ( agent->world().gameMode().type() != GameMode::PlayOn
+                      && agent->world().gameMode().type() != GameMode::GoalKick_
+                      ? 1
+                      : 3 );
+    if ( ! Body_SmartKick( target_point,
+                           first_speed,
+                           first_speed * 0.96,
+                           kick_step ).execute( agent ) )
+    {
+        if ( agent->world().gameMode().type() != GameMode::PlayOn
+             && agent->world().gameMode().type() != GameMode::GoalKick_ )
+        {
+            first_speed = std::min( agent->world().self().kickRate() * ServerParam::i().maxPower(),
+                                    first_speed );
+            Body_KickOneStep( target_point,
+                              first_speed
+                              ).execute( agent );
+            dlog.addText( Logger::ACTION,
+                          __FILE__": execute() one step kick" );
+        }
+        else
+        {
+            dlog.addText( Logger::ACTION,
+                          __FILE__": execute() failed to pass kick." );
+            return false;
+        }
+    }
+#endif
 
     if ( agent->config().useCommunication()
          && receiver != Unum_Unknown )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: execute() set pass communication."
-                      ,__FILE__, __LINE__ );
+                      __FILE__": execute() set pass communication." );
         Vector2D target_buf = target_point - agent->world().self().pos();
         target_buf.setLength( 1.0 );
 
@@ -537,10 +568,10 @@ Body_Pass::create_lead_pass( const WorldModel & world,
         = 0.7 * inertia_final_distance( ServerParam::i().ballSpeedMax(),
                                               ServerParam::i().ballDecay() );
     static const
-        Rect2D shrinked_pitch( -ServerParam::i().pitchHalfLength() + 3.0,
-                                     -ServerParam::i().pitchHalfWidth() + 3.0,
-                                     ServerParam::i().pitchLength() - 6.0,
-                                     ServerParam::i().pitchWidth() - 6.0 );
+        Rect2D shrinked_pitch( Vector2D( -ServerParam::i().pitchHalfLength() + 3.0,
+                                         -ServerParam::i().pitchHalfWidth() + 3.0 ),
+                               Size2D( ServerParam::i().pitchLength() - 6.0,
+                                       ServerParam::i().pitchWidth() - 6.0 ) );
 
     //static const double receiver_dash_speed = 0.9;
     static const double receiver_dash_speed = 0.8;
@@ -788,10 +819,10 @@ Body_Pass::create_through_pass(const WorldModel & world,
 
     ////////////////////////////////////////////////////////////////
     static const
-        Rect2D shrinked_pitch( -ServerParam::i().pitchHalfLength() + 3.0,
-                                     -ServerParam::i().pitchHalfWidth() + 3.0,
-                                     ServerParam::i().pitchLength() - 6.0,
-                                     ServerParam::i().pitchWidth() - 6.0 );
+        Rect2D shrinked_pitch( Vector2D( -ServerParam::i().pitchHalfLength() + 3.0,
+                                         -ServerParam::i().pitchHalfWidth() + 3.0 ),
+                               Size2D( ServerParam::i().pitchLength() - 6.0,
+                                       ServerParam::i().pitchWidth() - 6.0 ) );
 
     static const double receiver_dash_speed = 1.0;
     //static const double receiver_dash_speed = 0.85;
@@ -1015,7 +1046,7 @@ Body_Pass::create_through_pass(const WorldModel & world,
 */
 bool
 Body_Pass::verify_direct_pass( const WorldModel & world,
-                               const PlayerObject * receiver,
+                               const PlayerObject * /*receiver*/,
                                const Vector2D & target_point,
                                const double & target_dist,
                                const AngleDeg & target_angle,
@@ -1183,13 +1214,13 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
 */
 bool
 Body_Pass::verify_through_pass( const WorldModel & world,
-                                const PlayerObject * receiver,
+                                const PlayerObject * /*receiver*/,
                                 const Vector2D & receiver_pos,
                                 const Vector2D & target_point,
                                 const double & target_dist,
                                 const AngleDeg & target_angle,
                                 const double & first_speed,
-                                const double & reach_step )
+                                const double & /*reach_step*/ )
 {
     static const double player_dash_speed = 1.0;
 
