@@ -55,21 +55,18 @@ bool
 Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: Neck_TurnToBallAndPlayer"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Neck_TurnToBallAndPlayer" );
 
     if ( ! M_target_player )
     {
         dlog.addText( Logger::ACTION,
-                      "%s: no target player"
-                      ,__FILE__ );
+                      __FILE__": no target player" );
         return Neck_TurnToBall().execute( agent );
     }
     if ( M_target_player->posCount() < M_count_thr )
     {
         dlog.addText( Logger::ACTION,
-                      "%s: player %d.is already known"
-                      ,__FILE__,
+                      __FILE__": player %d.is already known",
                       M_target_player->unum() );
         return Neck_TurnToBall().execute( agent );
     }
@@ -78,15 +75,14 @@ Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
          || M_target_player->isGhost() )
     {
         dlog.addText( Logger::ACTION,
-                      "%s: player %d is ghost"
-                      ,__FILE__,
+                      __FILE__": player %d is ghost",
                       M_target_player->unum() );
         return Neck_TurnToBall().execute( agent );
     }
 
     const Vector2D ball_next = agent->effector().queuedNextBallPos();
-    const Vector2D my_next = agent->effector().queuedNextMyPos();
-    const AngleDeg my_next_body = agent->effector().queuedNextMyBody();
+    const Vector2D my_next = agent->effector().queuedNextSelfPos();
+    const AngleDeg my_next_body = agent->effector().queuedNextSelfBody();
 
     const Vector2D player_next = M_target_player->pos() + M_target_player->vel();
 
@@ -97,8 +93,7 @@ Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
 
 
     dlog.addText( Logger::ACTION,
-                  "%s: ball_angle = %.1f  player_angle = %.1f  my_next_body = %.1f"
-                  ,__FILE__,
+                  __FILE__": ball_angle = %.1f  player_angle = %.1f  my_next_body = %.1f",
                   ball_angle.degree(), player_angle.degree(), my_next_body.degree() );
 
     bool can_face_to_ball = false;
@@ -120,8 +115,7 @@ Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
          && ! can_face_to_player )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: cannot face to ball and player %d (%.1f %.1f)"
-                      ,__FILE__, __LINE__,
+                      __FILE__": cannot face to ball and player %d (%.1f %.1f)",
                       M_target_player->unum(),
                       player_next.x, player_next.y );
 
@@ -132,8 +126,7 @@ Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
          && ! can_face_to_player )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: cannot face to player %d (%.1f %.1f)"
-                      ,__FILE__, __LINE__,
+                      __FILE__": cannot face to player %d (%.1f %.1f)",
                       M_target_player->unum(),
                       player_next.x, player_next.y );
 
@@ -144,8 +137,7 @@ Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
          && can_face_to_player )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: cannot face to ball"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": cannot face to ball" );
 
         return Neck_TurnToPlayerOrScan( M_target_player, 0 ).execute( agent );
     }
@@ -153,8 +145,7 @@ Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
     if ( ( ball_angle - player_angle ).abs() > next_view_width - 5.0 )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: cannot look both ball and player %d (%.1f %.1f)"
-                      ,__FILE__, __LINE__,
+                      __FILE__": cannot look both ball and player %d (%.1f %.1f)",
                       M_target_player->unum(),
                       player_next.x, player_next.y );
 
@@ -166,8 +157,7 @@ Neck_TurnToBallAndPlayer::execute( PlayerAgent * agent )
                               : AngleDeg::bisect( player_angle, ball_angle ) );
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: face to ball and player %d (%.1f %.1f), face_dir = %.1f "
-                  ,__FILE__, __LINE__,
+                  __FILE__": face to ball and player %d (%.1f %.1f), face_dir = %.1f ",
                   M_target_player->unum(),
                   player_next.x, player_next.y,
                   target_angle.degree() );
