@@ -53,18 +53,26 @@ bool
 Neck_TurnToBallOrScan::execute( PlayerAgent * agent )
 {
     dlog.addText( Logger::ACTION,
-                  "%s:%d: Neck_TurnToBallOrScan"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": Neck_TurnToBallOrScan" );
 
+    const WorldModel & wm = agent->world();
+
+    if ( wm.ball().posCount() <= M_count_thr )
+    {
+        dlog.addText( Logger::ACTION,
+                      __FILE__": pos_count=%d <= count_thr=%d",
+                      wm.ball().posCount(), M_count_thr );
+        return Neck_ScanField().execute( agent );
+    }
 
     const Vector2D ball_next = agent->effector().queuedNextBallPos();
-    const Vector2D my_next = agent->effector().queuedNextMyPos();
-    const AngleDeg my_next_body = agent->effector().queuedNextMyBody();
+    const Vector2D my_next = agent->effector().queuedNextSelfPos();
+    const AngleDeg my_next_body = agent->effector().queuedNextSelfBody();
     const double next_view_width = agent->effector().queuedNextViewWidth().width();
 
     bool can_face = false;
 
-    if ( ( (ball_next - my_next).th() - my_next_body ).abs()
+    if ( ( ( ball_next - my_next ).th() - my_next_body ).abs()
          < ServerParam::i().maxNeckAngle() + next_view_width * 0.5 - 2.0 )
     {
         can_face = true;
@@ -73,68 +81,62 @@ Neck_TurnToBallOrScan::execute( PlayerAgent * agent )
     if ( ! can_face )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: never face to ball"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": never face to ball" );
         return Neck_ScanField().execute( agent );
     }
 
     // ball is not seen at current
     if ( can_face
-         && agent->world().ball().posCount() != 0
-         && ( agent->effector().queuedNextMyPos().dist( agent->effector().queuedNextBallPos() )
+         && wm.ball().posCount() > 0
+         && ( agent->effector().queuedNextSelfPos().dist( agent->effector().queuedNextBallPos() )
               > 2.5 ) )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: simply face ball"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": simply face to ball" );
         return Neck_TurnToBall().execute( agent );
     }
 
     // ball is seen at current
 
     // not exist kickable player
-    if ( ! agent->world().existKickableTeammate()
-         && ! agent->world().existKickableOpponent() )
+    if ( ! wm.existKickableTeammate()
+         && ! wm.existKickableOpponent() )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: no kickable player. only scan field"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": no kickable player. only scan field" );
         return Neck_ScanField().execute( agent );
     }
 
     // exist kickable player
     // they may accerelate the ball to max speed.
 
-    if ( agent->world().ball().distFromSelf() < 20.0 )
+    if ( wm.ball().distFromSelf() < 20.0 )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: ball close. check ball."
-                      ,__FILE__, __LINE__ );
+                      __FILE__": ball is near. check ball." );
         return Neck_TurnToPoint( ball_next ).execute( agent );
     }
 
     // consider ball reachable dist after 2step
-    const double narrow_width = ServerParam::i().visibleAngle() * 0.25; // half width
+    const double narrow_half_width = ViewWidth::width( ViewWidth::NARROW ) * 0.5;
 
     double reachable_angle = std::fabs( AngleDeg::atan2_deg
                                         ( ServerParam::i().ballSpeedMax() * 2.0,
                                           ( ball_next - my_next ).r() ) );
 
     // I can see the ball after 2 cycle even if ball is kicked.
-    if ( reachable_angle < narrow_width - 5.0 )
+    if ( reachable_angle < narrow_half_width - 5.0 )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d: exist kickable. but can see a fter 2 cycles."
-                      " scan field.  ball reachable angle = %.1f"
-                      ,__FILE__, __LINE__,
+                      __FILE__": exist kickable. but can see a fter 2 cycles."
+                      " scan field.  ball reachable angle = %.1f",
                       reachable_angle );
         return Neck_ScanField().execute( agent );
     }
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: ball reachable angle is over view range %.1f"
-                  " face to ball_next=(%.2f, %.2f)"
-                  ,__FILE__, __LINE__,
+                  __FILE__": ball reachable angle is over view range %.1f"
+                  " face to ball_next=(%.2f, %.2f)",
                   reachable_angle, ball_next.x, ball_next.y );
 
     return Neck_TurnToPoint( ball_next ).execute( agent );
