@@ -35,6 +35,8 @@
 #include <rcsc/player/soccer_action.h>
 #include <rcsc/geom/vector_2d.h>
 
+#include <vector>
+
 namespace rcsc {
 
 /*!
@@ -44,12 +46,25 @@ namespace rcsc {
 class Neck_TurnToPoint
     : public NeckAction {
 private:
-    //! facing target point
-    const Vector2D M_point;
+    //! facing target point container
+    const std::vector< Vector2D > M_points;
 public:
+    /*!
+      \brief construct with one point
+      \param point face target point
+     */
     explicit
     Neck_TurnToPoint( const Vector2D & point )
-        : M_point( point )
+        : M_points( 1, point )
+      { }
+
+    /*!
+      \brief construct with points container
+      \param points container to the face target point
+     */
+    explicit
+    Neck_TurnToPoint( const std::vector< Vector2D > & points )
+        : M_points( points )
       { }
 
     /*!
@@ -65,7 +80,7 @@ public:
      */
     NeckAction * clone() const
       {
-          return new Neck_TurnToPoint( M_point );
+          return new Neck_TurnToPoint( M_points );
       }
 };
 
