@@ -48,6 +48,10 @@ private:
     const AngleDeg M_angle_rel_to_body;
 
 public:
+    /*!
+      \brief constructor
+      \param rel_angle relative angle from the body
+     */
     explicit
     Neck_TurnToRelative( const AngleDeg & rel_angle )
         : M_angle_rel_to_body( rel_angle )
