@@ -36,6 +36,7 @@
 
 #include <string>
 #include <vector>
+#include <utility>
 #include <iostream>
 
 namespace rcsc {
@@ -60,6 +61,7 @@ public:
         EYE,
 
         CHANGE_PLAYER_TYPE,
+        CHANGE_PLAYER_TYPES,
         SAY,
 
         TEAM_GRAPHIC,
@@ -123,7 +125,7 @@ public:
 //////////////////////////////////////////////////////////////////////
 /*!
   \class CoachInitCommand
-  \brief coach's connection command
+  \brief initial connection command for coach
 
   <pre>
   Format:
@@ -193,7 +195,7 @@ private:
 
 public:
     /*!
-      \breif constructor. nothing to do
+      \brief constructor. nothing to do
      */
     CoachByeCommand()
       { }
@@ -244,7 +246,7 @@ private:
 
 public:
     /*!
-      \breif constructor. nothing to do
+      \brief constructor. nothing to do
      */
     CoachCheckBallCommand()
       { }
@@ -293,7 +295,7 @@ private:
 
 public:
     /*!
-      \breif constructor. nothing to do
+      \brief constructor. nothing to do
      */
     CoachLookCommand()
       { }
@@ -342,7 +344,7 @@ private:
 
 public:
     /*!
-      \breif constructor. nothing to do
+      \brief constructor. nothing to do
      */
     CoachTeamNamesCommand()
       { }
@@ -391,7 +393,7 @@ private:
     bool M_on; //!< if true, coach's eye is enabled
 public:
     /*!
-      \breif construct with eye mode switch
+      \brief construct with eye mode switch
       \param on if true, coach's eye is enabled.
      */
     explicit
@@ -448,7 +450,7 @@ class CoachChangePlayerTypeCommand
     : public CoachCommand {
 private:
     int M_unum; //!< target player's uniform number
-    HeteroID M_type; //!< changed player type id
+    int M_type; //!< changed player type id
 public:
     /*!
       \brief construct with target number and type id
@@ -456,7 +458,7 @@ public:
       \param type player type id
      */
     CoachChangePlayerTypeCommand( const int unum,
-                                  const HeteroID type )
+                                  const int type )
         : M_unum( unum )
         , M_type( type )
       { }
@@ -487,10 +489,85 @@ public:
       }
 };
 
+
+//////////////////////////////////////////////////////////////////////
+/*!
+  \class CoachChangePlayerTypesCommand
+  \brief command to change player type
+
+  <pre>
+  Format:
+  <- (change_player_types [(<unum> <typeid>)]+)
+  Response:
+  -> (ok change_player_type <unum> <typeid>)
+  -> (warning cannot_sub_while_playon)
+  -> (warning no_team_found)
+  -> (warning no_subs_left)
+  -> (error out_of_range_player_type)
+  -> (warning no_such_player)
+  -> (warning cannot_change_goalie)
+  -> (warning max_of_that_type_on_field)
+  </pre>
+*/
+class CoachChangePlayerTypesCommand
+    : public CoachCommand {
+private:
+    //! the set of player number & type id
+    std::vector< std::pair< int, int > > M_types;
+
+public:
+    /*!
+      \brief construct with one target player and type id
+      \param unum target player's uniform number
+      \param type player type id
+     */
+    CoachChangePlayerTypesCommand( const int unum,
+                                   const int type );
+
+    /*!
+      \brief construct with one target player and type id
+      \param types container of the pair of unum and player type id
+     */
+    CoachChangePlayerTypesCommand( const std::vector< std::pair< int, int > >  & types );
+
+    /*!
+      \brief add new change_player_type pair
+      \param unum target player's uniform number
+      \param type player type id
+     */
+    void add( const int unum,
+              const int type );
+
+    /*!
+      \brief get command type
+      \return command type Id
+     */
+    Type type() const
+      {
+          return CHANGE_PLAYER_TYPES;
+      }
+
+    /*!
+      \brief put command string to ostream
+      \param to reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & toStr( std::ostream & to ) const;
+
+    /*!
+      \brief get command name
+      \return command name string
+     */
+    std::string name() const
+      {
+          return std::string( "change_player_types" );
+      }
+};
+
 //////////////////////////////////////////////////////////////////////
 /*!
   \class CoachSayCommand
-  \brief coach's say command for CLang
+  \brief say command for CLang
 
   <pre>
   Format:
@@ -509,7 +586,7 @@ private:
     const std::string & M_clang_msg;
 public:
     /*!
-      \breif construct with message string.
+      \brief construct with message string.
       \param clang_msg reference to the clang message instance
      */
     explicit
@@ -568,7 +645,7 @@ private:
 
 public:
     /*!
-      \breif construct with xpm string and its coordinate.
+      \brief construct with xpm string and its coordinate.
       \param x x coordinate of this tile in xpm image
       \param y y coordinate of this tile in xpm image
       \param xpm_lines a tile of xpm
@@ -623,7 +700,7 @@ private:
     int M_level; //!< gzip compression level
 public:
     /*!
-      \breif construct with compression level
+      \brief construct with compression level
      */
     explicit
     CoachCompressionCommand( const int level )
@@ -673,7 +750,7 @@ private:
 
 public:
     /*!
-      \breif constructor. nothing to do
+      \brief constructor. nothing to do
      */
     CoachDoneCommand()
       { }
