@@ -67,7 +67,7 @@ CoachConfig::setDefaultParam()
 {
     // basic setting
     M_team_name = "HELIOS_base";
-    M_version = 9.4;
+    M_version = 13;
 
     M_coach_name = "Coach_base";
     M_use_coach_name = false;
@@ -91,6 +91,7 @@ CoachConfig::setDefaultParam()
     M_use_hetero = true;
 
     M_use_team_graphic = false;
+    M_team_graphic_file = "";
 
     M_max_team_graphic_per_cycle = 32;
 
@@ -134,7 +135,7 @@ CoachConfig::createParamMap( ParamMap & param_map )
         ( "server_wait_seconds", "", &M_server_wait_seconds )
 
         ( "host", "h",  &M_rcssserver_host )
-        ( "port", "", &M_rcssserver_port )
+        ( "port", "p", &M_rcssserver_port )
 
         ( "compression", "", &M_compression )
 
@@ -149,6 +150,7 @@ CoachConfig::createParamMap( ParamMap & param_map )
         ( "use_hetero", "", &M_use_hetero )
 
         ( "use_team_graphic", "", &M_use_team_graphic )
+        ( "team_graphic_file", "", &M_team_graphic_file )
         ( "max_team_graphic_per_cycle", "", &M_max_team_graphic_per_cycle )
 
         ( "log_dir", "", &M_log_dir )
