@@ -38,6 +38,7 @@
 #include <string>
 #include <cstring>
 #include <cstdio>
+#include <cstdlib>
 
 namespace rcsc {
 
@@ -101,10 +102,11 @@ GlobalVisualSensor::parseV7( const char * msg )
     /*
     sample version >= 7.0
     (see_global TIME ((g l) -52.5 0) ((g r) 52.5 0) ((b) <x> <y> <vx> <vy>)
-       ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>[ <arm>][ t])
+       ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>[ <arm>][ {t|k}])
        ....)
         <-- arm is global
         <-- 't' means tackle
+        <-- 'k' means kick
     (ok look TIME ((g l) -52.5 0) ((g r) 52.5 0) ((b) <x> <y> <vx> <vy>)
        ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>) <-- no arm & tackle
        ....)
@@ -179,7 +181,7 @@ GlobalVisualSensor::parseV7( const char * msg )
 
         if ( std::sscanf( msg,
                           " ((p %31[^ ] %d %n ",
-                          teamname, &unum ,
+                          teamname, &unum,
                           &n_read ) != 2 )
         {
             std::cerr << __FILE__ << ' ' << __LINE__
@@ -224,9 +226,21 @@ GlobalVisualSensor::parseV7( const char * msg )
         else
         {
             player.setTeam( side, unum, false );
+#if 0
+            if ( std::isdigit( *msg ) )
+            {
+                long type = std::strtol( msg, NULL, 10 );
+                if ( type != LONG_MAX
+                     && type != LONG_MIN )
+                {
+                    player.setPlayerType( static_cast< int >( type ) );
+                }
+            }
+#endif
         }
 
-        while ( *msg != ')' ) ++msg; // skip to last of the player name
+        // skip to last of the player name
+        while ( *msg != '\0' && *msg != ')' ) ++msg;
         ++msg; // skip ')'
 
         if ( std::sscanf( msg,
@@ -250,7 +264,11 @@ GlobalVisualSensor::parseV7( const char * msg )
             while ( *msg != '\0' && *msg == ' ' ) ++msg;
             if ( *msg == '\0' ) break;
 
-            if ( *msg == 't' )
+            if ( *msg == 'k' )
+            {
+                player.setKick( true );
+            }
+            else if ( *msg == 't' )
             {
                 player.setTackle();
             }
