@@ -38,11 +38,14 @@
 #include <rcsc/game_time.h>
 #include <rcsc/types.h>
 
+#include <boost/shared_ptr.hpp>
+
 #include <iostream>
 #include <string>
 
 namespace rcsc {
 
+class AudioMemory;
 class GlobalVisualSensor;
 
 /*!
@@ -51,12 +54,21 @@ class GlobalVisualSensor;
  */
 class GlobalWorldModel {
 private:
+
+    //!< heard info memory
+    boost::shared_ptr< AudioMemory > M_audio_memory;
+
+    //
+    // game info
+    //
+
     //! last updated time
     GameTime M_time;
     //! last see_global received time
     GameTime M_see_time;
 
-    SideID M_our_side; //!< if trainer, set NEUTRAL
+    //! if trainer, this becomes NEUTRAL
+    SideID M_our_side;
 
     std::string M_team_name_left; //!< left team name string
     std::string M_team_name_right; //!< right teamname string
@@ -64,8 +76,9 @@ private:
     //! last updated playmode
     GameMode M_game_mode;
 
-
-    // Objects
+    //
+    // objects
+    //
 
     //! seen ball info
     GlobalBallObject M_ball;
@@ -79,10 +92,11 @@ private:
     //! the container of player's pointer for the right team
     std::vector< const GlobalPlayerObject * > M_players_right;
 
-
+    //
     // player type management
+    //
 
-    //! type analyzer
+    //! player type analyzer instance
     PlayerTypeAnalyzer M_player_type_analyzer;
 
     //! number of player change
@@ -90,13 +104,19 @@ private:
     //! number of player change
     int M_change_count_right;
 
-    //! teammate player type Id
-    HeteroID M_player_types_left[11];
-    //! opponent player type Id
-    HeteroID M_player_types_right[11];
+    //! left team player type Id
+    int M_player_types_left[11];
+    //! right team player type Id
+    int M_player_types_right[11];
 
+    //! counter of each used player type for left team
+    std::vector< int > M_player_type_used_count_left;
+    //! counter of each used player type for right team
+    std::vector< int > M_player_type_used_count_right;
 
-    // Coach Language management
+    //
+    // coach language management
+    //
 
     //! the last playon start time
     long M_last_playon_start;
@@ -124,6 +144,23 @@ public:
      */
     void init( const SideID side );
 
+    /*!
+      \brief set new audio memory
+      \param memory pointer to the memory instance. This must be
+      a dynamically allocated object.
+     */
+    void setAudioMemory( boost::shared_ptr< AudioMemory > memory );
+
+    /*!
+      \brief get audio memory
+      \return co
+     */
+    const
+    AudioMemory & audioMemory() const
+      {
+          return *M_audio_memory;
+      }
+
     /*!
       \brief init max freeform message count.
 
@@ -140,11 +177,15 @@ public:
                       const std::string & name );
 
     /*!
-      \brief update teammate player type Id
+      \brief update player type info.
+      this method is called when change_player_type message is received.
+      \param side player's side
+      \param unum player's uniform number
+      \param type new player type id
      */
-    void recvChangePlayerType( const SideID side,
-                               const int unum,
-                               const HeteroID type );
+    void setPlayerType( const SideID side,
+                        const int unum,
+                        const int type );
 
     /*!
       \brief update playmode using heard referee info
@@ -215,7 +256,7 @@ public:
     const
     std::string & ourTeamName() const
       {
-          return ( ourSide() == rcsc::LEFT
+          return ( ourSide() == LEFT
                    ? M_team_name_left
                    : M_team_name_right );
       }
@@ -227,7 +268,7 @@ public:
     const
     std::string & theirTeamName() const
       {
-          return ( ourSide() == rcsc::LEFT
+          return ( ourSide() == LEFT
                    ? M_team_name_right
                    : M_team_name_left );
       }
@@ -262,15 +303,6 @@ public:
           return M_game_mode;
       }
 
-    /*!
-      \brief get teammate player type Id
-      \param side side ID of the target player
-      \param unum uniform number of the target player
-      \return player type Id
-     */
-    HeteroID getHeteroID( const SideID side,
-                          const int unum ) const;
-
     /*!
       \brief get the latest ball data
       \return const reference to the ball data
@@ -288,7 +320,8 @@ public:
     BallStatus getBallStatus() const;
 
     /*!
-      \brief get all players
+      \brief
+      get all players
       \return const reference to the data container
      */
     const
@@ -341,6 +374,45 @@ public:
                    : playersLeft() );
       }
 
+    //
+    // player type info
+    //
+
+    /*!
+      \brief get the count of change_player_type message after kickoff
+      \return the count of change_player_type message after kickoff
+     */
+    int playerTypeChangeCount( const SideID side ) const
+      {
+          return ( side == LEFT
+                   ? M_change_count_left
+                   : M_change_count_right );
+      }
+
+    /*!
+      \brief get teammate player type Id
+      \param side side ID of the target player
+      \param unum uniform number of the target player
+      \return player type id
+     */
+    int heteroID( const SideID side,
+                  const int unum ) const;
+
+    /*!
+      \brief get the counter of used player type for left team
+      \return container of counts for each player type
+     */
+    const
+    std::vector< int > & playerTypeUsedCount( const SideID side ) const
+      {
+          return ( side == LEFT
+                   ? M_player_type_used_count_left
+                   : M_player_type_used_count_right );
+      }
+
+    //
+    // coach language info
+    //
 
     /*!
       \brief get the last playon start cycle
