@@ -38,6 +38,7 @@
 #include "global_world_model.h"
 #include "global_object.h"
 
+#include <rcsc/common/player_param.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/common/player_type.h>
 #include <rcsc/game_mode.h>
@@ -49,16 +50,19 @@ namespace rcsc {
 
 */
 PlayerTypeAnalyzer::Data::Data()
+    : turned_( false )
+    , kicked_( false )
+    , tackling_( false )
+    , maybe_referee_( false )
+    , maybe_collide_( false )
+    , maybe_kick_( false )
+    , pos_( Vector2D::INVALIDATED )
+    , vel_( 0.0, 0.0 )
+    , body_( -360 )
+    , invalid_flags_( PlayerParam::i().playerTypes(), 0 )
+    , type_( Hetero_Default )
 {
-    turned_ = false;
-    maybe_referee_ = false;
-    maybe_collide_ = false;
-    maybe_kick_ = false;
-    body_ = -360.0;
-    pos_.invalidate();
-    vel_.assign( 0.0, 0.0 );
 
-    setDefaultType();
 }
 
 /*-------------------------------------------------------------------*/
@@ -68,10 +72,7 @@ PlayerTypeAnalyzer::Data::Data()
 void
 PlayerTypeAnalyzer::Data::setDefaultType()
 {
-    for ( int i = 0; i < Hetero_MAX; ++i )
-    {
-        invalid_flags_[i] = false;
-    }
+    invalid_flags_.assign( PlayerParam::i().playerTypes(), 0 );
 
     type_ = Hetero_Default;
 }
@@ -83,10 +84,7 @@ PlayerTypeAnalyzer::Data::setDefaultType()
 void
 PlayerTypeAnalyzer::Data::setUnknownType()
 {
-    for ( int i = 0; i < Hetero_MAX; ++i )
-    {
-        invalid_flags_[i] = false;
-    }
+    invalid_flags_.assign( PlayerParam::i().playerTypes(), 0 );
 
     type_ = Hetero_Unknown;
 }
@@ -115,6 +113,20 @@ PlayerTypeAnalyzer::update()
         return;
     }
 
+    const std::size_t max_types = static_cast< std::size_t >( PlayerParam::i().playerTypes() );
+    for ( int i = 0; i < 11; ++i )
+    {
+        if ( M_teammate_data[i].invalid_flags_.size() != max_types )
+        {
+            M_teammate_data[i].invalid_flags_.resize( max_types, 0 );
+        }
+
+        if ( M_opponent_data[i].invalid_flags_.size() != max_types )
+        {
+            M_opponent_data[i].invalid_flags_.resize( max_types, 0 );
+        }
+    }
+
     if ( M_updated_time.cycle() != M_world.time().cycle() - 1
          && M_updated_time.stopped() != M_world.time().stopped() - 1 )
     {
@@ -209,10 +221,8 @@ PlayerTypeAnalyzer::updateLastData()
             : M_world.playersLeft() );
 
 
-    const std::vector< const GlobalPlayerObject * >::const_iterator tend
-            = teammates.end();
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = teammates.begin();
+    const std::vector< const GlobalPlayerObject * >::const_iterator tend = teammates.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = teammates.begin();
           p != tend;
           ++p )
     {
@@ -226,10 +236,8 @@ PlayerTypeAnalyzer::updateLastData()
     }
 
 
-    const std::vector< const GlobalPlayerObject * >::const_iterator oend
-            = opponents.end();
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = opponents.begin();
+    const std::vector< const GlobalPlayerObject * >::const_iterator oend = opponents.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = opponents.begin();
           p != oend;
           ++p )
     {
@@ -258,6 +266,8 @@ PlayerTypeAnalyzer::analyze()
     checkPlayerDecay();
     //checkPlayerSpeedMax();
 
+    const int max_types = PlayerParam::i().playerTypes();
+
     const std::vector< const GlobalPlayerObject * > & players
         = ( M_world.ourSide() == LEFT
             ? M_world.playersRight()
@@ -293,15 +303,15 @@ PlayerTypeAnalyzer::analyze()
         if ( data.maybe_referee_ ) continue;
 
         int invalid_count = 0;
-        for ( int t = 0; t < Hetero_MAX; ++t )
+        for ( int t = 0; t < max_types; ++t )
         {
-            if ( data.invalid_flags_[t] )
+            if ( data.invalid_flags_[t] != 0 )
             {
                 ++invalid_count;
             }
         }
 
-        if ( invalid_count == static_cast< int >( Hetero_MAX ) )
+        if ( invalid_count == max_types )
         {
             // no candidate
             std::cout <<  M_world.time()
@@ -311,19 +321,19 @@ PlayerTypeAnalyzer::analyze()
                       << std::endl;
             data.setUnknownType();
         }
-        else if ( invalid_count == static_cast< int >( Hetero_MAX ) - 1 )
+        else if ( invalid_count == max_types - 1 )
         {
-            // success!
-            for ( int t = 0; t < Hetero_MAX; ++t )
+            // success! only 1 candidate.
+            for ( int t = 0; t < max_types; ++t )
             {
-                if ( ! data.invalid_flags_[t] )
+                if ( data.invalid_flags_[t] == 0 )
                 {
                     std::cout << M_world.time()
                               << ' ' << M_world.ourTeamName()
                               << " Coach: determined opponent " << (*p)->unum()
                               << " type = " << t
                               << std::endl;
-                    data.type_ = static_cast< HeteroID >( t );
+                    data.type_ = t;
                     break;
                 }
             }
@@ -358,11 +368,9 @@ PlayerTypeAnalyzer::checkTurn()
             : M_world.playersLeft() );
 
 
-    const std::vector< const GlobalPlayerObject * >::const_iterator tend
-            = teammates.end();
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = teammates.begin();
-          p != tend;
+    const std::vector< const GlobalPlayerObject * >::const_iterator t_end = teammates.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = teammates.begin();
+          p != t_end;
           ++p )
     {
         if ( (*p)->unum() < 1 || 11 < (*p)->unum() ) continue;
@@ -377,11 +385,9 @@ PlayerTypeAnalyzer::checkTurn()
     }
 
 
-    const std::vector< const GlobalPlayerObject * >::const_iterator oend
-            = opponents.end();
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = opponents.begin();
-          p != oend;
+    const std::vector< const GlobalPlayerObject * >::const_iterator o_end = opponents.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = opponents.begin();
+          p != o_end;
           ++p )
     {
         if ( (*p)->unum() < 1 || 11 < (*p)->unum() ) continue;
@@ -485,18 +491,15 @@ PlayerTypeAnalyzer::checkCollisions()
                     2.0 );
 
 
-    const std::list< GlobalPlayerObject >::const_iterator all_end
-        = M_world.players().end();
+    const std::list< GlobalPlayerObject >::const_iterator all_end = M_world.players().end();
 
     const std::vector< const GlobalPlayerObject * > & players
         = ( M_world.ourSide() == LEFT
             ? M_world.playersRight()
             : M_world.playersLeft() );
-    const std::vector< const GlobalPlayerObject * >::const_iterator end
-        = players.end();
 
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = players.begin();
+    const std::vector< const GlobalPlayerObject * >::const_iterator end = players.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = players.begin();
           p != end;
           ++p )
     {
@@ -512,8 +515,7 @@ PlayerTypeAnalyzer::checkCollisions()
     }
 
     // check other opponent players
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = players.begin();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = players.begin();
           p != end;
           ++p )
     {
@@ -544,8 +546,7 @@ PlayerTypeAnalyzer::checkCollisions()
         = ( M_world.ourSide() == LEFT
             ? M_world.playersLeft()
             : M_world.playersRight() );
-    const std::vector< const GlobalPlayerObject * >::const_iterator t_end
-        = teammates.end();
+    const std::vector< const GlobalPlayerObject * >::const_iterator t_end = teammates.end();
 
     for ( std::vector< const GlobalPlayerObject * >::const_iterator p
               = players.begin();
@@ -572,8 +573,7 @@ PlayerTypeAnalyzer::checkCollisions()
     }
 
     // check goal post
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = players.begin();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = players.begin();
           p != end;
           ++p )
     {
@@ -601,12 +601,13 @@ PlayerTypeAnalyzer::checkKick()
 {
     static double S_max_kickable_area2 = -1.0;
 
+    const int max_types = PlayerParam::i().playerTypes();
+
     if ( S_max_kickable_area2 < 0.0 )
     {
-        for ( int t = 0; t < Hetero_MAX; ++t )
+        for ( int t = 0; t < max_types; ++t )
         {
-            const PlayerType * player_type
-                = PlayerTypeSet::i().get( static_cast< HeteroID >( t ) );
+            const PlayerType * player_type = PlayerTypeSet::i().get( t );
             if ( ! player_type ) continue;
 
             double k2 = std::pow( player_type->kickableArea(), 2.0 );
@@ -619,6 +620,8 @@ PlayerTypeAnalyzer::checkKick()
 
     for ( int i = 0; i < 11; ++i )
     {
+        M_opponent_data[i].kicked_ = false;
+        M_teammate_data[i].kicked_ = false;
         M_opponent_data[i].maybe_kick_ = false;
         M_teammate_data[i].maybe_kick_ = false;
     }
@@ -697,18 +700,17 @@ PlayerTypeAnalyzer::checkKick()
         {
             const double ball_dist = M_prev_ball.pos().dist( data.pos_ );
 
-            for ( int t = 0; t < Hetero_MAX; ++t )
+            for ( int t = 0; t < max_types; ++t )
             {
-                if (  data.invalid_flags_[t] ) continue;
+                if ( data.invalid_flags_[t] != 0 ) continue;
 
-                const PlayerType * player_type
-                    = PlayerTypeSet::i().get( static_cast< HeteroID >( t ) );
+                const PlayerType * player_type = PlayerTypeSet::i().get( t );
                 if ( ! player_type ) continue;
 
 
                 if ( ball_dist > player_type->kickableArea() + 0.001 )
                 {
-                    data.invalid_flags_[t] = true;
+                    data.invalid_flags_[t] = 1;
 #ifdef DEBUG
                     std::cout << M_world.time()
                               << ' ' << M_world.ourTeamName()
@@ -734,15 +736,15 @@ PlayerTypeAnalyzer::checkKick()
 void
 PlayerTypeAnalyzer::checkPlayerDecay()
 {
+    const int max_types = PlayerParam::i().playerTypes();
+
     const std::vector< const GlobalPlayerObject * > & players
         = ( M_world.ourSide() == LEFT
             ? M_world.playersRight()
             : M_world.playersLeft() );
 
-    const std::vector< const GlobalPlayerObject * >::const_iterator end
-        = players.end();
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = players.begin();
+    const std::vector< const GlobalPlayerObject * >::const_iterator end = players.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = players.begin();
           p != end;
           ++p )
     {
@@ -751,19 +753,18 @@ PlayerTypeAnalyzer::checkPlayerDecay()
         Data & data = M_opponent_data[(*p)->unum() - 1];
 
         if ( data.maybe_collide_ || data.maybe_kick_ ) continue;
-        if ( ! data.turned_ ) continue;
+        if ( ! data.turned_ && ! data.kicked_ && ! data.tackling_ ) continue;
         if ( ! data.pos_.valid() ) continue;
         if ( data.pos_.dist2( (*p)->pos() ) < 0.0001 ) continue;
 
         double rand_max = data.vel_.r() * ServerParam::i().playerRand();
         if ( rand_max < 0.00001 ) continue;
 
-        for ( int t = 0; t < Hetero_MAX; ++t )
+        for ( int t = 0; t < max_types; ++t )
         {
-            if ( data.invalid_flags_[t] ) continue;
+            if ( data.invalid_flags_[t] != 0 ) continue;
 
-            const PlayerType * player_type
-                = PlayerTypeSet::i().get( static_cast< HeteroID >( t ) );
+            const PlayerType * player_type = PlayerTypeSet::i().get( t );
             if ( ! player_type ) continue;
 
             double rand_x
@@ -779,7 +780,7 @@ PlayerTypeAnalyzer::checkPlayerDecay()
             if ( rand_x > rand_max + 0.0000001
                  || rand_y > rand_max + 0.0000001 )
             {
-                data.invalid_flags_[t] = true;;
+                data.invalid_flags_[t] = 1;
 #ifdef DEBUG
                 std::cout << M_world.time()
                           << ' ' << M_world.ourTeamName()
@@ -800,15 +801,15 @@ PlayerTypeAnalyzer::checkPlayerDecay()
 void
 PlayerTypeAnalyzer::checkPlayerSpeedMax()
 {
+    const int max_types = PlayerParam::i().playerTypes();
+
     const std::vector< const GlobalPlayerObject * > & players
         = ( M_world.ourSide() == LEFT
             ? M_world.playersRight()
             : M_world.playersLeft() );
 
-    const std::vector< const GlobalPlayerObject * >::const_iterator end
-        = players.end();
-    for ( std::vector< const GlobalPlayerObject * >::const_iterator p
-              = players.begin();
+    const std::vector< const GlobalPlayerObject * >::const_iterator end = players.end();
+    for ( std::vector< const GlobalPlayerObject * >::const_iterator p = players.begin();
           p != end;
           ++p )
     {
@@ -822,12 +823,11 @@ PlayerTypeAnalyzer::checkPlayerSpeedMax()
         double x_move = std::fabs( (*p)->pos().x - data.pos_.x );
         double y_move = std::fabs( (*p)->pos().y - data.pos_.y );
 
-        for ( int t = 0; t < Hetero_MAX; ++t )
+        for ( int t = 0; t < max_types; ++t )
         {
-            if ( data.invalid_flags_[t] ) continue;
+            if ( data.invalid_flags_[t] != 0 ) continue;
 
-            const PlayerType * player_type
-                = PlayerTypeSet::i().get( static_cast< HeteroID >( t ) );
+            const PlayerType * player_type = PlayerTypeSet::i().get( t );
             if ( ! player_type ) continue;
 
             double rand_max
@@ -839,7 +839,7 @@ PlayerTypeAnalyzer::checkPlayerSpeedMax()
 
             if ( actual_speed > player_type->playerSpeedMax() + 0.0001 )
             {
-                data.invalid_flags_[t] = true;;
+                data.invalid_flags_[t] = 1;
 #ifdef DEBUG
                 std::cout << M_world.time()
                           << ' ' << M_world.ourTeamName()
