@@ -51,7 +51,9 @@ class PlayerTypeAnalyzer {
 private:
 
     struct Data {
-        bool turned_; //!< player did turn
+        bool turned_; //!< player performed turn
+        bool kicked_; //!< player perfomed kick
+        bool tackling_; //!< player perfoming tackle
         bool maybe_referee_; //!< player may be moved by referee
         bool maybe_collide_; //!< player may be collided with others
         bool maybe_kick_;  //!< player may kick the ball
@@ -87,7 +89,7 @@ private:
 public:
     /*!
       \brief default constructor
-      \param const reference to the world model instance
+      \param world const reference to the world model instance
      */
     explicit
     PlayerTypeAnalyzer( const GlobalWorldModel & world );
@@ -103,8 +105,7 @@ public:
       }
 
     /*!
-      \brief analyze world model
-      \param world const reference to the world model instance
+      \brief analyze world model.
      */
     void update();
 
