@@ -161,6 +161,7 @@ public:
         int unum_; //!< heard player unum
         Vector2D pos_; //!< heard player position
         AngleDeg body_; //!< heard player's body angle
+        double stamina_; //!< heard player's stamina info. negative value means stamina is unknown
 
         /*!
           \brief initialize all member
@@ -168,11 +169,13 @@ public:
         Player( const int sender,
                 const int unum,
                 const Vector2D & pos,
-                const AngleDeg & body )
+                const AngleDeg & body,
+                const double & stamina = -1.0 )
             : sender_( sender )
             , unum_( unum )
             , pos_( pos )
             , body_( body )
+            , stamina_( stamina )
           { }
     };
 
@@ -322,46 +325,46 @@ public:
 
 protected:
 
-    std::vector< Ball > M_ball;
+    std::vector< Ball > M_ball; //!< heard info
     GameTime M_ball_time; //!< info heard time
 
-    std::vector< Pass > M_pass;
+    std::vector< Pass > M_pass; //!< heard info
     GameTime M_pass_time; //!< info heard time
 
-    std::vector< OurIntercept > M_our_intercept;
+    std::vector< OurIntercept > M_our_intercept; //!< heard info
     GameTime M_our_intercept_time; //!< info heard time
 
-    std::vector< OppIntercept > M_opp_intercept;
+    std::vector< OppIntercept > M_opp_intercept; //!< heard info
     GameTime M_opp_intercept_time; //!< info heard time
 
-    std::vector< Goalie > M_goalie;
+    std::vector< Goalie > M_goalie; //!< heard info
     GameTime M_goalie_time; //!< info heard time
 
-    std::vector< Player > M_player;
+    std::vector< Player > M_player; //!< heard info
     GameTime M_player_time; //!< info heard time
 
-    std::vector< OffsideLine > M_offside_line;
+    std::vector< OffsideLine > M_offside_line; //!< heard info
     GameTime M_offside_line_time; //!< info heard time
 
-    std::vector< DefenseLine > M_defense_line;
+    std::vector< DefenseLine > M_defense_line; //!< heard info
     GameTime M_defense_line_time; //!< info heard time
 
-    std::vector< WaitRequest > M_wait_request;
+    std::vector< WaitRequest > M_wait_request; //!< heard info
     GameTime M_wait_request_time; //!< info heard time
 
-    std::vector< PassRequest > M_pass_request;
+    std::vector< PassRequest > M_pass_request; //!< heard info
     GameTime M_pass_request_time; //!< info heard time
 
-    std::vector< Stamina > M_stamina;
+    std::vector< Stamina > M_stamina; //!< heard info
     GameTime M_stamina_time; //!< info heard time
 
-    std::vector< Recovery > M_recovery;
+    std::vector< Recovery > M_recovery; //!< heard info
     GameTime M_recovery_time; //!< info heard time
 
-    std::vector< Dribble > M_dribble;
+    std::vector< Dribble > M_dribble; //!< heard info
     GameTime M_dribble_time; //!< info heard time
 
-    std::vector< FreeMessage > M_free_message;
+    std::vector< FreeMessage > M_free_message; //!< heard info
     GameTime M_free_message_time; //!< info heard time
 
 private:
@@ -700,6 +703,7 @@ public:
       \param sender sender's uniform number
       \param interceptor interceptor's number[1,22]. 11+ means opponent
       \param cycle estimated intercept cycle
+      \param current current game time
     */
     virtual
     void setIntercept( const int sender,
@@ -726,6 +730,7 @@ public:
       \param unum heard player unum
       \param pos heard player pos
       \param body heard player body angle
+      \param stamina heard player's stamina. negative value means stamina is unknown.
       \param current current game time
     */
     virtual
@@ -733,6 +738,7 @@ public:
                     const int unum,
                     const Vector2D & pos,
                     const AngleDeg & body,
+                    const double & stamina,
                     const GameTime & current );
 
     /*!
