@@ -121,11 +121,19 @@ Logger::setLogFlag( const GameTime * time,
 
 */
 void
-Logger::print( const char * msg )
+Logger::printDebugClientMessage( const long & cycle,
+                                 const std::string & msg )
 {
     if ( M_fout )
     {
-        fputs( msg, M_fout );
+        char buf[32];
+        snprintf( buf, 32, "%%%% step %ld\n", cycle );
+        fputs( buf, M_fout );
+        fputs( "%% debug [", M_fout );
+        fputs( msg.c_str(), M_fout );
+        fputs( "]\n", M_fout );
+
+        // fputs( msg, M_fout );
     }
 }
 
