@@ -52,40 +52,40 @@ class GameTime;
 class Logger {
 public:
 
-    static const boost::int32_t LEVEL_00 = 0x00000000;
-    static const boost::int32_t LEVEL_01 = 0x00000001;
-    static const boost::int32_t LEVEL_02 = 0x00000002;
-    static const boost::int32_t LEVEL_03 = 0x00000004;
-    static const boost::int32_t LEVEL_04 = 0x00000008;
-    static const boost::int32_t LEVEL_05 = 0x00000010;
-    static const boost::int32_t LEVEL_06 = 0x00000020;
-    static const boost::int32_t LEVEL_07 = 0x00000040;
-    static const boost::int32_t LEVEL_08 = 0x00000080;
-    static const boost::int32_t LEVEL_09 = 0x00000100;
-    static const boost::int32_t LEVEL_10 = 0x00000200;
-    static const boost::int32_t LEVEL_11 = 0x00000400;
-    static const boost::int32_t LEVEL_12 = 0x00000800;
-    static const boost::int32_t LEVEL_13 = 0x00001000;
-    static const boost::int32_t LEVEL_14 = 0x00002000;
-    static const boost::int32_t LEVEL_15 = 0x00004000;
-    static const boost::int32_t LEVEL_16 = 0x00008000;
-    static const boost::int32_t LEVEL_17 = 0x00010000;
-    static const boost::int32_t LEVEL_18 = 0x00020000;
-    static const boost::int32_t LEVEL_19 = 0x00040000;
-    static const boost::int32_t LEVEL_20 = 0x00080000;
-    static const boost::int32_t LEVEL_21 = 0x00100000;
-    static const boost::int32_t LEVEL_22 = 0x00200000;
-    static const boost::int32_t LEVEL_23 = 0x00400000;
-    static const boost::int32_t LEVEL_24 = 0x00800000;
-    static const boost::int32_t LEVEL_25 = 0x01000000;
-    static const boost::int32_t LEVEL_26 = 0x02000000;
-    static const boost::int32_t LEVEL_27 = 0x04000000;
-    static const boost::int32_t LEVEL_28 = 0x08000000;
-    static const boost::int32_t LEVEL_29 = 0x10000000;
-    static const boost::int32_t LEVEL_30 = 0x20000000;
-    static const boost::int32_t LEVEL_31 = 0x40000000;
-    static const boost::int32_t LEVEL_32 = 0x80000000;
-    static const boost::int32_t LEVEL_ANY = 0xffffffff;
+    static const boost::int32_t LEVEL_00 = 0x00000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_01 = 0x00000001; //!< log level definition variable
+    static const boost::int32_t LEVEL_02 = 0x00000002; //!< log level definition variable
+    static const boost::int32_t LEVEL_03 = 0x00000004; //!< log level definition variable
+    static const boost::int32_t LEVEL_04 = 0x00000008; //!< log level definition variable
+    static const boost::int32_t LEVEL_05 = 0x00000010; //!< log level definition variable
+    static const boost::int32_t LEVEL_06 = 0x00000020; //!< log level definition variable
+    static const boost::int32_t LEVEL_07 = 0x00000040; //!< log level definition variable
+    static const boost::int32_t LEVEL_08 = 0x00000080; //!< log level definition variable
+    static const boost::int32_t LEVEL_09 = 0x00000100; //!< log level definition variable
+    static const boost::int32_t LEVEL_10 = 0x00000200; //!< log level definition variable
+    static const boost::int32_t LEVEL_11 = 0x00000400; //!< log level definition variable
+    static const boost::int32_t LEVEL_12 = 0x00000800; //!< log level definition variable
+    static const boost::int32_t LEVEL_13 = 0x00001000; //!< log level definition variable
+    static const boost::int32_t LEVEL_14 = 0x00002000; //!< log level definition variable
+    static const boost::int32_t LEVEL_15 = 0x00004000; //!< log level definition variable
+    static const boost::int32_t LEVEL_16 = 0x00008000; //!< log level definition variable
+    static const boost::int32_t LEVEL_17 = 0x00010000; //!< log level definition variable
+    static const boost::int32_t LEVEL_18 = 0x00020000; //!< log level definition variable
+    static const boost::int32_t LEVEL_19 = 0x00040000; //!< log level definition variable
+    static const boost::int32_t LEVEL_20 = 0x00080000; //!< log level definition variable
+    static const boost::int32_t LEVEL_21 = 0x00100000; //!< log level definition variable
+    static const boost::int32_t LEVEL_22 = 0x00200000; //!< log level definition variable
+    static const boost::int32_t LEVEL_23 = 0x00400000; //!< log level definition variable
+    static const boost::int32_t LEVEL_24 = 0x00800000; //!< log level definition variable
+    static const boost::int32_t LEVEL_25 = 0x01000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_26 = 0x02000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_27 = 0x04000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_28 = 0x08000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_29 = 0x10000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_30 = 0x20000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_31 = 0x40000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_32 = 0x80000000; //!< log level definition variable
+    static const boost::int32_t LEVEL_ANY = 0xffffffff; //!< log level definition variable
 
     /*************************************************
     Log Message Line Format:
@@ -102,19 +102,22 @@ public:
     Message := <x:Real> <y:Real>[ (c <Color>)] <Str>
     **************************************************/
 
-    static const boost::int32_t SYSTEM    = LEVEL_01;
-    static const boost::int32_t SENSOR    = LEVEL_02;
-    static const boost::int32_t WORLD     = LEVEL_03;
-    static const boost::int32_t ACTION    = LEVEL_04;
-    static const boost::int32_t INTERCEPT = LEVEL_05;
-    static const boost::int32_t KICK      = LEVEL_06;
-    static const boost::int32_t DRIBBLE   = LEVEL_07;
-    static const boost::int32_t PASS      = LEVEL_08;
-    static const boost::int32_t CROSS     = LEVEL_09;
-    static const boost::int32_t SHOOT     = LEVEL_10;
-    static const boost::int32_t CLEAR     = LEVEL_11;
-    static const boost::int32_t TEAM      = LEVEL_12;
-    static const boost::int32_t ROLE      = LEVEL_13;
+    static const boost::int32_t SYSTEM    = LEVEL_01; //!< log level definition alias
+    static const boost::int32_t SENSOR    = LEVEL_02; //!< log level definition alias
+    static const boost::int32_t WORLD     = LEVEL_03; //!< log level definition alias
+    static const boost::int32_t ACTION    = LEVEL_04; //!< log level definition alias
+    static const boost::int32_t INTERCEPT = LEVEL_05; //!< log level definition alias
+    static const boost::int32_t KICK      = LEVEL_06; //!< log level definition alias
+    static const boost::int32_t HOLD      = LEVEL_07; //!< log level definition alias
+    static const boost::int32_t DRIBBLE   = LEVEL_08; //!< log level definition alias
+    static const boost::int32_t PASS      = LEVEL_09; //!< log level definition alias
+    static const boost::int32_t CROSS     = LEVEL_10; //!< log level definition alias
+    static const boost::int32_t SHOOT     = LEVEL_11; //!< log level definition alias
+    static const boost::int32_t CLEAR     = LEVEL_12; //!< log level definition alias
+    static const boost::int32_t BLOCK     = LEVEL_13; //!< log level definition alias
+    static const boost::int32_t MARK      = LEVEL_14; //!< log level definition alias
+    static const boost::int32_t ROLE      = LEVEL_15; //!< log level definition alias
+    static const boost::int32_t TEAM      = LEVEL_16; //!< log level definition alias
 
 private:
 
@@ -139,23 +142,23 @@ public:
     ~Logger();
 
     /*!
-      \brief set new flag Id
+      \brief set new log level
       \param time const pointer to the game time instance
-      \param id new flag Id
-      \param on if true, set flag for id
+      \param level log level
+      \param on if true, set flag for the level
      */
     void setLogFlag( const GameTime * time,
-                     const boost::int32_t id,
+                     const boost::int32_t level,
                      const bool on = true );
 
     /*!
-      \brief check if level is included
-      \param id checked flag Id
-      \return true if id is included in flags
+      \brief check if the level is enabled
+      \param level checked log level
+      \return true if the level is enabled or not
      */
-    bool isLogFlag( const boost::int32_t id ) const
+    bool isEnabled( const boost::int32_t level ) const
       {
-          return ( M_flags & id );
+          return ( M_flags & level );
       }
 
     /*!
@@ -175,9 +178,11 @@ public:
 
     /*!
       \brief put message to file directry. no flush. Do NOT use this method by yourself.
+      \param cycle cycle information
       \param msg message
      */
-    void print( const char * msg );
+    void printDebugClientMessage( const long & cycle,
+                                  const std::string & msg );
 
     /*!
       \brief flush stored message
@@ -189,78 +194,103 @@ public:
     */
     void clear();
 
-
     /*!
       \brief add free message to buffer with cycle, level & message tag 'T'
-      \param id debug flag id
+      \param level debug flag level
       \param msg message
      */
-    void addText( const boost::int32_t id,
-                  const char * msg, ... );
+    void addText( const boost::int32_t level,
+                  const char * msg,
+                  ... );
 
     /*!
       \brief add point info to buffer with cycle, level & message tag 'p'
+      \param level log level variable
       \param x point coordinate x
       \param y point coordinate y
       \param color color name string
      */
-    void addPoint( const boost::int32_t id,
+    void addPoint( const boost::int32_t level,
                    const double & x,
                    const double & y,
                    const char * color = NULL );
 
-    void addPoint( const boost::int32_t id,
+    /*!
+      \brief add point info to buffer with cycle, level & message tag 'p'
+      \param level log level variable
+      \param pos point coordinate
+      \param color color name string
+     */
+    void addPoint( const boost::int32_t level,
                    const Vector2D & pos,
                    const char * color = NULL )
       {
-          addPoint( id, pos.x, pos.y, color );
+          addPoint( level, pos.x, pos.y, color );
       }
 
     /*!
       \brief add point info to buffer with cycle, level & message tag 'p'
+      \param level log level variable
       \param x point coordinate x
       \param y point coordinate y
       \param r red value
       \param g green value
       \param b blue value
      */
-    void addPoint( const boost::int32_t id,
+    void addPoint( const boost::int32_t level,
                    const double & x,
                    const double & y,
                    const char r, const char g, const char b );
 
-    void addPoint( const boost::int32_t id,
+    /*!
+      \brief add point info to buffer with cycle, level & message tag 'p'
+      \param level log level variable
+      \param pos point coordinate
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addPoint( const boost::int32_t level,
                    const Vector2D & pos,
                    const char r, const char g, const char b )
       {
-          addPoint( id, pos.x, pos.y, r, g, b );
+          addPoint( level, pos.x, pos.y, r, g, b );
       }
 
     /*!
       \brief add line info to buffer with cycle, level & message tag 'l'
+      \param level log level variable
       \param x1 line start point coordinate x
       \param y1 line start point coordinate y
       \param x2 line end point coordinate x
       \param y2 line end point coordinate y
       \param color color name string
      */
-    void addLine( const boost::int32_t id,
+    void addLine( const boost::int32_t level,
                   const double & x1,
                   const double & y1,
                   const double & x2,
                   const double & y2,
                   const char * color = NULL );
 
-    void addLine( const boost::int32_t id,
+    /*!
+      \brief add line info to buffer with cycle, level & message tag 'l'
+      \param level log level variable
+      \param start line start point coordinate
+      \param end line end point coordinate
+      \param color color name string
+     */
+    void addLine( const boost::int32_t level,
                   const Vector2D & start,
                   const Vector2D & end,
                   const char * color = NULL )
       {
-          addLine( id, start.x, start.y, end.x, end.y, color );
+          addLine( level, start.x, start.y, end.x, end.y, color );
       }
 
     /*!
       \brief add line info to buffer with cycle, level & message tag 'l'
+      \param level log level variable
       \param x1 line start point coordinate x
       \param y1 line start point coordinate y
       \param x2 line end point coordinate x
@@ -269,51 +299,75 @@ public:
       \param g green value
       \param b blue value
      */
-    void addLine( const boost::int32_t id,
+    void addLine( const boost::int32_t level,
                   const double & x1,
                   const double & y1,
                   const double & x2,
                   const double & y2,
                   const char r, const char g, const char b );
 
-    void addLine( const boost::int32_t id,
+    /*!
+      \brief add line info to buffer with cycle, level & message tag 'l'
+      \param level log level variable
+      \param start line start point coordinate
+      \param end line end point coordinate
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addLine( const boost::int32_t level,
                   const Vector2D & start,
                   const Vector2D & end,
                   const char r, const char g, const char b )
       {
-          addLine( id, start.x, start.y, end.x, end.y, r, g, b );
+          addLine( level, start.x, start.y, end.x, end.y, r, g, b );
       }
 
     /*!
       \brief add circle info to buffer with cycle, level & message tag 'c'
+      \param level log level variable
       \param x circle center point coordinate x
       \param y circle center point coordinate y
       \param radius circle radius
       \param color color name string
      */
-    void addCircle( const boost::int32_t id,
+    void addCircle( const boost::int32_t level,
                     const double & x,
                     const double & y,
                     const double & radius,
                     const char * color = NULL );
 
-    void addCircle( const boost::int32_t id,
+    /*!
+      \brief add circle info to buffer with cycle, level & message tag 'c'
+      \param level log level variable
+      \param center circle center point coordinate
+      \param radius circle radius
+      \param color color name string
+     */
+    void addCircle( const boost::int32_t level,
                     const Vector2D & center,
                     const double & radius,
                     const char * color = NULL )
       {
-          addCircle( id, center.x, center.y, radius, color );
-      }
-
-    void addCircle( const boost::int32_t id,
-                    const Circle2D & circle,
-                    const char * color = NULL )
-      {
-          addCircle( id, circle.center().x, circle.center().y, circle.radius(), color );
+          addCircle( level, center.x, center.y, radius, color );
       }
 
     /*!
       \brief add circle info to buffer with cycle, level & message tag 'c'
+      \param level log level variable
+      \param circle circle object
+      \param color color name string
+     */
+    void addCircle( const boost::int32_t level,
+                    const Circle2D & circle,
+                    const char * color = NULL )
+      {
+          addCircle( level, circle.center().x, circle.center().y, circle.radius(), color );
+      }
+
+    /*!
+      \brief add circle info to buffer with cycle, level & message tag 'c'
+      \param level log level variable
       \param x circle center point coordinate x
       \param y circle center point coordinate y
       \param radius circle radius
@@ -321,40 +375,58 @@ public:
       \param g green value
       \param b blue value
      */
-    void addCircle( const boost::int32_t id,
+    void addCircle( const boost::int32_t level,
                     const double & x,
                     const double & y,
                     const double & radius,
                     const char r, const char g, const char b );
 
-    void addCircle( const boost::int32_t id,
+    /*!
+      \brief add circle info to buffer with cycle, level & message tag 'c'
+      \param level log level variable
+      \param center circle center point coordinate
+      \param radius circle radius
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addCircle( const boost::int32_t level,
                     const Vector2D & center,
                     const double & radius,
                     const char r, const char g, const char b )
       {
-          addCircle( id, center.x, center.y, radius, r, g, b );
+          addCircle( level, center.x, center.y, radius, r, g, b );
       }
 
-    void addCircle( const boost::int32_t id,
+    /*!
+      \brief add circle info to buffer with cycle, level & message tag 'c'
+      \param level log level variable
+      \param circle circle object
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addCircle( const boost::int32_t level,
                     const Circle2D & circle,
                     const char r, const char g, const char b )
       {
-          addCircle( id,
+          addCircle( level,
                      circle.center().x, circle.center().y, circle.radius(),
                      r, g, b );
       }
 
     /*!
       \brief add triangle info to buffer with cycle, level & message tag 't'
-      \param x1 line 1st point coordinate x
-      \param y1 line 1st point coordinate y
-      \param x2 line 2nd point coordinate x
-      \param y2 line 2nd point coordinate y
-      \param x3 line 3rd point coordinate x
-      \param y3 line 3rd point coordinate y
+      \param level log level variable
+      \param x1 1st vertex coordinate x
+      \param y1 1st vertex coordinate y
+      \param x2 2nd vertex coordinate x
+      \param y2 2nd vertex coordinate y
+      \param x3 3rd vertex coordinate x
+      \param y3 3rd vertex coordinate y
       \param color color name string
      */
-    void addTriangle( const boost::int32_t id,
+    void addTriangle( const boost::int32_t level,
                       const double & x1,
                       const double & y1,
                       const double & x2,
@@ -363,24 +435,38 @@ public:
                       const double & y3,
                       const char * color = NULL );
 
-    void addTriangle( const boost::int32_t id,
+    /*!
+      \brief add triangle info to buffer with cycle, level & message tag 't'
+      \param level log level variable
+      \param p1 1st vertex coordinate
+      \param p2 2nd vertex coordinate
+      \param p3 3rd vertex coordinate
+      \param color color name string
+     */
+    void addTriangle( const boost::int32_t level,
                       const Vector2D & p1,
                       const Vector2D & p2,
                       const Vector2D & p3,
                       const char * color = NULL )
       {
-          addTriangle( id,
+          addTriangle( level,
                        p1.x, p1.y,
                        p2.x, p2.y,
                        p3.x, p3.y,
                        color );
       }
 
-    void addTriangle( const boost::int32_t id,
+    /*!
+      \brief add triangle info to buffer with cycle, level & message tag 't'
+      \param level log level variable
+      \param tri triangle object
+      \param color color name string
+     */
+    void addTriangle( const boost::int32_t level,
                       const Triangle2D & tri,
                       const char * color = NULL )
       {
-          addTriangle( id,
+          addTriangle( level,
                        tri.a().x, tri.a().y,
                        tri.b().x, tri.b().y,
                        tri.c().x, tri.c().y,
@@ -389,17 +475,18 @@ public:
 
     /*!
       \brief add triangle info to buffer with cycle, level & message tag 't'
-      \param x1 line 1st point coordinate x
-      \param y1 line 1st point coordinate y
-      \param x2 line 2nd point coordinate x
-      \param y2 line 2nd point coordinate y
-      \param x3 line 3rd point coordinate x
-      \param y3 line 3rd point coordinate y
+      \param level log level variable
+      \param x1 1st vertex coordinate x
+      \param y1 1st vertex coordinate y
+      \param x2 2nd vertex coordinate x
+      \param y2 2nd vertex coordinate y
+      \param x3 3rd vertex coordinate x
+      \param y3 3rd vertex coordinate y
       \param r red value
       \param g green value
       \param b blue value
      */
-    void addTriangle( const boost::int32_t id,
+    void addTriangle( const boost::int32_t level,
                       const double & x1,
                       const double & y1,
                       const double & x2,
@@ -408,24 +495,42 @@ public:
                       const double & y3,
                       const char r, const char g, const char b );
 
-    void addTriangle( const boost::int32_t id,
+    /*!
+      \brief add triangle info to buffer with cycle, level & message tag 't'
+      \param level log level variable
+      \param p1 1st vertex coordinate
+      \param p2 2nd vertex coordinate
+      \param p3 3rd vertex coordinate
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addTriangle( const boost::int32_t level,
                       const Vector2D & p1,
                       const Vector2D & p2,
                       const Vector2D & p3,
                       const char r, const char g, const char b )
       {
-          addTriangle( id,
+          addTriangle( level,
                        p1.x, p1.y,
                        p2.x, p2.y,
                        p3.x, p3.y,
                        r, g, b );
       }
 
-    void addTriangle( const boost::int32_t id,
+    /*!
+      \brief add triangle info to buffer with cycle, level & message tag 't'
+      \param level log level variable
+      \param tri triangle object
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addTriangle( const boost::int32_t level,
                       const Triangle2D & tri,
                       const char r, const char g, const char b )
       {
-          addTriangle( id,
+          addTriangle( level,
                        tri.a().x, tri.a().y,
                        tri.b().x, tri.b().y,
                        tri.c().x, tri.c().y,
@@ -434,24 +539,31 @@ public:
 
     /*!
       \brief add rect info to buffer with cycle, level & message tag 'r'
+      \param level log level variable
       \param left top left point coordinate x
       \param top top left point coordinate y
       \param length x range of the rectangle
       \param width y range of the rectangle
       \param color color name string
      */
-    void addRect( const boost::int32_t id,
+    void addRect( const boost::int32_t level,
                   const double & left,
                   const double & top,
                   const double & length,
                   const double & width,
                   const char * color = NULL );
 
-    void addRect( const boost::int32_t id,
+    /*!
+      \brief add rect info to buffer with cycle, level & message tag 'r'
+      \param level log level variable
+      \param rect rectangle object
+      \param color color name string
+     */
+    void addRect( const boost::int32_t level,
                   const Rect2D & rect,
                   const char * color = NULL )
       {
-          addRect( id,
+          addRect( level,
                    rect.left(), rect.top(),
                    rect.size().length(), rect.size().width(),
                    color );
@@ -459,6 +571,7 @@ public:
 
     /*!
       \brief add rect info to buffer with cycle, level & message tag 'r'
+      \param level log level variable
       \param left top left point coordinate x
       \param top top left point coordinate y
       \param length x range of the rectangle
@@ -467,18 +580,26 @@ public:
       \param g green value
       \param b blue value
      */
-    void addRect( const boost::int32_t id,
+    void addRect( const boost::int32_t level,
                   const double & left,
                   const double & top,
                   const double & length,
                   const double & width,
                   const char r, const char g, const char b );
 
-    void addRect( const boost::int32_t id,
+    /*!
+      \brief add rect info to buffer with cycle, level & message tag 'r'
+      \param level log level variable
+      \param rect rectangle object
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addRect( const boost::int32_t level,
                   const Rect2D & rect,
                   const char r, const char g, const char b )
       {
-          addRect( id,
+          addRect( level,
                    rect.left(), rect.top(),
                    rect.size().length(), rect.size().width(),
                    r, g, b );
@@ -486,48 +607,66 @@ public:
 
     /*!
       \brief add message info to buffer with cycle, level & message tag 'm'
-      \param x text drawed point coordinate x
-      \param y text drawed point coordinate y
-      \param msg drawd text
+      \param level log level variable
+      \param x painted point coordinate x
+      \param y painted point coordinate y
+      \param msg painted text
       \param color color name string
      */
-    void addMessage( const boost::int32_t id,
+    void addMessage( const boost::int32_t level,
                      const double & x,
                      const double & y,
                      const char * msg,
                      const char * color = NULL );
 
-    void addMessage( const boost::int32_t id,
+    /*!
+      \brief add message info to buffer with cycle, level & message tag 'm'
+      \param level log level variable
+      \param pos painted point coordinate
+      \param msg drawd text
+      \param color color name string
+     */
+    void addMessage( const boost::int32_t level,
                      const Vector2D & pos,
                      const char * msg,
                      const char * color = NULL )
       {
-          addMessage( id,
+          addMessage( level,
                       pos.x, pos.y, msg,
                       color );
       }
 
     /*!
       \brief add message info to buffer with cycle, level & message tag 'm'
-      \param x text drawed point coordinate x
-      \param y text drawed point coordinate y
-      \param msg drawd text
+      \param level log level variable
+      \param x painted point coordinate x
+      \param y painted point coordinate y
+      \param msg painted text
       \param r red value
       \param g green value
       \param b blue value
      */
-    void addMessage( const boost::int32_t id,
+    void addMessage( const boost::int32_t level,
                      const double & x,
                      const double & y,
                      const char * msg,
                      const char r, const char g, const char b );
 
-    void addMessage( const boost::int32_t id,
+    /*!
+      \brief add message info to buffer with cycle, level & message tag 'm'
+      \param level log level variable
+      \param pos painted point coordinate
+      \param msg painted text
+      \param r red value
+      \param g green value
+      \param b blue value
+     */
+    void addMessage( const boost::int32_t level,
                      const Vector2D & pos,
                      const char * msg,
                      const char r, const char g, const char b )
       {
-          addMessage( id,
+          addMessage( level,
                       pos.x, pos.y, msg,
                       r, g, b );
       }
