@@ -394,8 +394,7 @@ PlayerType::initAdditionalParams( const ServerParam & sparam )
 
     ///////////////////////////////////////////////////////////////////
 
-    double accel
-        = sparam.maxPower() * dashPowerRate() * effortMax();
+    double accel = sparam.maxDashPower() * dashPowerRate() * effortMax();
 
     // see also soccer_math.h
     M_real_speed_max = accel / (1.0 - playerDecay()); // sum inf geom series
@@ -410,7 +409,7 @@ PlayerType::initAdditionalParams( const ServerParam & sparam )
 
     /////////////////////////////////////////////////////////////////////
     double speed = 0.0;
-    double dash_power = sparam.maxPower();
+    double dash_power = sparam.maxDashPower();
     double stamina = sparam.staminaMax();
     double effort = this->effortMax();
     double recover = sparam.recoverInit();
@@ -427,7 +426,7 @@ PlayerType::initAdditionalParams( const ServerParam & sparam )
         if ( speed + accel > playerSpeedMax() )
         {
             accel = playerSpeedMax() - speed;
-            dash_power = std::min( sparam.maxPower(),
+            dash_power = std::min( sparam.maxDashPower(),
                                    accel / ( dashPowerRate() * effort ) );
         }
 
@@ -445,9 +444,9 @@ PlayerType::initAdditionalParams( const ServerParam & sparam )
 
         speed *= playerDecay();
 
-        predictStaminaAfterDash( sparam,
-                                 dash_power,
-                                 &stamina, &effort, &recover );
+        predictStaminaAfterOneDash( sparam,
+                                    dash_power,
+                                    &stamina, &effort, &recover );
 
         //M_stamina_table.push_back( sparam.staminaMax() - stamina );
 
@@ -564,13 +563,13 @@ PlayerType::cyclesToReachDistance( const double & dash_dist ) const
     std::vector< double >::const_iterator
         it = std::lower_bound( M_dash_distance_table.begin(),
                                M_dash_distance_table.end(),
-                               dash_dist );
+                               dash_dist - 0.001 );
 
     if ( it != M_dash_distance_table.end() )
     {
         return ( static_cast< int >
-                 ( it - M_dash_distance_table.begin() )
-                 + 1 );
+                 ( std::distance( M_dash_distance_table.begin(), it ) )
+                 + 1 ); // is it necessary?
     }
 
     double rest_dist = dash_dist - M_dash_distance_table.back();
@@ -594,9 +593,9 @@ PlayerType::getDashPowerToKeepMaxSpeed(  const ServerParam & sparam,
     // required dash power to keep max speed
     required_power /= ( effort * dashPowerRate() );
 
-    if ( required_power > sparam.maxPower() )
+    if ( required_power > sparam.maxDashPower() )
     {
-        required_power = sparam.maxPower();
+        required_power = sparam.maxDashPower();
     }
     return required_power;
 }
@@ -622,6 +621,17 @@ PlayerType::kickRate( const ServerParam & sparam,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+double
+PlayerType::dashRate( const double & effort,
+                      const double & rel_dir ) const
+{
+    return dashRate( effort ) * ServerParam::i().dashDirRate( rel_dir );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 PlayerType::normalizeAccel( const Vector2D & velocity,
@@ -631,9 +641,9 @@ PlayerType::normalizeAccel( const Vector2D & velocity,
     // if ( *accel_mag > sparam.playerAccelMax() )
     //     *accel_mag = sparam.playerAccelMax();
 
-    Vector2D dash_move
-        = Vector2D::polar2vector( *accel_mag, accel_angle ); // set dash accel
-    dash_move += velocity; // add current vel -> next move == player vel
+    Vector2D dash_move = velocity;
+    dash_move += Vector2D::polar2vector( *accel_mag, accel_angle ); // add dash accel
+
     if ( dash_move.r2() > playerSpeedMax2() + 0.0001 )
     {
         Vector2D rel_vel = velocity.rotatedVector( -accel_angle );
@@ -726,11 +736,11 @@ PlayerType::predictStaminaAfterWait( const ServerParam & sparam,
 
 */
 void
-PlayerType::predictStaminaAfterDash( const ServerParam & sparam,
-                                     const double & dash_power,
-                                     double * stamina,
-                                     double * effort,
-                                     double * recovery ) const
+PlayerType::predictStaminaAfterOneDash( const ServerParam & sparam,
+                                        const double & dash_power,
+                                        double * stamina,
+                                        double * effort,
+                                        double * recovery ) const
 {
     // decay stamina
     *stamina -= ( dash_power > 0.0 ? dash_power : (dash_power * -2.0) );
@@ -777,7 +787,7 @@ PlayerType::predictStaminaAfterDash( const ServerParam & sparam,
 
 */
 void
-PlayerType::predictStaminaAfterNrDash( const ServerParam & sparam,
+PlayerType::predictStaminaAfterDashes( const ServerParam & sparam,
                                        const double & dash_power,
                                        const int n_dash,
                                        double * stamina,
@@ -799,7 +809,7 @@ PlayerType::predictStaminaAfterNrDash( const ServerParam & sparam,
     const int n_exhaust_dash = n_dash - n_safety_dash;
     for ( int i = 0; i < n_exhaust_dash; ++i )
     {
-        predictStaminaAfterDash( sparam, dash_power, stamina, effort, recovery );
+        predictStaminaAfterOneDash( sparam, dash_power, stamina, effort, recovery );
     }
 }
 
