@@ -912,10 +912,10 @@ int
 OnePlayerMessageParser::parse( const int sender,
                                const double & ,
                                const char * msg,
-                                const GameTime & current )
+                               const GameTime & current )
 {
     // format:
-    //    "1<unum_pos_body:4>"
+    //    "P<unum_pos_body:4>"
     // the length of message == 5
 
     // 21 * 105/0.4 * 68/0.4 * 360/12
@@ -975,7 +975,104 @@ OnePlayerMessageParser::parse( const int sender,
                   player_pos.x, player_pos.y,
                   player_body.degree() );
 
-    M_memory->setPlayer( sender, player_unum, player_pos, player_body, current );
+    M_memory->setPlayer( sender,
+                         player_unum,
+                         player_pos,
+                         player_body,
+                         -1.0, // unknown stamina
+                         current );
+
+    return slength();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+SelfMessageParser::SelfMessageParser( boost::shared_ptr< AudioMemory > memory )
+    : M_memory( memory )
+{
+
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+SelfMessageParser::parse( const int sender,
+                          const double & ,
+                          const char * msg,
+                          const GameTime & current )
+{
+    // format:
+    //    "S<pos_body_stamina:4>"
+    // the length of message == 5
+
+    if ( *msg != sheader() )
+    {
+        return 0;
+    }
+
+    if ( std::strlen( msg ) < slength() )
+    {
+        std::cerr << "SelfMessageParser::parse()"
+                  << " Illegal message ["
+                  << msg << "] len = " << std::strlen( msg )
+                  << std::endl;
+        return -1;
+    }
+    ++msg;
+
+    boost::int64_t ival = 0;
+    if ( ! AudioCodec::i().decodeStrToInt64( std::string( msg, slength() - 1 ),
+                                             &ival ) )
+    {
+        std::cerr << "SelfMessageParser::parse()"
+                  << " Failed to parse [" << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "SelfMessageParser: Failed to decode Player Info [%s]",
+                      msg );
+        return -1;
+    }
+
+    int player_unum = sender;
+    Vector2D player_pos;
+    AngleDeg player_body;
+    double stamina = -1.0;
+
+    // 22
+    stamina = ServerParam::i().staminaMax()
+        * static_cast< double >( ( ival % 22 ) ) / 22.0;
+    ival /= 22;
+
+    // 30=360/12
+    player_body = static_cast< double >( ( ival % 30 ) * 12 - 180 );
+    ival /= 30;
+
+    // 170=68/0.4
+    player_pos.y = ( ival % 170 ) * 0.4 - 34.0;
+    ival /= 170;
+
+    // 263>105/0.4=262.5
+    player_pos.x = ( ival % 263 ) * 0.4 - 52.5;
+    //ival /= 263;
+
+    dlog.addText( Logger::SENSOR,
+                  "SelfMessageParser: success! "
+                  "unum = %d  pos(%.1f %.1f) body=%.1f stamina=%f",
+                  player_unum,
+                  player_pos.x, player_pos.y,
+                  player_body.degree(),
+                  stamina );
+
+    M_memory->setPlayer( sender,
+                         player_unum,
+                         player_pos,
+                         player_body,
+                         stamina,
+                         current );
 
     return slength();
 }
@@ -1079,7 +1176,12 @@ BallPlayerMessageParser::parse( const int sender,
                   player_body.degree() );
 
     M_memory->setBall( sender, ball_pos, ball_vel, current );
-    M_memory->setPlayer( sender, player_unum, player_pos, player_body, current );
+    M_memory->setPlayer( sender,
+                         player_unum,
+                         player_pos,
+                         player_body,
+                         -1.0, // unknown stamina
+                         current );
 
     return slength();
 }
