@@ -457,7 +457,7 @@ public:
   \brief offside line info message parser
 
   format:
-  "o<x_rate:1>"
+  "d<x_rate:1>"
   the length of message == 2
  */
 class DefenseLineMessageParser
@@ -954,7 +954,7 @@ public:
   \brief player info message parser
 
   format:
-  "1<unum_pos_body:>"
+  "P<unum_pos_body:>"
   the length of message == 5
  */
 class OnePlayerMessageParser
@@ -1019,6 +1019,77 @@ public:
 
 };
 
+/*-------------------------------------------------------------------*/
+/*!
+  \class SelfMessageParser
+  \brief self info message parser
+
+  format:
+  "S<pos_body_stamina>"
+  the length of message == 5
+ */
+class SelfMessageParser
+    : public SayMessageParser {
+private:
+
+    //! pointer to the audio memory
+    boost::shared_ptr< AudioMemory > M_memory;
+
+public:
+
+    /*!
+      \brief construct with audio memory
+      \param memory pointer to the memory
+     */
+    explicit
+    SelfMessageParser( boost::shared_ptr< AudioMemory > memory );
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    static
+    char sheader()
+      {
+          return 'S';
+      }
+
+    /*!
+      \brief get the header character.
+      \return header character.
+     */
+    char header() const
+      {
+          return sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    std::size_t slength()
+      {
+          return 5;
+      }
+
+    /*!
+      \brief virtual method which analyzes audio messages.
+      \param sender sender's uniform number
+      \param dir sender's direction
+      \param msg raw audio message
+      \param current current game time
+      \retval bytes read if success
+      \retval 0 message ID is not match. other parser should be tried.
+      \retval -1 failed to parse
+    */
+    int parse( const int sender,
+               const double & dir,
+               const char * msg,
+               const GameTime & current );
+
+};
+
 /*-------------------------------------------------------------------*/
 /*!
   \class BallPlayerMessageParser
