@@ -88,7 +88,6 @@ public:
     static const double DEFAULT_PLAYER_RAND;
     static const double DEFAULT_PLAYER_WEIGHT;
     static const double DEFAULT_PLAYER_SPEED_MAX;
-    static const double DEFAULT_PLAYER_SPEED_MAX_MIN;
     static const double DEFAULT_PLAYER_ACCEL_MAX;
 
     static const double DEFAULT_STAMINA_MAX;
@@ -314,12 +313,23 @@ public:
     static const std::string DEFAULT_MODULE_DIR;
 
     static const double DEFAULT_BALL_STUCK_AREA;
-
-    static const bool DEFAULT_ALLOW_MULT_DEFAULT_TYPE;
+    // 12
     static const double DEFAULT_MAX_TACKLE_POWER;
     static const double DEFAULT_MAX_BACK_TACKLE_POWER;
+    static const double DEFAULT_PLAYER_SPEED_MAX_MIN;
     static const double DEFAULT_EXTRA_STAMINA;
-
+    static const int DEFAULT_SYNCH_SEE_OFFSET;
+    // 12.1.3
+    static const int EXTRA_HALF_TIME;
+    // 13.0.0
+    static const double	STAMINA_CAPACITY;
+    static const double MAX_DASH_ANGLE;
+    static const double MIN_DASH_ANGLE;
+    static const double DASH_ANGLE_STEP;
+    static const double SIDE_DASH_RATE;
+    static const double BACK_DASH_RATE;
+    static const double MAX_DASH_POWER;
+    static const double MIN_DASH_POWER;
 private:
 
     //////////////////////////////////////////////////////
@@ -581,6 +591,19 @@ private:
     double M_extra_stamina;
     int M_synch_see_offset;
 
+    // 12.1.3
+    int M_extra_half_time;
+
+    // 13.0.0
+    double M_stamina_capacity;
+    double M_max_dash_angle;
+    double M_min_dash_angle;
+    double M_dash_angle_step;
+    double M_side_dash_rate;
+    double M_back_dash_rate;
+    double M_max_dash_power;
+    double M_min_dash_power;
+
     // 999
     int M_max_monitors;
 
@@ -1680,6 +1703,55 @@ public:
       }
 
 
+    // v12.1.3
+    int extraHalfTime() const
+      {
+          return M_extra_half_time;
+      }
+
+    // v13
+    const
+    double & staminaCapacity() const
+      {
+          return M_stamina_capacity;
+      }
+    const
+    double & maxDashAngle() const
+      {
+          return M_max_dash_angle;
+      }
+    const
+    double & minDashAngle() const
+      {
+          return M_min_dash_angle;
+      }
+    const
+    double & dashAngleStep() const
+      {
+          return M_dash_angle_step;
+      }
+    const
+    double & sideDashRate() const
+      {
+          return M_side_dash_rate;
+      }
+    const
+    double & backDashRate() const
+      {
+          return M_back_dash_rate;
+      }
+    const
+    double & maxDashPower() const
+      {
+          return M_max_dash_power;
+      }
+    const
+    double & minDashPower() const
+      {
+          return M_min_dash_power;
+      }
+
+
     // automatically defined values
 
     double pitchHalfLength() const
@@ -1779,6 +1851,52 @@ public:
           else if ( power > maxPower() ) return maxPower();
           return power;
       }
+
+    /*!
+      \brief normalize dash power
+      \param power dash power value
+      \return normalized dash power value
+     */
+    const
+    double & normalizeDashPower( const double & power ) const
+      {
+          if ( power < minDashPower() ) return  minDashPower();
+          else if ( power > maxDashPower() ) return maxDashPower();
+          return power;
+      }
+
+    /*!
+      \brief normalize dash direction
+      \param dir dash direction
+      \return normalized dash direction
+     */
+    const
+    double & normalizeDashAngle( const double & dir ) const
+      {
+          if ( dir < minDashAngle() ) return  minDashAngle();
+          else if ( dir > maxDashAngle() ) return maxDashAngle();
+          return dir;
+      }
+
+    /*!
+      \brief discretize dash direction
+      \param dir dash direction
+      \return discretized dash direction
+     */
+    double discretizeDashAngle( const double & dir ) const
+      {
+          return ( dashAngleStep() < 1.0e-10 // SERVER_EPS
+                   ? dir
+                   : dashAngleStep() * rint( dir / dashAngleStep() ) );
+      }
+
+    /*!
+      \brief calculate effectiveness for the dash direction.
+      \param dir dash direction
+      \return dash power effectiveness for dir
+     */
+    double dashDirRate( const double & dir ) const;
+
     /*!
       \brief normalize turn moment range
       \param moment command argument moment
