@@ -55,15 +55,15 @@ public:
     */
     class Param {
     public:
-        static const double PITCH_LENGTH;
-        static const double PITCH_WIDTH;
+        static const double PITCH_LENGTH; //!< field length
+        static const double PITCH_WIDTH; //!< field width
 
         //typedef BPNetwork1< 2, 4, 2 > PosNet;  // bad
         //typedef BPNetwork1< 2, 5, 2 > PosNet;  // not bad
         //typedef BPNetwork1< 2, 6, 2 > PosNet;   // not bad
         //typedef BPNetwork1< 2, 7, 2 > PosNet;   // good?
         //typedef BPNetwork1< 2, 8, 2 > PosNet;   // good?
-        typedef BPNetwork1< 2, 10, 2 > PosNet; // good
+        typedef BPNetwork1< 2, 10, 2 > PosNet; //!< alias for convenience // good
         //typedef BPNetwork1< 2, 12, 2 > PosNet;   // good
     private:
 
@@ -115,7 +115,7 @@ public:
         /*!
           \brief get strategic position
           \param ball_pos focus point, usually ball position.
-          \param type side type, if this is mirror, the refered param is used.
+          \param type side type, if this is synmetry type, the refered param is used.
           \return coordinate value
         */
         Vector2D getPosition( const Vector2D & ball_pos,
@@ -178,7 +178,7 @@ public:
 
 private:
 
-    //! key: unum. but size is not always 11 if mirror player exists.
+    //! key: unum. but size is not always 11 if synmetric player exists.
     std::map< int, boost::shared_ptr< Param > > M_param_map;
 
 public:
@@ -264,13 +264,13 @@ public:
     /*!
       \brief get position for the current focus point
       \param unum player number
-      \param forcus_point current focus point, usually ball position.
+      \param focus_point current focus point, usually ball position.
     */
     virtual
     Vector2D getPosition( const int unum,
                           const Vector2D & focus_point ) const;
 
-    /*
+    /*!
       \brief get all positions for the current focus point
       \param focus_point current focus point, usually ball position
       \param positions contaner to store the result
