@@ -444,8 +444,8 @@ FormationDT::interpolate( const int unum,
 void
 FormationDT::train( const std::list< Snapshot > & train_data )
 {
-    Rect2D pitch( - 60.0, - 45.0,
-                  120.0, 90.0 );
+    Rect2D pitch( Vector2D( -60.0, -45.0 ),
+                  Size2D( 120.0, 90.0 ) );
     M_triangulation.init( pitch );
     M_param.clear();
 
@@ -512,8 +512,8 @@ FormationDT::read( std::istream & is )
 
     ////////////////////////////////////////////////////////
     {
-        Rect2D pitch( - 60.0, - 45.0,
-                      120.0, 90.0 );
+        Rect2D pitch( Vector2D( -60.0, -45.0 ),
+                      Size2D( 120.0, 90.0 ) );
         M_triangulation.init( pitch );
 
         for ( std::vector< Param >::const_iterator it = M_param.begin();
