@@ -63,7 +63,7 @@ FormationRBF::Param::Param()
  */
 Vector2D
 FormationRBF::Param::getPosition( const Vector2D & ball_pos,
-                                  const Formation::SideType type ) const
+                                  const Formation::SideType ) const
 {
     RBFNetwork::input_vector input( 2, 0.0 );
 
