@@ -52,23 +52,44 @@ public:
      */
     struct Role {
         int number_; //!< player number
-        int mirror_; //!< mirror reference nuber. =0:center, -1:side, >0:refered number
+        int synmetry_; //!< mirror reference nuber. =0:center, -1:side, >0:refered number
         std::string name_; //!< role name string
         Vector2D pos_; //!< basic position
         Vector2D attract_; //!< attraction parameter
         Rect2D region_; //!< movable area
         bool behind_ball_; //!< defensive flag
 
+        /*!
+          \brief default constructor
+         */
         Role();
 
+        /*!
+          \brief construct with parameters
+          \param attract attraction parameter vector
+          \param region movable region
+          \param behind_ball behind flag
+         */
         Role( const Vector2D & attract,
               const Rect2D & region,
               const bool behind_ball );
 
+        /*!
+          \brief create random parameters
+         */
         void randomize();
 
+        /*!
+          \brief read data from an input stream
+          \return parsing result
+         */
         bool read( std::istream & is );
 
+        /*!
+          \brief write data to an output stream
+          \param os reference to the output stream
+          \return reference to the output stream
+         */
         std::ostream & print( std::ostream & os ) const;
     };
 
@@ -84,38 +105,74 @@ public:
         Param(); // not used
     public:
 
+        /*!
+          \brief construct with formation name
+          \param name formation name
+         */
         explicit
         Param( const std::string & name )
             : M_name( name )
           { }
 
-        int getMirror( const int unum ) const
+        /*!
+          \brief get the synmetry information of the specified player
+          \param unum target player number
+         */
+        int getSynmetry( const int unum ) const
           {
-              return M_roles.at( unum - 1 ).mirror_;
+              return M_roles.at( unum - 1 ).synmetry_;
           }
 
+        /*!
+          \brief get the role of the specified number
+          \param unum target player number
+         */
         Role & getRole( const int unum )
           {
               return M_roles.at( unum - 1 );
           }
 
+        /*!
+          \brief get the role of the specified number
+          \param unum target player number
+         */
         const
         Role & getRole( const int unum ) const
           {
               return M_roles.at( unum - 1 );
           }
 
-
+        /*!
+          \brief get the position for the input
+          \param unum target player number
+          \param ball_pos input ball position
+          \return player's move position
+         */
         Vector2D getPosition( const int unum,
                               const Vector2D & ball_pos ) const;
 
-
+        /*!
+          \brief check the validity of this formation
+          \return true if this is valid formation.
+         */
         bool check();
 
-        void createMirrorParam();
+        /*!
+          \brief set synmetric parameters
+         */
+        void createSynmetryParam();
 
+        /*!
+          \brief read data from an input stream
+          \return parsing result
+         */
         bool read( std::istream & is );
 
+        /*!
+          \brief write data to an output stream
+          \param os reference to the output stream
+          \return reference to the output stream
+         */
         std::ostream & print( std::ostream & os ) const;
 
     };
@@ -214,7 +271,7 @@ public:
     Vector2D getPosition( const int unum,
                           const Vector2D & ball_pos ) const;
 
-    /*
+    /*!
       \brief get all positions for the current focus point
       \param focus_point current focus point, usually ball position
       \param positions contaner to store the result
