@@ -50,9 +50,13 @@ namespace rcsc {
 class DelaunayTriangulation {
 public:
 
-    static const double EPSILON;
+    static const double EPSILON; //!< tolerance threshold
 
     ////////////////////////////////////////////////////////////////
+    /*!
+      \enum ContainedType
+      \brief containment type in triangles
+     */
     enum ContainedType {
         NOT_CONTAINED,
         CONTAINED,
@@ -171,8 +175,8 @@ public:
     class Edge;
     class Triangle;
 
-    typedef Edge* EdgePtr;
-    typedef Triangle* TrianglePtr;
+    typedef Edge* EdgePtr; //!< alias of Edge pointer
+    typedef Triangle* TrianglePtr; //!< alias of Triangle pointer
 
     ////////////////////////////////////////////////////////////////
     /*!
