@@ -45,15 +45,17 @@ namespace rcsc {
   Line Fomula: aX + bY + c = 0
 */
 class Line2D {
+public:
+
+    static const double EPSILON; //!< tolerance threshold
+    static const double ERROR_VALUE; //!< error value
+
 private:
 
     double M_a; //!< line fomula A, coefficient for x
     double M_b; //!< line fomula B, coefficient for y
     double M_c; //!< line fomula constant C
 
-    static const double EPSILON; //! epsilon value
-    static const double ERROR_VAL; //!< same as system max value(=HUGE_VAL)
-
     // never used
     Line2D();
 
@@ -199,7 +201,7 @@ public:
       {
           if ( std::fabs( M_a ) < EPSILON )
           {
-              return ERROR_VAL;
+              return ERROR_VALUE;
           }
           return -( M_b * y + M_c ) / M_a;
       }
@@ -213,7 +215,7 @@ public:
       {
           if ( std::fabs( M_b ) < EPSILON )
           {
-              return ERROR_VAL;
+              return ERROR_VALUE;
           }
 
           return -( M_a * x + M_c ) / M_b;
