@@ -62,8 +62,8 @@ public:
      */
     Segment2D( const Vector2D & a,
                const Vector2D & b )
-        : M_a( a )
-        , M_b( b )
+        : M_a( a ),
+          M_b( b )
       { }
 
     /*!
@@ -77,8 +77,21 @@ public:
                const double & ay,
                const double & bx,
                const double & by )
-        : M_a( ax, ay )
-        , M_b( bx, by )
+        : M_a( ax, ay ),
+          M_b( bx, by )
+      { }
+
+    /*!
+      \brief construct using origin, direction and length
+      \param a origin point
+      \param length length of line segment
+      \param dir line direction from origin point
+     */
+    Segment2D( const Vector2D & a,
+               const double & length,
+               const AngleDeg & dir )
+        : M_a( a ),
+          M_b( a + Vector2D::from_polar( length, dir ) )
       { }
 
     /*!
@@ -114,6 +127,22 @@ public:
           return *this;
       }
 
+    /*!
+      \brief construct using origin, direction and length
+      \param a origin point
+      \param dir line direction from origin point
+      \param length length of line segment
+     */
+    const
+    Segment2D & assign( const Vector2D & a,
+                        const double & length,
+                        const AngleDeg & dir )
+      {
+          M_a = a;
+          M_b = a + Vector2D::from_polar( length, dir );
+          return *this;
+      }
+
     /*!
       \brief swap segment edge point
       \return const reference to itself
@@ -122,7 +151,7 @@ public:
     Segment2D & swap()
       {
           // std::swap( M_a, M_b );
-          rcsc::Vector2D tmp = M_a;
+          Vector2D tmp = M_a;
           M_a = M_b;
           M_b = tmp;
           return *this;
@@ -247,10 +276,10 @@ public:
     */
     double farthestDist( const Vector2D & p ) const;
 
-    /*
+    /*!
       \brief check point is on segment or not
-      \p point to check
-      \p return true if point is on this segment
+      \param p checked point
+      \return true if point is on this segment
     */
     bool onSegment( const Vector2D & p ) const;
 };
