@@ -35,11 +35,13 @@
 
 #include "vector_2d.h"
 
-#include <cmath>
+#include <limits>
 
 namespace rcsc {
 
-const double Vector2D::ERROR_VALUE = HUGE_VAL; // defined in cmath (math.h)
+const double Vector2D::EPSILON = 1.0e-6;
+
+const double Vector2D::ERROR_VALUE = std::numeric_limits< double >::max();
 
 const Vector2D Vector2D::INVALIDATED( Vector2D::ERROR_VALUE, Vector2D::ERROR_VALUE );
 
