@@ -8,7 +8,7 @@
 /*
  *Copyright:
 
- Copyright (C) Hidehisa AKIYAMA
+ Copyright (C) Hidehisa AKIYAMA, Hiroki Shimora
 
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -52,7 +52,10 @@ class Vector2D {
 
 public:
 
-    //! constant error value for XY.
+    //! constant threshold value for calculation error
+    static const double EPSILON;
+
+    //! constant error value for XY (= std::numeric_limits< doulble >::max()).
     static const double ERROR_VALUE;
 
     //! invalidated value vector
@@ -65,8 +68,8 @@ public:
       \brief default constructor.
     */
     Vector2D()
-        : x( 0.0 )
-        , y( 0.0 )
+        : x( 0.0 ),
+          y( 0.0 )
       { }
 
     /*!
@@ -76,13 +79,13 @@ public:
     */
     Vector2D( const double & xx,
               const double & yy )
-        : x( xx )
-        , y( yy )
+        : x( xx ),
+          y( yy )
       { }
 
     /*!
-      \brief check if this vector has validate values.
-      \return true if value is validate.
+      \brief check if this vector has validate values. equivalent to isValid().
+      \return true if component values are validate.
     */
     bool valid() const
       {
@@ -90,21 +93,29 @@ public:
       }
 
     /*!
-      \brief type conversion operator. alias of valid().
-      \return true if value is validate.
-     */
-    operator bool() const
+      \brief check if this vector has validate values. equivalent to valid().
+      \return true if component values are validate.
+    */
+    bool isValid() const
       {
-          return valid();
+          return ( ( x != ERROR_VALUE ) && ( y != ERROR_VALUE ) );
       }
 
+    //     /*!
+    //       \brief type conversion operator. alias of valid().
+    //       \return true if value is validate.
+    //      */
+    //     operator bool() const
+    //       {
+    //           return valid();
+    //       }
+
     /*!
       \brief assign XY value directly.
       \param xx assigned x value
       \param yy assigned y value
-      \return const reference to itself
+      \return reference to itself
      */
-    const
     Vector2D & assign( const double & xx,
                        const double & yy )
       {
@@ -117,9 +128,8 @@ public:
       \brief assign XY value from POLAR value.
       \param radius vector's radius
       \param dir vector's angle
-      \return const reference to itself
+      \return reference to itself
      */
-    const
     Vector2D & setPolar( const double & radius,
                          const AngleDeg & dir )
       {
@@ -128,6 +138,10 @@ public:
           return *this;
       }
 
+    /*!
+      \brief invalidate this object
+      \return this
+     */
     const
     Vector2D & invalidate()
       {
@@ -147,22 +161,50 @@ public:
 
     /*!
       \brief get the length of vector.
-      \return length
+      \return length value
      */
     double r() const
       {
+          //return std::hypot( x, y );
           return std::sqrt( r2() );
       }
 
     /*!
-      \brief get the length of vector.
-      \return length
+      \brief get the norm value. this method is equivalent to r().
+      \return norm value
+    */
+    double norm() const
+      {
+          return r();
+      }
+
+    /*!
+      \brief get the squared norm value. this method is equivalent to r2().
+      \return squared norm value
+    */
+    double norm2() const
+      {
+          return r2();
+      }
+
+    /*!
+      \brief get the length of vector. this method is equivalent to r().
+      \return length value
      */
     double length() const
       {
           return r();
       }
 
+    /*!
+      \brief get the squared length value. this method is equivalent to r2().
+      \return squared length value
+    */
+    double length2() const
+      {
+          return r2();
+      }
+
     /*!
       \brief get the angle of vector.
       \return angle
@@ -173,7 +215,7 @@ public:
       }
 
     /*!
-      \brief get the angle of vector.
+      \brief get the angle of vector. this method is equivalent to th().
       \return angle
      */
     AngleDeg dir() const
@@ -208,13 +250,24 @@ public:
           return std::fabs( y );
       }
 
+    /*!
+      \brief add vector.
+      \param v added vector
+      \return reference to itself
+     */
+    Vector2D & add( const Vector2D & v )
+      {
+          x += v.x;
+          y += v.y;
+          return *this;
+      }
+
     /*!
       \brief add XY values respectively.
       \param xx added x value
       \param yy added y value
-      \return const reference to itself
+      \return reference to itself
      */
-    const
     Vector2D & add( const double & xx,
                     const double & yy )
       {
@@ -223,6 +276,28 @@ public:
           return *this;
       }
 
+    /*!
+      \brief scale this vector
+      \param scalar scaling factor
+      \return *this.
+    */
+    Vector2D & scale( const double & scalar )
+      {
+          x *= scalar;
+          y *= scalar;
+          return *this;
+      }
+
+    /*!
+      \brief return this vector
+      \return const reference of this vector
+     */
+    const
+    Vector2D & operator+() const
+      {
+          return *this;
+      }
+
     /*!
       \brief create reversed vector
       \return new vector that XY values are reversed.
@@ -266,9 +341,10 @@ public:
     const
     Vector2D & operator*=( const double & scalar )
       {
-          x *= scalar;
-          y *= scalar;
-          return *this;
+          //x *= scalar;
+          //y *= scalar;
+          //return *this;
+          return scale( scalar );
       }
 
     /*!
@@ -279,7 +355,8 @@ public:
     const
     Vector2D & operator/=( const double & scalar )
       {
-          if ( scalar != 0 )
+          //if ( scalar != 0 )
+          if ( scalar > EPSILON )
           {
               x /= scalar;
               y /= scalar;
@@ -306,34 +383,46 @@ public:
     */
     double dist( const Vector2D & p ) const
       {
+          //return std::hypot( this->x - p.x,
+          //                   this->y - p.y );
           return std::sqrt( dist2( p ) );
       }
 
+    /*!
+      \brief reverse vector components
+      \return *this.
+    */
+    Vector2D & reverse()
+      {
+          x = -x;
+          y = -y;
+          return *this;
+      }
+
+    /*!
+      \brief get reversed vector.
+      \return new vector object
+     */
+    Vector2D reversedVector() const
+      {
+          return Vector2D( *this ).reverse();
+      }
+
     /*!
       \brief set vector length to 'len'.
       \param len new length to be set
-      \return const reference to itself
+      \return reference to itself
     */
-    const
     Vector2D & setLength( const double & len )
       {
           double mag = this->r();
-          if ( mag == 0 )
+          //if ( mag == 0 )
+          if ( mag < EPSILON )
           {
               return *this;
           }
-          return ( (*this) *= ( len / mag ) );
-      }
-
-    /*!
-      \brief depricated.
-      \param len new length to be set
-      \return const reference to itself
-    */
-    const
-    Vector2D & resize( const double & len )
-      {
-          return setLength( len );
+          //return ( (*this) *= ( len / mag ) );
+          return scale( len / mag );
       }
 
     /*!
@@ -341,26 +430,15 @@ public:
       \param len new length to be set
       \return new vector that the length is set to 'len'
     */
-    Vector2D setLengthVector( const double & len )
+    Vector2D setLengthVector( const double & len ) const
       {
           return Vector2D( *this ).setLength( len );
       }
 
     /*!
-      \brief depricated.
-      \param len new length to be set
-      \return new vector that the length is set to 'len'
-    */
-    Vector2D resizedVector( const double & len )
-      {
-          return setLengthVector( len );
-      }
-
-    /*!
-      \brief normaliz vector. length is set to 1.0.
+      \brief normalize vector. length is set to 1.0.
       \return const reference to itself
     */
-    const
     Vector2D & normalize()
       {
           return setLength( 1.0 );
@@ -371,7 +449,7 @@ public:
       but angle is same
       \return new normalized vector
     */
-    Vector2D norm() const
+    Vector2D normalizedVector() const
       {
           return Vector2D( *this ).normalize();
       }
@@ -379,9 +457,8 @@ public:
     /*!
       \brief rotete this vector with 'deg'
       \param deg rotated angle by double type
-      \return const reference to itself
+      \return reference to itself
      */
-    const
     Vector2D & rotate( const double & deg )
       {
           double radius = this->r();
@@ -396,9 +473,8 @@ public:
     /*!
       \brief rotate this vector with 'angle'.
       \param angle rotated angle
-      \return const reference to itself
+      \return reference to itself
      */
-    const
     Vector2D & rotate( const AngleDeg & angle )
       {
           return rotate( angle.degree() );
@@ -427,9 +503,8 @@ public:
     /*!
       \brief set vector's angle to 'angle'
       \param dir new angle to be set
-      \return const reference to itself
+      \return reference to itself
      */
-    const
     Vector2D & setDir( const AngleDeg & dir )
       {
           double radius = this->r();
@@ -469,6 +544,18 @@ public:
           // == |this| * |v| * (*this - v).th().sin()
       }
 
+    /*!
+      \brief check if this vector is same as given vector
+      \param other compared vector
+      \return true if same, otherwise false
+    */
+    bool equals( const Vector2D & other ) const
+      {
+          //return dist2( other ) < EPSILON * EPSILON;
+          return std::fabs( this->x - other.x ) < EPSILON
+              && std::fabs( this->y - other.y ) < EPSILON;
+      }
+
     //////////////////////////////////////////////
     // static utility
 
@@ -486,6 +573,20 @@ public:
           return Vector2D( mag * theta.cos(), mag * theta.sin() );
       }
 
+    /*!
+      \brief get new Vector created by POLAR value.
+      \param mag length of vector
+      \param theta angle of vector
+      \return new vector object
+    */
+    inline
+    static
+    Vector2D from_polar( const double & mag,
+                         const AngleDeg & theta )
+      {
+          return Vector2D( mag * theta.cos(), mag * theta.sin() );
+      }
+
     /*!
       \brief get inner(dot) product for v1 and v2.
       \param v1 input 1
@@ -635,6 +736,39 @@ public:
 } // end of namespace
 
 
+////////////////////////////////////////////////////////
+// comparison operators
+/*!
+  \brief check vectors are same or not.
+  \param lhs left hand side parameter
+  \param rhs right hand side parameter
+  \return true if vectors are same.
+*/
+inline
+bool
+operator==( const rcsc::Vector2D & lhs,
+            const rcsc::Vector2D & rhs )
+{
+    //return lhs.x == rhs.x
+    //    && lhs.y == rhs.y;
+    return lhs.equals( rhs );
+}
+
+/*!
+  \brief check vectors are different or not.
+  \param lhs left hand side parameter
+  \param rhs right hand side parameter
+  \return true if vectors are not same.
+*/
+inline
+bool
+operator!=( const rcsc::Vector2D & lhs,
+            const rcsc::Vector2D & rhs )
+{
+    return ! operator==( lhs, rhs );
+}
+
+
 ////////////////////////////////////////////////////////
 // arithmetic operators
 
@@ -698,6 +832,89 @@ operator/( const rcsc::Vector2D & lhs,
     return rcsc::Vector2D( lhs ) /= rhs;
 }
 
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator<( const rcsc::Vector2D & lhs,
+           const T & rhs );
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator<=( const rcsc::Vector2D & lhs,
+            const T & rhs );
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator>( const rcsc::Vector2D & lhs,
+           const T & rhs );
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator>=( const rcsc::Vector2D & lhs,
+           const T & rhs );
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator<( const T & lhs,
+           const rcsc::Vector2D & rhs );
+
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator<=(const T & lhs,
+           const rcsc::Vector2D & rhs );
+
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator>( const T & lhs,
+           const rcsc::Vector2D & rhs );
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator>=( const T & lhs,
+            const rcsc::Vector2D & rhs );
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator==( const T & lhs,
+            const rcsc::Vector2D & rhs );
+
+/*!
+  \brief never used
+ */
+template < typename T >
+bool
+operator!=( const T & lhs,
+            const rcsc::Vector2D & rhs );
+
+
 
 ////////////////////////////////////////////////////////
 
