@@ -32,12 +32,14 @@
 #ifndef RCSC_PARAM_CMD_LINE_PARSER_H
 #define RCSC_PARAM_CMD_LINE_PARSER_H
 
-#include <string>
-#include <vector>
-#include <set>
-
 #include <rcsc/param/param_parser.h>
 
+#include <list>
+#include <set>
+#include <vector>
+#include <string>
+#include <ostream>
+
 namespace rcsc {
 
 /*!
@@ -47,21 +49,20 @@ namespace rcsc {
 class CmdLineParser
     : public ParamParser {
 private:
+
     //! copy of command line arguments without program name
-    std::vector< std::string > M_args;
+    std::list< std::string > M_args;
 
-    //! positional values
-    std::vector< std::string > M_positional_args;
+    //! container of the parsed positional options
+    std::vector< std::string > M_positional_options;
 
-    //! the set of accepted options
-    std::set< std::string > M_accepted_options;
-
-    //! the set of detected invalid options
-    std::set< std::string > M_invalid_options;
+    //! container of the parsed option names
+    std::vector< std::string > M_parsed_option_names;
 
     //! not used
     CmdLineParser();
 public:
+
     /*!
       \brief construct with original command line arguments
       \param argc number of argument
@@ -70,6 +71,13 @@ public:
     CmdLineParser( const int argc,
                    const char * const * argv );
 
+    /*!
+      \brief construct with original command line arguments
+      \param args argument string container
+     */
+    explicit
+    CmdLineParser( const std::list< std::string > & args );
+
     /*!
       \brief analyze arguments and results are stored to parameter map
       \param param_map reference to the parameter container
@@ -77,51 +85,67 @@ public:
      */
     bool parse( ParamMap & param_map );
 
-    /*!
-      \brief get the positional arguments
-      \return const reference to the positional arguments container
-     */
-    const
-    std::vector< std::string > & positionalArgs() const
-      {
-          return M_positional_args;
-      }
-
-    /*!
-      \brief get the set of invalie options
-      \return const reference to the string set
-     */
-    const
-    std::set< std::string > & invalidOptions() const
-      {
-          return M_invalid_options;
-      }
-
 private:
 
     /*!
-      \brief analyze long name option
-      \param param_map reference to the param map instance
-      \param long_name option name string
-      \param cur_idx index number of the name string in arguments.
-      \retval new index value if successfully parsed.
-      \retval 0 if failed.
+      \brief analyze positional options and store them to M_positional_args
      */
-    std::size_t parseLongOption( ParamMap & param_map,
-                                 const std::string & long_name,
-                                 const std::size_t cur_idx );
+    void parsePositional();
+
+public:
 
     /*!
-      \brief analyze short name option
-      \param param_map reference to the param map instance
-      \param set_name set of the option name character
-      \param cur_idx index number of the name string in arguments.
-      \retval new index value if successfully parsed.
-      \retval 0 if failed.
+      \brief get the stored arguments
+      \return const reference to the container
      */
-    std::size_t parseShortOption( ParamMap & param_map,
-                                  const std::string & short_name,
-                                  const std::size_t cur_idx );
+    const
+    std::list< std::string > & args() const
+      {
+          return M_args;
+      }
+
+    /*!
+      \brief check if all arguments are successfully parsed.
+      \return status of parsing result.
+     */
+    bool failed() const
+      {
+          return M_args.size() != M_positional_options.size();
+      }
+
+    /*!
+      \brief get the positional arguments
+      \return const reference to the container of positional argument string
+     */
+    const
+    std::vector< std::string > & positionalOptions() const
+      {
+          return M_positional_options;
+      }
+
+    /*!
+      \brief get the parsed option names
+      \return const reference to the container of parsed option names
+     */
+    const
+    std::vector< std::string > & parsedOptionNames() const
+      {
+          return M_parsed_option_names;
+      }
+
+    /*!
+      \brief get the number of the parsed option name
+      \param option_name checked option name string
+      \return the number of the parsed option name
+     */
+    std::size_t count( const std::string & option_name ) const;
+
+    /*!
+      \brief put the stored arguments to the output stream
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & print( std::ostream & os ) const;
 
 };
 
