@@ -51,6 +51,9 @@ private:
     //! delimiter character set
     const std::string M_delimiters;
 
+    //! parameter's namespace
+    const std::string M_namespace;
+
     //! not used
     ConfFileParser();
 public:
@@ -58,10 +61,12 @@ public:
       \brief construct with file path and delimiters
       \param file_path path string to the config file to be parsed
       \param delim delimiter character set
+      \param name_space namespace string
      */
     explicit
     ConfFileParser( const std::string & file_path,
-                    const std::string & delim = ":=" );
+                    const std::string & delim = ":=",
+                    const std::string & name_space = "" );
 
     /*!
       \brief analyze file content and results are stored to parameter map
