@@ -43,6 +43,29 @@
 
 namespace rcsc {
 
+/*!
+  \struct NegateBool
+  \brief negate wrapper.
+ */
+struct NegateBool {
+    bool * ptr_; //!< raw pointer to the parameter variable
+
+    /*!
+      \brief constructor
+      \param ptr raw pointer to the parameter variable.
+     */
+    explicit
+    NegateBool( bool * ptr )
+        : ptr_( ptr )
+      {
+          assert( ptr );
+      }
+
+private:
+    // not used
+    NegateBool();
+};
+
 /*!
   \struct BoolSwitch
   \brief wrapper for bool switch.
@@ -51,7 +74,7 @@ struct BoolSwitch {
     bool * ptr_; //!< raw pointer to the parameter variable
 
     /*!
-      \param constructor
+      \brief constructor
       \param ptr raw pointer to the parameter variable.
      */
     explicit
@@ -60,14 +83,53 @@ struct BoolSwitch {
       {
           assert( ptr );
       }
+
+private:
+    // not used
+    BoolSwitch();
+};
+
+
+/*!
+  \struct NegateSwitch
+  \brief negate wrapper for bool switch.
+*/
+struct NegateSwitch {
+    bool * ptr_; //!< pointer to the parameter variable.
+
+    /*!
+      \brief constructor
+      \param ptr raw pointer to the parameter variable.
+     */
+    explicit
+    NegateSwitch( bool * ptr )
+        : ptr_( ptr )
+      {
+          assert( ptr );
+      }
+
+    /*!
+      \brief constructor
+      \param value bool switch object
+     */
+    explicit
+    NegateSwitch( const BoolSwitch & value )
+        : ptr_( value.ptr_ )
+      {
+          assert( ptr_ );
+      }
+
+private:
+    // not used
+    NegateSwitch();
 };
 
 /*-------------------------------------------------------------------*/
 /*!
-  \class ParamEntry
+  \class ParamEntity
   \brief abstract parameter
 */
-class ParamEntry {
+class ParamEntity {
 private:
     //! long parameter name
     std::string M_long_name;
@@ -77,7 +139,7 @@ private:
     std::string M_description;
 
     //! not used
-    ParamEntry();
+    ParamEntity();
 
 protected:
 
@@ -87,7 +149,7 @@ protected:
       \param short_name short parameter name
       \param description description message about this parameter
     */
-    ParamEntry( const std::string & long_name,
+    ParamEntity( const std::string & long_name,
                 const std::string & short_name,
                 const char * description = "" )
         : M_long_name( long_name )
@@ -101,7 +163,7 @@ public:
       \brief destructor as virtual method
     */
     virtual
-    ~ParamEntry()
+    ~ParamEntity()
       { }
 
     /*!
@@ -134,8 +196,14 @@ public:
       }
 
     /*!
-      \brief virtual method.
-      \return always false.
+      \brief get the formatted name string for help messages
+      \return formatted name string
+     */
+    std::string helpName() const;
+
+    /*!
+      \brief (virtual) check if this parameter is switch type or not.
+      \return true if this parameter is switch type..
     */
     virtual
     bool isSwitch() const
@@ -150,6 +218,14 @@ public:
     virtual
     bool analyze( const std::string & value_str ) = 0;
 
+    /*!
+      \brief print help name strings
+      \param os reference to the stream
+      \return reference to the stream
+     */
+    virtual
+    std::ostream & printFormat( std::ostream & os ) const;
+
     /*!
       \brief pure virtual method. print value to stream
       \param os reference to the stream
@@ -161,9 +237,49 @@ public:
 };
 
 /*!
-  \brief type of ParamEntry pointer
+  \brief type of ParamEntity pointer
  */
-typedef boost::shared_ptr< ParamEntry > ParamPtr;
+typedef boost::shared_ptr< ParamEntity > ParamPtr;
+
+// /*!
+//   \class ParamPrinter
+//   \brief template value printer
+//  */
+// template < typename T >
+// class ParamPrinter {
+// public:
+//     /*!
+//       \brief print value to the output stream
+//       \param os reference to the output stream
+//       \param value output value
+//       \return reference to the output stream
+//      */
+//     std::ostream & operator()( std::ostream & os,
+//                                const T & value ) const
+//       {
+//           return os << value;
+//       }
+// };
+
+// /*!
+//   \class ParamPrinter< std::string >
+//   \brief std::string specialized value printer
+//  */
+// template <>
+// class ParamPrinter< std::string > {
+// public:
+//     /*!
+//       \brief print value to the output stream
+//       \param os reference to the output stream
+//       \param value output value
+//       \return reference to the output stream
+//      */
+//     std::ostream & operator()( std::ostream & os,
+//                                const std::string & value ) const
+//       {
+//           return os << '"' << value << '"';
+//       }
+// };
 
 
 /*-------------------------------------------------------------------*/
@@ -173,10 +289,13 @@ typedef boost::shared_ptr< ParamEntry > ParamPtr;
 */
 template < typename ValueType >
 class ParamGeneric
-    : public ParamEntry {
+    : public ParamEntity {
+public:
+    typedef ValueType Type; //!< value type alias
+
 private:
     //! pointer to parameter variable instance
-    ValueType * M_value_ptr;
+    Type * M_value_ptr;
 
 public:
 
@@ -189,9 +308,9 @@ public:
      */
     ParamGeneric( const std::string & long_name,
                   const std::string & short_name,
-                  ValueType * value_ptr,
+                  Type * value_ptr,
                   const char * description = "" )
-        : ParamEntry( long_name, short_name, description )
+        : ParamEntity( long_name, short_name, description )
         , M_value_ptr( value_ptr )
       {
           assert( value_ptr );
@@ -206,7 +325,7 @@ public:
       {
           try
           {
-              *M_value_ptr = boost::lexical_cast< ValueType >( value_str );
+              *M_value_ptr = boost::lexical_cast< Type >( value_str );
               return true;
           }
           catch ( boost::bad_lexical_cast & e )
@@ -217,6 +336,24 @@ public:
           }
       }
 
+private:
+
+    template < typename T >
+    std::ostream & printValueImpl( std::ostream & os,
+                                   const T & value ) const
+      {
+          os << value;
+          return os;
+      }
+
+    std::ostream & printValueImpl( std::ostream & os,
+                                   const std::string & value ) const
+      {
+          os << '"' << value << '"';
+          return os;
+      }
+
+public:
     /*!
       \brief print value to stream
       \param os reference to the stream
@@ -224,18 +361,32 @@ public:
      */
     std::ostream & printValue( std::ostream & os ) const
       {
-          return os << *M_value_ptr;
+          //return ParamPrinter< Type >()( os, *M_value_ptr );
+          return printValueImpl( os, *M_value_ptr );
+//           if ( typeid( ValueType ) == typeid( std::string ) )
+//           {
+//               os << '"' << *M_value_ptr << '"';
+//           }
+//           else
+//           {
+//               os << *M_value_ptr;
+//           }
+//           return os;
       }
+
 };
 
 /*-------------------------------------------------------------------*/
 /*!
-  \brief specificated template class
+  \class ParamGeneric< bool >
+  \brief specialized template class
 */
 template <>
 class ParamGeneric< bool >
-    : public ParamEntry {
+    : public ParamEntity {
 private:
+    const bool M_negate;
+
     //! pointer to parameter variable instance
     bool * M_value_ptr;
 
@@ -252,12 +403,31 @@ public:
                   const std::string & short_name,
                   bool * value_ptr,
                   const char * description = "" )
-        : ParamEntry( long_name, short_name, description )
+        : ParamEntity( long_name, short_name, description )
+        , M_negate( false )
         , M_value_ptr( value_ptr )
       {
           assert( value_ptr );
       }
 
+    /*!
+      \brief constructor
+      \param long_name parameter's long name string
+      \param short_name parameter's short name string(ommitable)
+      \param value negate bool object
+      \param description description message(ommitable)
+     */
+    ParamGeneric( const std::string & long_name,
+                  const std::string & short_name,
+                  const NegateBool & value,
+                  const char * description = "" )
+        : ParamEntity( long_name, short_name, description )
+        , M_negate( true )
+        , M_value_ptr( value.ptr_ )
+      {
+          assert( M_value_ptr );
+      }
+
     /*!
       \brief analyze value string and substitute it to variable.
       \param value_str string that contains value
@@ -271,6 +441,7 @@ public:
       \return reference to the stream
      */
     std::ostream & printValue( std::ostream & os ) const;
+
 };
 
 /*-------------------------------------------------------------------*/
@@ -279,8 +450,10 @@ public:
   \brief switch type parameter
 */
 class ParamSwitch
-    : public ParamEntry {
+    : public ParamEntity {
 private:
+    const bool M_negate;
+
     //! pointer to parameter variable instance
     bool * M_value_ptr;
 
@@ -297,14 +470,34 @@ public:
                  const std::string & short_name,
                  bool * value_ptr,
                  const char * description = "" )
-        : ParamEntry( long_name, short_name, description )
+        : ParamEntity( long_name, short_name, description )
+        , M_negate( false )
         , M_value_ptr( value_ptr )
       {
           assert( value_ptr );
       }
 
+
     /*!
-      \brief overrided method
+      \brief constructor
+      \param long_name parameter's long name string
+      \param short_name parameter's short name string(ommitable)
+      \param value negate bool object
+      \param description description message(ommitable)
+     */
+    ParamSwitch( const std::string & long_name,
+                 const std::string & short_name,
+                 const NegateSwitch & value,
+                 const char * description = "" )
+        : ParamEntity( long_name, short_name, description )
+        , M_negate( true )
+        , M_value_ptr( value.ptr_ )
+      {
+          assert( M_value_ptr );
+      }
+
+    /*!
+      \brief (virtual) check if this parameter is switch type or not.
       \return always true.
     */
     virtual
@@ -320,12 +513,20 @@ public:
     */
     bool analyze( const std::string & value_str );
 
+    /*!
+      \brief print usage format.
+      \param os reference to the stream
+      \return reference to the stream
+     */
+    std::ostream & printFormat( std::ostream & os ) const;
+
     /*!
       \brief print value to stream
       \param os reference to the stream
       \return reference to the stream
      */
     std::ostream & printValue( std::ostream & os ) const;
+
 };
 
 
@@ -357,22 +558,11 @@ private:
             : M_param_map( pmap )
           { }
 
-    private:
-
-        /*!
-          \brief check the validity of the option name string
-          \return true if the option name is valid.
-         */
-        bool checkName( const std::string & long_name,
-                        const std::string & short_name );
-
-    public:
-
         /*!
           \brief parameter registration operator
-          \param value_ptr pointer to parameter variable instance
           \param long_name parameter's long name
           \param short_name parameter's short name(ommitable)
+          \param value_ptr pointer to parameter variable instance
           \param description parameter's description message(ommitable)
           \return reference to itself
          */
@@ -397,12 +587,24 @@ private:
               return *this;
           }
 
+        /*!
+          \brief parameter registration operator for negate bool
+          \param long_name parameter's long name
+          \param short_name parameter's short name(ommitable)
+          \param value wrapper object
+          \param description parameter's description message(ommitable)
+          \return reference to itself
+         */
+        Registrar & operator()( const std::string & long_name,
+                                const std::string & short_name,
+                                const NegateBool & value,
+                                const char * description = "" );
 
         /*!
           \brief parameter registration operator for bool switch
-          \param value switch wrapper object
           \param long_name parameter's long name
           \param short_name parameter's short name(ommitable)
+          \param value wrapper object
           \param description parameter's description message(ommitable)
           \return reference to itself
          */
@@ -411,13 +613,25 @@ private:
                                 const BoolSwitch & value,
                                 const char * description = "" );
 
+       /*!
+          \brief parameter registration operator for negate bool switch
+          \param long_name parameter's long name
+          \param short_name parameter's short name(ommitable)
+          \param value wrapper object
+          \param description parameter's description message(ommitable)
+          \return reference to itself
+         */
+        Registrar & operator()( const std::string & long_name,
+                                const std::string & short_name,
+                                const NegateSwitch & value,
+                                const char * description = "" );
     };
 
     //! parameter registrar
     Registrar M_registrar;
 
-    //! option name: output before help messages
-    std::string M_option_name;
+    //! option group name: output before help messages
+    std::string M_group_name;
 
     //! parameter container
     std::vector< ParamPtr > M_parameters;
@@ -439,12 +653,12 @@ public:
 
     /*!
       \brief construct with option name string
-      \param option_name option name string
+      \param group_name option group name string
      */
     explicit
-    ParamMap( const std::string & option_name )
+    ParamMap( const std::string & group_name )
         : M_registrar( *this )
-        , M_option_name( option_name )
+        , M_group_name( group_name )
       { }
 
     /*!
@@ -454,10 +668,50 @@ public:
       { }
 
     /*!
-      \brief copy parameter map from argument
-      \param param_map reference to the parameter map
+      \brief get the name of parameter group
+      \return name string
      */
-    ParamMap & add( ParamMap & param_map );
+    const
+    std::string & groupName() const
+      {
+          return M_group_name;
+      }
+
+    /*!
+      \brief get the container of all parameters
+      \return const reference to the container instance
+     */
+    const
+    std::vector< ParamPtr > & parameters() const
+      {
+          return M_parameters;
+      }
+
+    /*!
+      \brief get the long name parameter map
+      \return const reference to the container instance
+     */
+    const
+    std::map< std::string, ParamPtr > & longNameMap() const
+      {
+          return M_long_name_map;
+      }
+
+    /*!
+      \brief get the short name parameter map
+      \return const reference to the container instance
+     */
+    const
+    std::map< std::string, ParamPtr > & shortNameMap() const
+      {
+          return M_short_name_map;
+      }
+
+//     /*
+//       \brief copy parameter map from argument
+//       \param param_map reference to the parameter map
+//      */
+//     ParamMap & add( ParamMap & param_map );
 
     /*!
       \brief get a parameter registrar
@@ -485,14 +739,14 @@ public:
       \param long_name long version parameter name string
       \return parameter entry pointer. if not found, NULL is returned.
      */
-    ParamPtr getParamPtr( const std::string & long_name );
+    ParamPtr findLongName( const std::string & long_name );
 
     /*!
       \brief get parameter entry that has the argument name
       \param short_name set of the parameter name character
       \return parameter entry pointer. if not found, NULL is returned.
      */
-    ParamPtr getParamPtrShortName( const std::string & short_name );
+    ParamPtr findShortName( const std::string & short_name );
 
     /*!
       \brief output parameter usage by command line option style
