@@ -35,6 +35,9 @@
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/game_time.h>
 #include <rcsc/soccer_math.h>
+#include <rcsc/math_util.h>
+
+#include <deque>
 
 namespace rcsc {
 
@@ -62,61 +65,65 @@ private:
     //! validation count threshold value for M_vel
     static int S_vel_count_thr;
 
+    static const std::size_t MAX_RECORD; //!< max record size
+
+    /*!
+      \brief internal state representation
+     */
+    struct State {
+        Vector2D pos_; //!< estimated global position
+        Vector2D pos_error_; //!< estimated error of global position
+        int pos_count_; //!< accuracy count. this is usually the cycle count since last observation
+
+        Vector2D rpos_; //!< estimated relative position
+        Vector2D rpos_error_; //!< estimated error fo relative position
+        int rpos_count_; //!< accuracy count. this is usually the cycle count since last observation
+
+        Vector2D seen_pos_; //!< seen global position
+        Vector2D seen_rpos_; //!< seen relative position
+        int seen_pos_count_; //!< accuracy count. the cycle count since last see update
+
+        Vector2D heard_pos_; //!< heard global position
+        int heard_pos_count_; //!< accuracy count. the cycle count since last hear update
+
+        Vector2D vel_; //!< estimated velocity
+        Vector2D vel_error_; //!< estimated error of velocity
+        int vel_count_; //!< accuracy count. this is usually the cycle count since last observation
+
+        Vector2D seen_vel_; //!< seen velocity
+        int seen_vel_count_; //!< accuracy count. the cycle count since last see update
+
+        Vector2D heard_vel_; //!< heard velocity
+        int heard_vel_count_; //!< accuracy count. the cycle count since last hear update
+
+        int lost_count_; //!< the cycle count since ball lost
+
+        bool ghost_; //!< ghost flag
+
+        /*!
+          \brief initialize all variables
+         */
+        State();
+    };
+
+    //! current state
+    State M_state;
+
+    std::deque< State > M_state_record;
+
     //! estimated distance from self
     double M_dist_from_self;
     //! estimated global angle from self
     AngleDeg M_angle_from_self;
 
-    //! estimated global coordinate
-    Vector2D M_pos;
-    //! estimated global coordinate error
-    Vector2D M_pos_error;
-    //! cycle count since last observation
-    int M_pos_count;
-
-    //! estimated position relative to self
-    Vector2D M_rpos;
-    //! estimated error of relative coordinate
-    Vector2D M_rpos_error;
-    //! cycle count since last relative pos observation
-    int M_rpos_count;
     //! relative position at previous cycle. updated only in pure internal update
     Vector2D M_rpos_prev;
 
-    //! last seen global position
-    Vector2D M_seen_pos;
-    //! cycle count since last observation
-    int M_seen_pos_count;
-    //! previous seen relative position
-    Vector2D M_seen_rpos;
 
-    //! last heard global position
-    Vector2D M_heard_pos;
-    //! cycle count since last observation
-    int M_heard_pos_count;
-
-    //! estimated global velocity
-    Vector2D M_vel;
-    //! estimated error of global velocity
-    Vector2D M_vel_error;
-    //! velocity accuracy. count since last observation
-    int M_vel_count;
-
-    //! last heard velocity
-    Vector2D M_heard_vel;
-    //! cycle count since last observation
-    int M_heard_vel_count;
-
-    //! count since ball lost
-    int M_lost_count;
-
-    //! last ghost check time
-    GameTime M_ghost_time;
-
-    //! not used
+    // not used
     BallObject( const BallObject & ball );
-    //! not used
     BallObject & operator=( const BallObject & ball );
+
 public:
     /*!
       \brief constructor. initialize member variables
@@ -134,6 +141,26 @@ public:
                         const int rpos_thr,
                         const int vel_thr );
 
+    /*!
+      \brief get current state
+      \return const reference to the state object
+     */
+    const
+    State & state() const
+      {
+          return M_state;
+      }
+
+    /*!
+      \brief get state record container
+      \return const reference to the state container
+     */
+    const
+    std::deque< State > & stateRecord() const
+      {
+          return M_state_record;
+      }
+
     /*!
       \brief get estimated distance from self
       \return distance value
@@ -160,7 +187,7 @@ public:
     const
     Vector2D & pos() const
       {
-          return M_pos;
+          return M_state.pos_;
       }
 
     /*!
@@ -170,7 +197,7 @@ public:
     const
     Vector2D & posError() const
       {
-          return M_pos_error;
+          return M_state.pos_error_;
       }
 
     /*!
@@ -179,7 +206,7 @@ public:
     */
     int posCount() const
       {
-          return M_pos_count;
+          return M_state.pos_count_;
       }
 
     /*!
@@ -189,7 +216,7 @@ public:
     const
     Vector2D & rpos() const
       {
-          return M_rpos;
+          return M_state.rpos_;
       }
 
     /*!
@@ -199,7 +226,7 @@ public:
     const
     Vector2D & rposError() const
       {
-          return M_rpos_error;
+          return M_state.rpos_error_;
       }
 
     /*!
@@ -208,7 +235,7 @@ public:
     */
     int rposCount() const
       {
-          return M_rpos_count;
+          return M_state.rpos_count_;
       }
 
     /*!
@@ -228,7 +255,7 @@ public:
     const
     Vector2D & seenPos() const
       {
-          return M_seen_pos;
+          return M_state.seen_pos_;
       }
 
     /*!
@@ -237,7 +264,7 @@ public:
     */
     int seenPosCount() const
       {
-          return M_seen_pos_count;
+          return M_state.seen_pos_count_;
       }
 
     /*!
@@ -247,7 +274,7 @@ public:
     const
     Vector2D & seenRPos() const
       {
-          return M_seen_rpos;
+          return M_state.seen_rpos_;
       }
 
     /*!
@@ -257,7 +284,7 @@ public:
     const
     Vector2D & heardPos() const
       {
-          return M_heard_pos;
+          return M_state.heard_pos_;
       }
 
     /*!
@@ -266,7 +293,7 @@ public:
     */
     int heardPosCount() const
       {
-          return M_heard_pos_count;
+          return M_state.heard_pos_count_;
       }
 
     /*!
@@ -276,7 +303,7 @@ public:
     const
     Vector2D & vel() const
       {
-          return M_vel;
+          return M_state.vel_;
       }
 
     /*!
@@ -286,7 +313,7 @@ public:
     const
     Vector2D & velError() const
       {
-          return M_vel_error;
+          return M_state.vel_error_;
       }
 
     /*!
@@ -295,7 +322,25 @@ public:
     */
     int velCount() const
       {
-          return M_vel_count;
+          return M_state.vel_count_;
+      }
+    /*!
+      \brief get the last seen velocity
+      \return const reference to the point object
+     */
+    const
+    Vector2D & seenVel() const
+      {
+          return M_state.seen_vel_;
+      }
+
+    /*!
+      \brief get the number of cycles since last velocity seen
+      \return count since last velocity seen
+    */
+    int seenVelCount() const
+      {
+          return M_state.seen_vel_count_;
       }
 
     /*!
@@ -304,17 +349,7 @@ public:
     */
     int lostCount() const
       {
-          return M_lost_count;
-      }
-
-    /*!
-      \brief get ghost detected time
-      \return game time value
-     */
-    const
-    GameTime & ghostTime() const
-      {
-          return M_ghost_time;
+          return M_state.lost_count_;
       }
 
     /*!
@@ -323,7 +358,7 @@ public:
     */
     bool posValid() const
       {
-          return M_pos_count < S_pos_count_thr;
+          return M_state.pos_count_ < S_pos_count_thr;
       }
 
     /*!
@@ -332,7 +367,7 @@ public:
     */
     bool rposValid() const
       {
-          return M_rpos_count < S_rpos_count_thr;
+          return M_state.rpos_count_ < S_rpos_count_thr;
       }
 
     /*!
@@ -341,7 +376,7 @@ public:
     */
     bool velValid() const
       {
-          return M_vel_count < S_vel_count_thr;
+          return M_state.vel_count_ < S_vel_count_thr;
       }
 
     /*!
@@ -414,6 +449,11 @@ public:
                         const Vector2D & vel_err,
                         const int vel_count );
 
+    /*!
+      \brief update by opponent control effect
+     */
+    void setOpponentControlEffect();
+
     /*!
       \brief update position by see info (not include velocity)
       \param pos estimated global position
@@ -452,13 +492,15 @@ public:
 
     /*!
       \brief update ball status using heared info
+      \param act const referenct to the ActionEffector instance
       \param heard_pos heard position
       \param heard_vel heard velocity
       \param current current game time
 
       This method is called just before decision.
     */
-    void updateByHear( const Vector2D & heard_pos,
+    void updateByHear( const ActionEffector & act,
+                       const Vector2D & heard_pos,
                        const Vector2D & heard_vel,
                        const GameTime & current );
 
@@ -470,6 +512,10 @@ public:
     */
     void updateSelfRelated( const SelfObject & self );
 
+    /*!
+      \brief update historical record data
+     */
+    void updateRecord();
     // ------------------------------------------
     // utilities
 
@@ -500,18 +546,20 @@ public:
     */
     Vector2D inertiaPoint( const int cycle ) const;
 
+
     /*!
-      \brief estimate final movement travel vector
-      \return estimated travel vector
+      \brief estimate reach point
+      \return estimated point
     */
-    Vector2D inertiaFinalTravel() const;
+    Vector2D inertiaFinalPoint() const;
 
     /*!
       \brief estimate final reach point
       \return estimated point vector
     */
-    Vector2D inertiaFinalPoint() const;
-
+    static
+    double calc_travel_step( const double & distance,
+                             const double & first_speed );
 };
 
 }
