@@ -50,6 +50,7 @@ BodySensor::BodySensor()
     : M_time( 0, 0 )
     , M_stamina( 4000.0 )
     , M_effort( 1.0 )
+    , M_stamina_capacity( -1.0 )
     , M_speed_mag( 0.0 )
     , M_speed_dir_relative( 0.0 )
     , M_neck_relative( 0.0 )
@@ -117,6 +118,13 @@ BodySensor::parse1( const char * msg,
     //  (focus (target none) (count 0)) (tackle (expires 0) (count 0))
     //  (collision {none|[(ball)][player][post]}))
 
+    // ver. 13
+    // (sense_body 66 (view_mode high normal) (stamina 3503.4 1 124000) (speed 0.06 -79)
+    //  (head_angle 89) (kick 4) (dash 20) (turn 24) (say 0) (turn_neck 28) (catch 0)
+    //  (move 1) (change_view 16) (arm (movable 0) (expires 0) (target 0 0) (count 0))
+    //  (focus (target none) (count 0)) (tackle (expires 0) (count 0))
+    //  (collision {none|[(ball)][player][post]}))
+
     //char ss[8];
 
     M_time = current;
@@ -163,10 +171,15 @@ BodySensor::parse1( const char * msg,
     // read stamina values
     while ( *msg != '(' ) ++msg;
     while ( *msg != ' ' ) ++msg; // skip "(stamina"
-    M_stamina = std::strtod( msg, &next );
-    msg = next;
-    M_effort = std::strtod( msg, &next );
-    msg = next;
+    M_stamina = std::strtod( msg, &next ); msg = next;
+    M_effort = std::strtod( msg, &next ); msg = next;
+    if ( version >= 13.0 )
+    {
+        if ( *msg != ')' )
+        {
+            M_stamina_capacity = std::strtod( msg, &next ); msg = next;
+        }
+    }
 
     // read speed values
     while ( *msg != '(' ) ++msg;
