@@ -59,15 +59,19 @@ class WorldModel;
 */
 class DebugClient {
 public:
+    /*!
+      \enum ServerType
+      \brief debug server types
+     */
     enum ServerType {
         SoccerViewer,
         SoccerWindow2,
     };
 
-    static const std::size_t MAX_LINE = 50;
-    static const std::size_t MAX_TRIANGLE = 50;
-    static const std::size_t MAX_RECT = 50;
-    static const std::size_t MAX_CIRCLE = 50;
+    static const std::size_t MAX_LINE = 50; //!< maximum number of lines in one message.
+    static const std::size_t MAX_TRIANGLE = 50; //!< maximum number of triangles in one message.
+    static const std::size_t MAX_RECT = 50; //!< maximum number of rectangles in one message.
+    static const std::size_t MAX_CIRCLE = 50; //!< maximum number of circles in one message.
 
 private:
 
@@ -166,7 +170,8 @@ public:
     /*!
       \brief add formated string to buffer
     */
-    void addMessage( char * msg, ... );
+    void addMessage( const char * msg,
+                     ... );
 
     /*!
       \brief set target player
