@@ -47,7 +47,7 @@
 
 namespace rcsc {
 
-const std::size_t InterceptTable::MAX_CYCLE = 24;
+const std::size_t InterceptTable::MAX_CYCLE = 30;
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -59,7 +59,7 @@ InterceptTable::InterceptTable( const WorldModel & world )
     M_ball_pos_cache.reserve( MAX_CYCLE + 2 );
     //M_ball_vel_cache.reserve( MAX_CYCLE + 2 );
 
-    M_self_cache.reserve( MAX_CYCLE + 2 );
+    M_self_cache.reserve( ( MAX_CYCLE + 2 ) * 2 );
 
     clear();
 }
@@ -86,10 +86,14 @@ InterceptTable::clear()
     M_self_reach_cycle = 1000;
     M_self_exhaust_reach_cycle = 1000;
     M_teammate_reach_cycle = 1000;
+    M_second_teammate_reach_cycle = 1000;
     M_opponent_reach_cycle = 1000;
+    M_second_opponent_reach_cycle = 1000;
 
     M_fastest_teammate = static_cast< PlayerObject * >( 0 );
+    M_second_teammate = static_cast< PlayerObject * >( 0 );
     M_fastest_opponent = static_cast< PlayerObject * >( 0 );
+    M_second_opponent = static_cast< PlayerObject * >( 0 );
 
     M_self_cache.clear();
 }
@@ -119,7 +123,8 @@ InterceptTable::update()
     // if ball is always stopped in the current playmode,
     // not needed to predict ball chaser.
     if ( M_world.gameMode().type() != GameMode::PlayOn
-         && M_world.gameMode().type() != GameMode::GoalKick_ )
+         && M_world.gameMode().type() != GameMode::GoalKick_
+         && M_world.gameMode().type() != GameMode::PenaltyTaken_ )
     {
         return;
     }
@@ -128,8 +133,7 @@ InterceptTable::update()
          || ! M_world.ball().posValid() )
     {
         dlog.addText( Logger::INTERCEPT,
-                      "%s:%d: Invalid self or ball pos"
-                      ,__FILE__, __LINE__ );
+                      "InterceptTable Invalid self or ball pos" );
         return;
     }
 
@@ -138,8 +142,7 @@ InterceptTable::update()
          || M_world.existKickableOpponent() )
     {
         dlog.addText( Logger::INTERCEPT,
-                      "%s:%d: Already exist kickable player"
-                      ,__FILE__, __LINE__ );
+                      "InterceptTable Already exist kickable player" );
     }
 
     createBallCache();
@@ -165,9 +168,8 @@ InterceptTable::update()
 
     predictTeammate();
 
-
     dlog.addText( Logger::INTERCEPT,
-                  "<-----Intercet Self reach cycle = %d. exhaust reach step = %d ",
+                  "<-----Intercept Self reach cycle = %d. exhaust reach step = %d ",
                   M_self_reach_cycle,
                   M_self_exhaust_reach_cycle );
     if ( M_fastest_teammate )
@@ -181,16 +183,39 @@ InterceptTable::update()
                       M_fastest_teammate->pos().y );
 
     }
+
+    if ( M_second_teammate )
+    {
+        dlog.addText( Logger::INTERCEPT,
+                      "<-----Intercept Teammate  2nd     reach step = %d."
+                      " teammate %d (%.1f %.1f)",
+                      M_second_teammate_reach_cycle,
+                      M_second_teammate->unum(),
+                      M_second_teammate->pos().x,
+                      M_second_teammate->pos().y );
+    }
+
     if ( M_fastest_opponent )
     {
         dlog.addText( Logger::INTERCEPT,
                       "<-----Intercept Opponent  fastest reach step = %d."
-                      " player %d (%.1f %.1f)",
+                      " opponent %d (%.1f %.1f)",
                       M_opponent_reach_cycle,
                       M_fastest_opponent->unum(),
                       M_fastest_opponent->pos().x,
                       M_fastest_opponent->pos().y );
     }
+
+    if ( M_second_opponent )
+    {
+        dlog.addText( Logger::INTERCEPT,
+                      "<-----Intercept Opponent  2nd     reach step = %d."
+                      " opponent %d (%.1f %.1f)",
+                      M_second_opponent_reach_cycle,
+                      M_second_opponent->unum(),
+                      M_second_opponent->pos().x,
+                      M_second_opponent->pos().y );
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -329,7 +354,9 @@ InterceptTable::createBallCache()
             // out of pitch
             break;
         }
-        if ( bvel.r2() < 0.01 )
+
+        if ( i >= 5
+             && bvel.r2() < 0.01*0.01 )
         {
             // ball stopped
             M_ball_pos_cache.push_back( bpos );
@@ -353,7 +380,7 @@ InterceptTable::predictSelf()
     if ( M_world.self().isKickable() )
     {
         dlog.addText( Logger::INTERCEPT,
-                      "Intercept Teammate. already kickable. no estimation loop!" );
+                      "Intercept Self. already kickable. no estimation loop!" );
         M_self_reach_cycle = 0;
         return;
     }
@@ -372,9 +399,37 @@ InterceptTable::predictSelf()
         return;
     }
 
+#if 0
+    {
+        const std::vector< InterceptInfo >::iterator send = M_self_cache.end();
+        for ( std::vector< InterceptInfo >::iterator it = M_self_cache.begin();
+              it != send;
+              ++it )
+        {
+            dlog.addText( Logger::INTERCEPT,
+                          "== type=%d cycle=%d (turn=%d dash=%d0 dash_power=%.2f",
+                          it->mode(),
+                          it->reachCycle(),
+                          it->turnCycle(),
+                          it->dashCycle(),
+                          it->dashPower() );
+        }
+    }
+#endif
+
+#ifdef SELF_INTERCEPT_USE_NO_SAVE_RECEVERY
+    std::sort( M_self_cache.begin(),
+               M_self_cache.end(),
+               InterceptInfoCmp() );
+    M_self_cache.erase( std::unique( M_self_cache.begin(),
+                                     M_self_cache.end(),
+                                     InterceptInfoEqual() ),
+                        M_self_cache.end() );
+#endif
 
     int min_cycle = M_self_reach_cycle;
     int exhaust_min_cycle = M_self_exhaust_reach_cycle;
+
     const std::vector< InterceptInfo >::iterator end = M_self_cache.end();
     for ( std::vector< InterceptInfo >::iterator it = M_self_cache.begin();
           it != end;
@@ -385,16 +440,37 @@ InterceptTable::predictSelf()
             if ( it->reachCycle() < min_cycle )
             {
                 min_cycle = it->reachCycle();
+                break;
             }
         }
-        else
+        else if ( it->mode() == InterceptInfo::EXHAUST )
         {
             if ( it->reachCycle() < exhaust_min_cycle )
             {
                 exhaust_min_cycle = it->reachCycle();
+                break;
             }
         }
     }
+
+    dlog.addText( Logger::INTERCEPT,
+                  "Intercept Self. solution size = %d",
+                  M_self_cache.size() );
+#if 0
+    for ( std::vector< InterceptInfo >::iterator it = M_self_cache.begin();
+          it != end;
+          ++it )
+    {
+        dlog.addText( Logger::INTERCEPT,
+                      "-- type=%d cycle=%d (turn=%d dash=%d) dash_power=%.2f",
+                      it->mode(),
+                      it->reachCycle(),
+                      it->turnCycle(),
+                      it->dashCycle(),
+                      it->dashPower() );
+    }
+#endif
+
     M_self_reach_cycle = min_cycle;
     M_self_exhaust_reach_cycle = exhaust_min_cycle;
 }
@@ -421,19 +497,22 @@ InterceptTable::predictTeammate()
         {
             if ( (*t)->isGhost()
                    || (*t)->posCount() > M_world.ball().posCount() + 1 )
-              {
-                  continue;
-              }
+            {
+                continue;
+            }
 
-              if ( (*t)->isKickable() )
-              {
-                  M_fastest_teammate = *t;
-              }
+            M_fastest_teammate = *t;
+            dlog.addText( Logger::INTERCEPT,
+                          "---> set fastest teammate %d (%.1f %.1f)",
+                          (*t)->unum(),
+                          (*t)->pos().x, (*t)->pos().y );
+            break;
         }
         return;
     }
 
-    int max_cycle = 1000;
+    int min_cycle = 1000;
+    int second_min_cycle = 1000;
 
     PlayerIntercept predictor( M_world, M_ball_pos_cache );
 
@@ -444,15 +523,14 @@ InterceptTable::predictTeammate()
         if ( (*it)->posCount() >= 10 )
         {
             dlog.addText( Logger::INTERCEPT,
-                          "Intercept  %s Teammate %d.(%.1f %.1f) Low accuracy %d. skip...",
-                          ( (*it)->unum() < 0 ? "Unknown" : "" ),
+                          "Intercept Teammate %d.(%.1f %.1f) Low accuracy %d. skip...",
                           (*it)->unum(),
                           (*it)->pos().x, (*it)->pos().y,
                           (*it)->posCount() );
             continue;
         }
 
-        const PlayerType * player_type = M_world.teammatePlayerType( (*it)->unum() );
+        const PlayerType * player_type = (*it)->playerTypePtr();
         if ( ! player_type )
         {
             std::cerr << M_world.time()
@@ -468,24 +546,34 @@ InterceptTable::predictTeammate()
         }
 
         int cycle = predictor.predict( *(*it), *player_type,
-                                       max_cycle );
+                                       second_min_cycle );
         dlog.addText( Logger::INTERCEPT,
-                      "---> %s Teammate %d.(%.1f %.1f) cycle = %d",
-                      ( (*it)->unum() < 0 ? "Unknown" : "" ),
+                      "---> Teammate %d.(%.1f %.1f) type=%d cycle=%d",
                       (*it)->unum(),
                       (*it)->pos().x, (*it)->pos().y,
+                      player_type->id(),
                       cycle );
-        if ( cycle < max_cycle )
+        if ( cycle < second_min_cycle )
         {
-            max_cycle = cycle;
-            M_fastest_teammate = *it;
+            second_min_cycle = cycle;
+            M_second_teammate = *it;
+
+            if ( second_min_cycle < min_cycle )
+            {
+                std::swap( min_cycle, second_min_cycle );
+                std::swap( M_fastest_teammate, M_second_teammate );
+            }
         }
     }
 
-
-    if ( M_fastest_teammate && max_cycle < 1000 )
+    if ( M_second_teammate && second_min_cycle < 1000 )
     {
-        M_teammate_reach_cycle = max_cycle;
+        M_second_teammate_reach_cycle = second_min_cycle;
+    }
+
+    if ( M_fastest_teammate && min_cycle < 1000 )
+    {
+        M_teammate_reach_cycle = min_cycle;
     }
 }
 
@@ -511,20 +599,23 @@ InterceptTable::predictOpponent()
         {
             if ( (*o)->isGhost()
                    || (*o)->posCount() > M_world.ball().posCount() + 1 )
-              {
-                  continue;
-              }
+            {
+                continue;
+            }
 
-              if ( (*o)->isKickable() )
-              {
-                  M_fastest_opponent = *o;
-              }
+            M_fastest_opponent = *o;
+            dlog.addText( Logger::INTERCEPT,
+                          "---> set fastest opponent %d (%.1f %.1f)",
+                          (*o)->unum(),
+                          (*o)->pos().x, (*o)->pos().y );
+            break;
         }
 
         return;
     }
 
-    int max_cycle = 1000;
+    int min_cycle = 1000;
+    int second_min_cycle = 1000;
 
     PlayerIntercept predictor( M_world, M_ball_pos_cache );
 
@@ -532,18 +623,17 @@ InterceptTable::predictOpponent()
           it != o_end;
           ++it )
     {
-        if ( (*it)->posCount() >= 10 )
+        if ( (*it)->posCount() >= 15 )
         {
             dlog.addText( Logger::INTERCEPT,
-                          "Intercept  %s Opponent %d. .(%.1f %.1f) Low accuracy %d. skip...",
-                          ( (*it)->unum() < 0 ? "Unknown" : "" ),
+                          "Intercept Opponent %d.(%.1f %.1f) Low accuracy %d. skip...",
                           (*it)->unum(),
                           (*it)->pos().x, (*it)->pos().y,
                           (*it)->posCount() );
             continue;
         }
 
-        const PlayerType * player_type = M_world.opponentPlayerType( (*it)->unum() );
+        const PlayerType * player_type = (*it)->playerTypePtr();
         if ( ! player_type )
         {
             std::cerr << M_world.time()
@@ -557,131 +647,37 @@ InterceptTable::predictOpponent()
                           (*it)->unum());
             continue;
         }
+
         int cycle = predictor.predict( *(*it), *player_type,
-                                       max_cycle );
+                                       second_min_cycle );
         dlog.addText( Logger::INTERCEPT,
-                      "---> %s Opponent.%d (%.1f %.1f) reach cycle = %d",
-                      ( (*it)->unum() < 0 ? "Unknown" : "" ),
+                      "---> Opponent.%d (%.1f %.1f) type=%d cycle=%d",
                       (*it)->unum(),
                       (*it)->pos().x, (*it)->pos().y,
+                      player_type->id(),
                       cycle );
-        if ( cycle < max_cycle )
+        if ( cycle < second_min_cycle )
         {
-            max_cycle = cycle;
-            M_fastest_opponent = *it;
+            second_min_cycle = cycle;
+            M_second_opponent = *it;
+
+            if ( second_min_cycle < min_cycle )
+            {
+                std::swap( min_cycle, second_min_cycle );
+                std::swap( M_fastest_opponent, M_second_opponent );
+            }
         }
     }
 
-    if ( M_fastest_opponent && max_cycle < 1000 )
+    if ( M_second_opponent && second_min_cycle < 1000 )
     {
-        M_opponent_reach_cycle = max_cycle;
+        M_second_opponent_reach_cycle = second_min_cycle;
+    }
+
+    if ( M_fastest_opponent && min_cycle < 1000 )
+    {
+        M_opponent_reach_cycle = min_cycle;
     }
 }
 
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-Vector2D
-InterceptTable::selfInterceptPoint() const
-{
-    return M_world.ball().inertiaPoint( selfReachCycle() );
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-bool
-InterceptTable::isSelfFastestPlayer() const
-{
-    if ( M_world.existKickableOpponent()
-         || M_world.existKickableTeammate() )
-    {
-        dlog.addText( Logger::TEAM,
-                      "%s: isSelfFastestPlayer. exist other kickable player"
-                      ,__FILE__ );
-        return false;
-    }
-    if ( M_world.self().isKickable() )
-    {
-        dlog.addText( Logger::TEAM,
-                      "%s: isSelfFastestPlayer. I am already kickable"
-                      ,__FILE__ );
-        return true;
-    }
-
-    if ( selfReachCycle() == 1 )
-    {
-        dlog.addText( Logger::TEAM,
-                      "%s: isSelfFastestPlayer. self min = 1"
-                      ,__FILE__ );
-        return true;
-    }
-
-    Vector2D reach_point = M_world.ball().inertiaPoint( selfReachCycle() );
-    double self_dash = M_world.self().pos().dist( reach_point );
-
-    double teammate_dash = 500.0, opponent_dash = 300.0;
-    M_world.getTeammateNearestTo( reach_point, 30, &teammate_dash );
-    M_world.getOpponentNearestTo( reach_point, 30, &opponent_dash );
-
-    if ( teammateReachCycle() > 2
-         && self_dash < teammate_dash - 1.0
-         && opponentReachCycle() > 2
-         && self_dash < opponent_dash )
-    {
-        dlog.addText( Logger::TEAM,
-                      "%s: isSelfFastestPlayer. Maybe I am fastest. cycle = %d"
-                      " move_dist=%.2f  teammate_move=%.2f  opponent_move=%.2f"
-                      ,__FILE__,
-                      teammateReachCycle(),
-                      self_dash, teammate_dash, opponent_dash );
-        return true;
-    }
-
-    dlog.addText( Logger::TEAM,
-                  "%s: isSelfFastestPlayer. I am *NOT* a fastest player"
-                  ,__FILE__ );
-    return false;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-bool
-InterceptTable::isOurTeamBallPossessor() const
-{
-    if ( M_world.existKickableTeammate() )
-    {
-        dlog.addText( Logger::TEAM,
-                      "%s: isOurTeamBallPossessor. exist kickable teammate"
-                      ,__FILE__ );
-        return true;
-    }
-
-    if ( teammateReachCycle() < opponentReachCycle() - 1 )
-    {
-        dlog.addText( Logger::TEAM,
-                      "%s: isOurTeamBallPossessor. teammate can reach faster than opps"
-                      ,__FILE__ );
-        return true;
-    }
-
-    if ( isSelfFastestPlayer() )
-    {
-        dlog.addText( Logger::TEAM,
-                      "%s: isOurTeamBallPossessor. I am fastest"
-                      ,__FILE__ );
-        return true;
-    }
-
-    dlog.addText( Logger::TEAM,
-                  "%s: isOurTeamBallPossessor. Ball is out of our posession"
-                  ,__FILE__ );
-
-    return false;
-}
-
 }
