@@ -48,6 +48,10 @@ class WorldModel;
 */
 class InterceptInfo {
 public:
+    /*!
+      \enum Mode
+      \brief stamina consumption mode
+     */
     enum Mode {
         NORMAL = 0, //!< ball gettable without stamina exhaust
         EXHAUST = 100, //!< fastest ball gettable, but recovery may be consumed.
@@ -75,6 +79,33 @@ public:
         , M_dash_power( dash_power )
       { }
 
+    /*!
+      \brief copy cnstructor
+      \param other copied variable
+     */
+    InterceptInfo( const InterceptInfo & other )
+        : M_mode( other.mode() )
+        , M_turn_cycle( other.turnCycle() )
+        , M_dash_cycle( other.dashCycle() )
+        , M_dash_power( other.dashPower() )
+      { }
+
+    /*!
+      \brief substitution operator
+      \param other substitution variable
+     */
+    const InterceptInfo & operator=( const InterceptInfo & other )
+      {
+          if ( this != &other )
+          {
+              M_mode = other.mode();
+              M_turn_cycle = other.turnCycle();
+              M_dash_cycle = other.dashCycle();
+              M_dash_power = other.dashPower();
+          }
+          return *this;
+      }
+
     /*!
       \brief get interception mode Id
       \return mode Id
@@ -102,7 +133,7 @@ public:
           return M_dash_cycle;
       }
 
-    /*
+    /*!
       \brief get esitimated total cycle to reach
       \return the number of total cycles
     */
@@ -122,6 +153,44 @@ public:
       }
 };
 
+/*!
+  \class InterceptInfoCmp
+  \brief interception info compare functor
+*/
+struct InterceptInfoCmp {
+    /*!
+      \brief operator function
+      \param lhs left hand side variable
+      \param rhs right hand side variable
+      \return compared result
+     */
+    bool operator()( const InterceptInfo & lhs,
+                     const InterceptInfo & rhs ) const
+      {
+          return lhs.reachCycle() < rhs.reachCycle();
+      }
+};
+
+/*!
+  \class InterceptInfoEqual
+  \brief interception info compare functor
+*/
+struct InterceptInfoEqual {
+    /*!
+      \brief operator function
+      \param lhs left hand side variable
+      \param rhs right hand side variable
+      \return compared result
+     */
+    bool operator()( const InterceptInfo & lhs,
+                     const InterceptInfo & rhs ) const
+      {
+          return ( lhs.mode() == rhs.mode()
+                   && lhs.turnCycle() == rhs.turnCycle()
+                   && lhs.dashCycle() == rhs.dashCycle() );
+      }
+};
+
 /*-------------------------------------------------------------------*/
 
 /*!
@@ -147,13 +216,21 @@ private:
     int M_self_exhaust_reach_cycle;
     //! predicted min reach cycle for teammate
     int M_teammate_reach_cycle;
+    //! predicted reach cycle for second fastest teammate
+    int M_second_teammate_reach_cycle;
     //! predicted min reach cycle for opponent
     int M_opponent_reach_cycle;
+    //! predicted reach cycle for second fastest opponent
+    int M_second_opponent_reach_cycle;
 
     //! const pointer to the fastest ball gettable teammate player object
     const PlayerObject * M_fastest_teammate;
+    //! const pointer to the second fastest ball gettable teammate player object
+    const PlayerObject * M_second_teammate;
     //! const pointer to the fastest ball gettable opponent player object
     const PlayerObject * M_fastest_opponent;
+    //! const pointer to the second fastest ball gettable opponent player object
+    const PlayerObject * M_second_opponent;
 
     //! interception info cache for smart interception
     std::vector< InterceptInfo > M_self_cache;
@@ -198,7 +275,7 @@ public:
                        const int cycle );
 
     /*!
-      \brief get minimal ball gettable cycle for self without stamina exaust
+      \brief get minimal ball gettable cycle for self without stamina exhaust
       \return cycle value to get the ball
     */
     int selfReachCycle() const
@@ -207,7 +284,7 @@ public:
       }
 
     /*!
-      \brief get minimal ball gettable cycle for self with stamina exaust
+      \brief get minimal ball gettable cycle for self with stamina exhaust
       \return cycle value to get the ball
     */
     int selfExhaustReachCycle() const
@@ -224,6 +301,15 @@ public:
           return M_teammate_reach_cycle;
       }
 
+    /*!
+      \brief get the ball access cycle for the second teammate
+      \return cycle value to get the ball
+    */
+    int secondTeammateReachCycle() const
+      {
+          return M_second_teammate_reach_cycle;
+      }
+
     /*!
       \brief get minimal ball gettable cycle for opponent
       \return cycle value to get the ball
@@ -234,7 +320,16 @@ public:
       }
 
     /*!
-      \brief get fastest ball gettable teammate object
+      \brief get the ball access cycle for the second opponent
+      \return cycle value to get the ball
+    */
+    int secondOpponentReachCycle() const
+      {
+          return M_second_opponent_reach_cycle;
+      }
+
+    /*!
+      \brief get the teammate object fastest to the ball
       \return const pointer to the PlayerObject.
       if not exist such a player, return NULL
     */
@@ -245,7 +340,18 @@ public:
       }
 
     /*!
-      \brief get fastest ball gettable oppnent object
+      \brief get the teammate object second fastest to the ball
+      \return const pointer to the PlayerObject.
+      if not exist such a player, return NULL
+    */
+    const
+    PlayerObject * secondTeammate() const
+      {
+          return M_second_teammate;
+      }
+
+    /*!
+      \brief get the oppnent object fastest to the ball
       \return const pointer to the PlayerObject.
       if not exist such a player, return NULL
     */
@@ -255,6 +361,17 @@ public:
           return M_fastest_opponent;
       }
 
+    /*!
+      \brief get the oppnent object second fastest to the ball
+      \return const pointer to the PlayerObject.
+      if not exist such a player, return NULL
+    */
+    const
+    PlayerObject * secondOpponent() const
+      {
+          return M_second_opponent;
+      }
+
     /*!
       \brief get self interception cache container
       \return const reference to the interception info container
@@ -265,24 +382,6 @@ public:
           return M_self_cache;
       }
 
-    /*!
-      \brief get ball gettable point for self
-      \return ball gettaable point
-    */
-    Vector2D selfInterceptPoint() const;
-
-    /*!
-      \brief check if self is the fastest ball bettable player
-      \return true if estimated self is the fastest player
-    */
-    bool isSelfFastestPlayer() const;
-
-    /*!
-      \brief check if our team has the ball.
-      \return true if estimated our team has the ball.
-    */
-    bool isOurTeamBallPossessor() const;
-
 private:
     /*!
       \brief clear all cached data
