@@ -59,6 +59,7 @@
 #include <rcsc/math_util.h>
 #include <rcsc/game_time.h>
 #include <rcsc/game_mode.h>
+#include <rcsc/timer.h>
 
 #include <sstream>
 #include <cstdio>
@@ -111,7 +112,7 @@ struct PlayerAgentImpl {
 
     //! time when sense_body is received
     TimeStamp body_time_stamp_;
-    //!< time when see is received
+    //! time when see is received
     TimeStamp see_time_stamp_;
 
     //! status of the see messaege arrival timing
@@ -166,9 +167,11 @@ struct PlayerAgentImpl {
     /*!
       \brief check if now decision timing
       \param msec_from_sense elapsed milli seconds from last sense_body message arrival
+      \param timeout_count timeout count since last sensing message.
       \return true if player should send action
     */
-    bool isDecisionTiming( const long & msec_from_sense ) const;
+    bool isDecisionTiming( const long & msec_from_sense,
+                           const int timeout_count ) const;
 
 };
 
@@ -203,7 +206,7 @@ PlayerAgentImpl::updateCurrentTime( const long & new_time,
                           << " Stop Mode: previous server time is incorrect?? "
                           << old_time << " -> " << new_time << std::endl;
                 dlog.addText( Logger::SYSTEM,
-                              "System: stop mode: previous server time is incorrect??  (%ld, %ld) -> %ld",
+                              __FILE__": System: stop mode: previous server time is incorrect??  (%ld, %ld) -> %ld",
                               old_time.cycle(), old_time.stopped(),
                               new_time );
             }
@@ -231,7 +234,7 @@ PlayerAgentImpl::updateCurrentTime( const long & new_time,
                     else
                     {
                         dlog.addText( Logger::SYSTEM,
-                                      "missed last action(1)..." );
+                                      __FILE__": missed last action(1)..." );
                         std::cout << agent_.config().teamName() << ' '
                                   << agent_.world().self().unum() << ": "
                                   << current_time_
@@ -261,7 +264,7 @@ PlayerAgentImpl::updateCurrentTime( const long & new_time,
                           << " skipped server time?? "
                           << old_time << " -> " << new_time << std::endl;
                 dlog.addText( Logger::SYSTEM,
-                              "skipped server time?? (%ld, %ld) -> %ld",
+                              __FILE__": skipped server time?? (%ld, %ld) -> %ld",
                               old_time.cycle(), old_time.stopped(),
                               new_time );
             }
@@ -270,7 +273,7 @@ PlayerAgentImpl::updateCurrentTime( const long & new_time,
                  && last_decision_time_.cycle() != new_time - 1 )
             {
                 dlog.addText( Logger::SYSTEM,
-                              "missed last action(2)..." );
+                              __FILE__": missed last action(2)..." );
                 std::cout << agent_.config().teamName() << ' '
                           << agent_.world().self().unum() << ": "
                           << current_time_
@@ -310,7 +313,8 @@ PlayerAgentImpl::updateServerStatus()
 
 */
 bool
-PlayerAgentImpl::isDecisionTiming( const long & msec_from_sense ) const
+PlayerAgentImpl::isDecisionTiming( const long & msec_from_sense,
+                                   const int timeout_count ) const
 {
     // if server is synch mode, 'think' message is sent to all clients
     // so it is not necessary to check decision timing.
@@ -319,6 +323,12 @@ PlayerAgentImpl::isDecisionTiming( const long & msec_from_sense ) const
         return false;
     }
 
+    // sense_body is not received yet
+    if ( msec_from_sense < 0 )
+    {
+        return false;
+    }
+
     // already done in current cycle
     if ( last_decision_time_ == current_time_ )
     {
@@ -337,37 +347,56 @@ PlayerAgentImpl::isDecisionTiming( const long & msec_from_sense ) const
         return true;
     }
 
+    int wait_thr = ( see_state_.isSynch()
+                     ? agent_.config().waitTimeThrSynchView()
+                     : agent_.config().waitTimeThrNoSynchView() );
+
+    // already done in sense_body received cycle.
+    // When referee message is sent before sense_body,
+    // it may cause confusion to players.
+    if ( last_decision_time_ == agent_.world().senseBodyTime()
+         && timeout_count <= 2
+         //&& msec_from_sense < ( ( ServerParam::i().simulatorStep() + wait_thr )
+         //                       * ServerParam::i().slowDownFactor() )
+        )
+    {
+        dlog.addText( Logger::SYSTEM,
+                      __FILE__": last_decision_time=(%ld,%ld) is same as last_sense_body=(%ld,%ld)",
+                      last_decision_time_.cycle(),
+                      last_decision_time_.stopped(),
+                      agent_.world().senseBodyTime().cycle(),
+                      agent_.world().senseBodyTime().stopped() );
+        return false;
+    }
+
+    // no see info during the current cycle.
     if ( see_state_.isSynch()
          && see_state_.cyclesTillNextSee() > 0 )
     {
         dlog.addText( Logger::SYSTEM,
-                      "estimated cycles till next see ----- %d",
+                      __FILE__": estimated cycles till next see ----- %d",
                       see_state_.cyclesTillNextSee() );
         return true;
     }
 
-    // sense_body is not received yet
-    if ( msec_from_sense < 0 )
-    {
-        return false;
-    }
-
-    int wait_thr = ( see_state_.isSynch()
-                     ? agent_.config().waitTimeThrSynchView()
-                     : agent_.config().waitTimeThrNoSynchView() );
+    // over the wait threshold
     if ( msec_from_sense >= wait_thr * ServerParam::i().slowDownFactor() )
     {
         if ( see_state_.isSynch() )
         {
-            std::cout << agent_.config().teamName() << ' '
-                      << agent_.world().self().unum() << ": "
-                      << current_time_
-                      << " over offset - " << msec_from_sense
-                      << "   server response delayed"
-                      << std::endl;
+            if ( ! SeeState::synch_see_mode()
+                 || msec_from_sense > ServerParam::i().synchSeeOffset() * ServerParam::i().slowDownFactor() )
+            {
+                std::cout << agent_.config().teamName() << ' '
+                          << agent_.world().self().unum() << ": "
+                          << current_time_
+                          << " over offset - " << msec_from_sense
+                          << "   server response delayed"
+                          << std::endl;
+            }
         }
         dlog.addText( Logger::SYSTEM,
-                      "over offset - %ld",
+                      __FILE__": over offset - %ld",
                       msec_from_sense );
         return true;
     }
@@ -386,12 +415,17 @@ PlayerAgent::PlayerAgent()
     , M_impl( new PlayerAgentImpl( *this ) )
     , M_debug_client()
     , M_worldmodel()
+    , M_fullstate_worldmodel()
     , M_effector( *this )
 {
     // std::cerr << "construct player" << std::endl;
 
     boost::shared_ptr< AudioMemory > audio_memory( new AudioMemory );
     M_worldmodel.setAudioMemory( audio_memory );
+
+    boost::shared_ptr< AudioMemory > fullstate_audio_memory( new AudioMemory );
+    M_fullstate_worldmodel.setAudioMemory( fullstate_audio_memory );
+    M_fullstate_worldmodel.setValid( false );
 }
 
 /*-------------------------------------------------------------------*/
@@ -498,14 +532,12 @@ PlayerAgent::initImpl( CmdLineParser & cmd_parser )
     ParamMap player_param_map( "Player options" );
     M_config.createParamMap( player_param_map );
 
-
     // analyze command line for system options.
     cmd_parser.parse( system_param_map );
     if ( help )
     {
         system_param_map.printHelp( std::cout );
         player_param_map.printHelp( std::cout );
-        M_client->setServerAlive( false );
         return false;
     }
 
@@ -520,7 +552,7 @@ PlayerAgent::initImpl( CmdLineParser & cmd_parser )
     cmd_parser.parse( player_param_map );
 
     if ( config().version() < 8.0
-         || 13.0 <= config().version() )
+         || 14.0 <= config().version() )
     {
         std::cerr << "Unsupported client version: " << config().version()
                   << std::endl;
@@ -606,7 +638,7 @@ PlayerAgent::handleMessage()
         parse( M_client->message() );
     }
 
-    // game cycle is change while several message parsing
+    // game cycle is changed while several message parsing
     if ( M_impl->current_time_.cycle() > start_time.cycle() + 1
          && start_time.stopped() == 0
          && M_impl->current_time_.stopped() == 0 )
@@ -619,7 +651,7 @@ PlayerAgent::handleMessage()
                   << "  end_time= " << M_impl->current_time_
                   << std::endl;
         dlog.addText( Logger::SYSTEM,
-                      "parser used several steps -- action missed! sensed %d"
+                      __FILE__": parser used several steps -- action missed! sensed %d"
                       " start=(%ld, %ld) end=(%ld, %ld)",
                       counter,
                       start_time.cycle(), start_time.stopped(),
@@ -630,7 +662,7 @@ PlayerAgent::handleMessage()
     if ( M_impl->think_received_ )
     {
         dlog.addText( Logger::SYSTEM,
-                      "Got think message: decide action" );
+                      __FILE__": Got think message: decide action" );
         action();
         M_impl->think_received_ = false;
     }
@@ -643,7 +675,7 @@ PlayerAgent::handleMessage()
             // player got a current cycle visual information
             // decide action imeddiately
             dlog.addText( Logger::SYSTEM,
-                          "Got see info: decide action" );
+                          __FILE__": Got see info: decide action" );
             action();
         }
     }
@@ -698,7 +730,7 @@ PlayerAgent::handleTimeout( const int timeout_count,
     }
 
     // check alarm count etc...
-    if ( ! M_impl->isDecisionTiming( msec_from_sense ) )
+    if ( ! M_impl->isDecisionTiming( msec_from_sense, timeout_count ) )
     {
         return;
     }
@@ -854,13 +886,16 @@ PlayerAgent::setDebugFlags()
     dlog.setLogFlag( &(M_impl->current_time_), Logger::ACTION, config().debugAction() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::INTERCEPT, config().debugIntercept() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::KICK, config().debugKick() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::HOLD, config().debugHold() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::DRIBBLE, config().debugDribble() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::PASS, config().debugPass() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::CROSS, config().debugCross() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::SHOOT, config().debugShoot() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::CLEAR, config().debugClear() );
-    dlog.setLogFlag( &(M_impl->current_time_), Logger::TEAM, config().debugTeam() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::BLOCK, config().debugBlock() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::MARK, config().debugMark() );
     dlog.setLogFlag( &(M_impl->current_time_), Logger::ROLE, config().debugRole() );
+    dlog.setLogFlag( &(M_impl->current_time_), Logger::TEAM, config().debugTeam() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -957,7 +992,7 @@ PlayerAgent::analyzeCycle( const char * msg,
                   << msg
                   << std::endl;
         dlog.addText( Logger::SENSOR,
-                      "Cycle parse error [%s]", msg );
+                      __FILE__": Cycle parse error [%s]", msg );
         return false;
     }
 
@@ -1020,7 +1055,7 @@ PlayerAgent::analyzeSee( const char * msg )
                   << world().time()
                   << " got see without sense_body." << std::endl;
         dlog.addText( Logger::SYSTEM,
-                      "analyzeSee. (%ld, %ld) without sense_body",
+                      __FILE__": analyzeSee. (%ld, %ld) without sense_body",
                       M_impl->visual_.time().cycle(),
                       M_impl->visual_.time().stopped() );
     }
@@ -1041,7 +1076,7 @@ PlayerAgent::analyzeSee( const char * msg )
          && ! M_impl->see_state_.isSynch() )
     {
         dlog.addText( Logger::SYSTEM,
-                      "SEE received. but see timing is not synched. try to adjust" );
+                      __FILE__": SEE received. but see timing is not synched. try to adjust" );
         adjustSeeSynchNormalMode();
     }
 }
@@ -1080,9 +1115,9 @@ PlayerAgent::analyzeSenseBody( const char * msg )
     // check command counter
     M_effector.checkCommandCount( M_impl->body_ );
     // pure internal update
-    M_worldmodel.updateAfterSense( M_impl->body_,
-                                   M_effector,
-                                   M_impl->current_time_ );
+    M_worldmodel.updateAfterSenseBody( M_impl->body_,
+                                       M_effector,
+                                       M_impl->current_time_ );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1130,7 +1165,7 @@ PlayerAgent::analyzeHear( const char * msg )
                   << msg << ']'
                   << std::endl;
         dlog.addText( Logger::SENSOR,
-                      "Hear parse error [%s]", msg );
+                      __FILE__": Hear parse error [%s]", msg );
         return;
     }
 
@@ -1238,6 +1273,7 @@ PlayerAgent::analyzeHearReferee( const char * msg )
     }
 
     M_worldmodel.updateGameMode( M_impl->game_mode_, M_impl->current_time_ );
+    M_fullstate_worldmodel.updateGameMode( M_impl->game_mode_, M_impl->current_time_ );
 
     ////////////////////////////////////////////////////////
     // if playmode change to NOT play_on mode,
@@ -1293,7 +1329,7 @@ PlayerAgent::analyzeHearOurCoach( const char * msg )
 
 */
 void
-PlayerAgent::analyzeHearOpponentCoach( const char * msg )
+PlayerAgent::analyzeHearOpponentCoach( const char * )
 {
     dlog.addText( Logger::SENSOR,
                   "===receive say message from opponent coach" );
@@ -1335,6 +1371,11 @@ PlayerAgent::analyzeFullstate( const char * msg )
         M_impl->fullstate_.reverse();
     }
 
+    M_fullstate_worldmodel.updateAfterFullstate( M_impl->fullstate_,
+                                                 effector(),
+                                                 M_impl->current_time_ );
+    M_fullstate_worldmodel.setValid( true );
+
     if ( config().useFullstate() )
     {
         M_worldmodel.updateAfterFullstate( M_impl->fullstate_,
@@ -1356,6 +1397,8 @@ PlayerAgent::analyzePlayerType( const char * msg )
                             msg,
                             config().version() );
     PlayerTypeSet::instance().insert( player_type );
+
+    handlePlayerType();
 }
 
 /*-------------------------------------------------------------------*/
@@ -1368,6 +1411,8 @@ PlayerAgent::analyzePlayerParam( const char * msg )
     dlog.addText( Logger::SENSOR,
                   "===receive player_param" );
     PlayerParam::instance().parse( msg, config().version() );
+
+    handlePlayerParam();
 }
 
 /*-------------------------------------------------------------------*/
@@ -1385,6 +1430,8 @@ PlayerAgent::analyzeServerParam( const char * msg )
 
     M_worldmodel.setTeammatePlayerType( world().self().unum(),
                                         Hetero_Default );
+    M_fullstate_worldmodel.setTeammatePlayerType( fullstateWorld().self().unum(),
+                                                  Hetero_Default );
 
     // update alarm interval
     if ( ! ServerParam::i().synchMode()
@@ -1400,6 +1447,8 @@ PlayerAgent::analyzeServerParam( const char * msg )
         */
         M_client->setIntervalMSec( interval );
     }
+
+    handleServerParam();
 }
 
 /*-------------------------------------------------------------------*/
@@ -1483,7 +1532,10 @@ PlayerAgent::analyzeInit( const char * msg )
 
     if ( ! M_worldmodel.initTeamInfo( config().teamName(),
                                       sidetype, myunum,
-                                      config().goalie() ) )
+                                      config().goalie() )
+        || ! M_fullstate_worldmodel.initTeamInfo( config().teamName(),
+                                                  sidetype, myunum,
+                                                  config().goalie() ) )
     {
         M_client->setServerAlive( false );
         return;
@@ -1604,18 +1656,20 @@ PlayerAgent::analyzeChangePlayerType( const char * msg )
     int type = Hetero_Unknown;
 
     dlog.addText( Logger::SENSOR,
-                  "parse change_player_type [%s]",
+                  __FILE__": parse change_player_type [%s]",
                   msg );
 
     if ( std::sscanf( msg, " ( change_player_type %d %d ) ",
                       &unum, &type ) == 2 )
     {
         M_worldmodel.setTeammatePlayerType( unum, type );
+        M_fullstate_worldmodel.setTeammatePlayerType( unum, type );
     }
     else if ( std::sscanf( msg, " ( change_player_type %d ) ",
                            &unum ) == 1 )
     {
         M_worldmodel.setOpponentPlayerType( unum, Hetero_Unknown );
+        M_fullstate_worldmodel.setOpponentPlayerType( unum, Hetero_Unknown );
     }
     else
     {
@@ -1624,7 +1678,7 @@ PlayerAgent::analyzeChangePlayerType( const char * msg )
                   << world().time()
                   << " ***ERROR*** Failed to analyze change_player_type" << std::endl;
         dlog.addText( Logger::SENSOR,
-                      "error: change_player_type" );
+                      __FILE__": error: change_player_type" );
     }
 }
 
@@ -1693,7 +1747,7 @@ PlayerAgent::analyzeOK( const char * msg )
               << M_impl->current_time_
               << " recv unsupported or illegal ok message [" << msg << "]" << std::endl;
     dlog.addText( Logger::SENSOR,
-                  "unsupported ok" );
+                  __FILE__": unsupported ok" );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1747,8 +1801,9 @@ PlayerAgent::analyzeWarning( const char * msg )
 void
 PlayerAgent::action()
 {
+    MSecTimer timer;
     dlog.addText( Logger::SYSTEM,
-                  "decide action" );
+                  "PlayerAgent::action() start" );
 
     // check see synchronization
     if ( M_impl->see_state_.isSynch()
@@ -1756,7 +1811,7 @@ PlayerAgent::action()
          && world().seeTime() != M_impl->current_time_ )
     {
         dlog.addText( Logger::SYSTEM,
-                      "missed see synch. action without see" );
+                      __FILE__": missed see synch. action without see" );
         std::cout << world().teamName() << ' '
                   << world().self().unum() << ": "
                   << world().time()
@@ -1819,6 +1874,10 @@ PlayerAgent::action()
     // ------------------------------------------------------------------------
     // debugger output
     outputDebug();
+
+    dlog.addText( Logger::SYSTEM,
+                  "PlayerAgent::action() elapsed %lf {ms}",
+                  timer.elapsedReal() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1846,8 +1905,7 @@ PlayerAgent::doViewAction()
          && world().gameMode().type() != GameMode::PlayOn )
     {
         dlog.addText( Logger::SYSTEM,
-                      "%s:%d: not play_on. system must be adjust see synch..."
-                      ,__FILE__, __LINE__ );
+                      __FILE__": doViewAction() not play_on. system must be adjust see synch..." );
         return;
     }
 
@@ -1896,7 +1954,7 @@ PlayerAgent::adjustSeeSynchNormalMode()
     if ( M_impl->see_state_.isSynchedSeeCountNormalMode() )
     {
         dlog.addText( Logger::SYSTEM,
-                      "adjustSeeSynchNormalMode. see count is synch case" );
+                      __FILE__": adjustSeeSynchNormalMode. see count is synch case" );
         // set synchronized see timing
         M_impl->see_state_.setLastSeeTiming( SeeState::TIME_0_00 );
 
@@ -2076,19 +2134,35 @@ PlayerAgent::outputDebug()
                                        effector().getSayMessage().c_str() );
         }
 
-        // set defense line
-        M_debug_client.addLine( Vector2D( world().defenseLineX(),
-                                          world().self().pos().y - 2.0 ),
-                                Vector2D( world().defenseLineX(),
-                                          world().self().pos().y + 2.0 ) );
-        // set offside line
-        M_debug_client.addLine( Vector2D( world().offsideLineX(),
-                                          world().self().pos().y - 15.0 ),
-                                Vector2D( world().offsideLineX(),
-                                          world().self().pos().y + 15.0 ) );
         // compose worldmodel & some debug message
         // send to debug server or write to disc
         M_debug_client.writeAll( world() );
+
+
+        // print coordinate error of world model if fullstate information
+        // is available
+        if ( world().time().stopped() == 0
+             && fullstateWorld().valid() )
+        {
+            dlog.addText( Logger::WORLD,
+                          "SELF_ERROR r = %lf,"
+                          " wm=(%lf,%lf), fullstate=(%lf, %lf)",
+                          (world().self().pos()
+                           - fullstateWorld().self().pos()).r(),
+                          world().self().pos().x,
+                          world().self().pos().y,
+                          fullstateWorld().self().pos().x,
+                          fullstateWorld().self().pos().y );
+            dlog.addText( Logger::WORLD,
+                          "BALL_ERROR r = %lf,"
+                          " wm=(%lf,%lf), fullstate=(%lf, %lf)",
+                          (world().ball().pos()
+                           - fullstateWorld().ball().pos()).r(),
+                          world().ball().pos().x,
+                          world().ball().pos().y,
+                          fullstateWorld().ball().pos().x,
+                          fullstateWorld().ball().pos().y );
+        }
     }
     else
     {
@@ -2107,7 +2181,7 @@ PlayerAgent::doKick( const double & power,
     if ( ! world().self().isKickable() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doKick. but not kickable" );
+                      __FILE__": agent->doKick. but not kickable" );
         std::cerr << world().teamName() << ' '
                   << world().self().unum() << ": "
                   << world().time()
@@ -2117,7 +2191,7 @@ PlayerAgent::doKick( const double & power,
     if ( world().self().isFreezed() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doKick. but in tackle expire period  %d",
+                      __FILE__": agent->doKick. but in tackle expire period  %d",
                       world().self().tackleExpires() );
         std::cerr << world().teamName() << ' '
                   << world().self().unum() << ": "
@@ -2140,7 +2214,7 @@ PlayerAgent::doTurn( const AngleDeg & moment )
     if ( world().self().isFreezed() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doTurn. but in tackle expire period  %d",
+                      __FILE__": agent->doTurn. but in tackle expire period  %d",
                       world().self().tackleExpires() );
         std::cerr << world().teamName() << ' '
                   << world().self().unum() << ": "
@@ -2158,12 +2232,13 @@ PlayerAgent::doTurn( const AngleDeg & moment )
 
 */
 bool
-PlayerAgent::doDash( const double & power )
+PlayerAgent::doDash( const double & power,
+                     const AngleDeg & rel_dir )
 {
     if ( world().self().isFreezed() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doDash. but in tackle expire period  %d",
+                      __FILE__": agent->doDash. but in tackle expire period  %d",
                       world().self().tackleExpires() );
         std::cerr << world().teamName() << ' '
                   << world().self().unum() << ": "
@@ -2172,7 +2247,7 @@ PlayerAgent::doDash( const double & power )
         return false;
     }
 
-    M_effector.setDash( power );
+    M_effector.setDash( power, rel_dir );
     return true;
 }
 
@@ -2187,7 +2262,7 @@ PlayerAgent::doMove( const double & x,
     if ( world().self().isFreezed() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doMove. but in tackle expire period  %d",
+                      __FILE__": agent->doMove. but in tackle expire period  %d",
                       world().self().tackleExpires() );
         std::cerr << world().teamName() << ' '
                   << world().self().unum() << ": "
@@ -2211,7 +2286,7 @@ PlayerAgent::doMove( const double & x,
                   << " Can move only in before kickoff mode (or after goalie catch)"
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                      "agent->doMove. cannot move to (%.1f %.1f)",
+                      __FILE__": agent->doMove. cannot move to (%.1f %.1f)",
                       x, y );
         return false;
     }
@@ -2230,7 +2305,7 @@ PlayerAgent::doCatch()
     if ( world().self().isFreezed() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doCatch. refused. tackle expire period  %d",
+                      __FILE__": agent->doCatch. refused. tackle expire period  %d",
                       world().self().tackleExpires() );
         std::cerr << world().teamName() << ' '
                   << world().self().unum() << ": "
@@ -2246,7 +2321,7 @@ PlayerAgent::doCatch()
                   << world().time()
                   << " Only goalies can catch" << std::endl;
         dlog.addText( Logger::ACTION,
-                      "agent->doCatch. only goalie can catch" );
+                      __FILE__": agent->doCatch. only goalie can catch" );
         return false;
     }
 
@@ -2259,7 +2334,7 @@ PlayerAgent::doCatch()
                   << " not play_on mode, cannot catch"
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                      "agent->doCatch. playmode is not play_on" );
+                      __FILE__": agent->doCatch. playmode is not play_on" );
         return false;
     }
 
@@ -2270,7 +2345,7 @@ PlayerAgent::doCatch()
                   << world().time()
                   << " doCatch: ball is unknown." << std::endl;
         dlog.addText( Logger::ACTION,
-                      "Effector::setCatch. ball is unknown. rpos conf count = %d",
+                      __FILE__": Effector::setCatch. ball is unknown. rpos conf count = %d",
                       world().ball().rposCount() );
         return false;
     }
@@ -2288,13 +2363,13 @@ PlayerAgent::doTackle( const double & power_or_dir )
 {
     if ( world().self().isFreezed() )
     {
-        dlog.addText( Logger::ACTION,
-                      "agent->doTackle. refused. tackle expire period  %d",
-                      world().self().tackleExpires() );
         std::cerr << world().teamName() << ' '
                   << world().self().unum() << ": "
                   << world().time()
                   << " Now Tackle expire period" << std::endl;
+        dlog.addText( Logger::ACTION,
+                      __FILE__": agent->doTackle. refused. tackle expire period  %d",
+                      world().self().tackleExpires() );
         return false;
     }
 
@@ -2326,7 +2401,7 @@ PlayerAgent::doChangeView( const ViewWidth & width )
                                                    world().time() ) )
         {
             dlog.addText( Logger::ACTION,
-                          "agent->doChangeView. width(%d) will break see synch... ",
+                          __FILE__": agent->doChangeView. width(%d) will break see synch... ",
                           width.type() );
             return false;
         }
@@ -2336,17 +2411,16 @@ PlayerAgent::doChangeView( const ViewWidth & width )
         if ( world().gameMode().type() != GameMode::PlayOn )
         {
             dlog.addText( Logger::ACTION,
-                          "agent->doChangeView. no synch. not play on."
+                           __FILE__": agent->doChangeView. no synch. not play on."
                           " should try to adjust. " );
             return false;
         }
     }
 
-
-    if ( width == world().self().viewWidth() )
+    if ( width == M_effector.queuedNextViewWidth() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doChangeView. already same view mode %d",
+                       __FILE__": agent->doChangeView. already same view mode %d",
                       width.type() );
         return false;
     }
@@ -2399,7 +2473,7 @@ PlayerAgent::doPointto( const double & x,
     if ( world().self().armMovable() > 0 )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doPointto. now pointing and cannot move arm." );
+                       __FILE__": agent->doPointto. now pointing and cannot move arm." );
         return false;
     }
 
@@ -2410,7 +2484,7 @@ PlayerAgent::doPointto( const double & x,
                   << world().time()
                   << " doPointto : invalid localization" << std::endl;
         dlog.addText( Logger::ACTION,
-                      "Effector::setPointto. invalid self localization..." );
+                       __FILE__": Effector::setPointto. invalid self localization..." );
         return false;
     }
 
@@ -2428,7 +2502,7 @@ PlayerAgent::doPointtoOff()
     if ( world().self().armMovable() > 0 )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doPointtoOff. now pointing and cannot move arm." );
+                       __FILE__": agent->doPointtoOff. now pointing and cannot move arm." );
         return false;
     }
 
@@ -2451,7 +2525,7 @@ PlayerAgent::doAttentionto( SideID side,
                   << world().time()
                   << "  attentionto. invalid side " << side << std::endl;
         dlog.addText( Logger::ACTION,
-                      "agent->doAttentionto. Invalid side %d",
+                       __FILE__": agent->doAttentionto. Invalid side %d",
                       side );
         return false;
     }
@@ -2468,7 +2542,7 @@ PlayerAgent::doAttentionto( SideID side,
                   << world().time()
                   << "  attentionto. invalid unum " << unum << std::endl;
         dlog.addText( Logger::ACTION,
-                      "agent->doAttentionto. Invalid unum %d",
+                       __FILE__": agent->doAttentionto. Invalid unum %d",
                       unum );
         return false;
     }
@@ -2481,7 +2555,7 @@ PlayerAgent::doAttentionto( SideID side,
                   << world().time()
                   << "  attentionto. try to attention to itself " << std::endl;
         dlog.addText( Logger::ACTION,
-                      "agent->doAttentionto. try to attention to self." );
+                       __FILE__": agent->doAttentionto. try to attention to self." );
         return false;
     }
 
@@ -2489,7 +2563,7 @@ PlayerAgent::doAttentionto( SideID side,
          && world().self().attentiontoSide() == side )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->doAttentionto. already attended." );
+                       __FILE__": agent->doAttentionto. already attended." );
         return false;
     }
 
@@ -2515,7 +2589,14 @@ PlayerAgent::doAttentiontoOff()
 void
 PlayerAgent::setArmAction( ArmAction * act )
 {
-    M_impl->arm_action_ = boost::shared_ptr< ArmAction >( act );
+    if ( act )
+    {
+        M_impl->arm_action_ = boost::shared_ptr< ArmAction >( act );
+    }
+    else
+    {
+        M_impl->arm_action_.reset();
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -2525,7 +2606,14 @@ PlayerAgent::setArmAction( ArmAction * act )
 void
 PlayerAgent::setNeckAction( NeckAction * act )
 {
-    M_impl->neck_action_ = boost::shared_ptr< NeckAction >( act );
+    if ( act )
+    {
+        M_impl->neck_action_ = boost::shared_ptr< NeckAction >( act );
+    }
+    else
+    {
+        M_impl->neck_action_.reset();
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -2535,7 +2623,14 @@ PlayerAgent::setNeckAction( NeckAction * act )
 void
 PlayerAgent::setViewAction( ViewAction * act )
 {
-    M_impl->view_action_ = boost::shared_ptr< ViewAction >( act );
+    if ( act )
+    {
+        M_impl->view_action_ = boost::shared_ptr< ViewAction >( act );
+    }
+    else
+    {
+        M_impl->view_action_.reset();
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -2548,7 +2643,7 @@ PlayerAgent::addSayMessage( const SayMessage * message )
     if ( ! config().useCommunication() )
     {
         dlog.addText( Logger::ACTION,
-                      "agent->addSayMessage. communication is not allowed" );
+                       __FILE__": agent->addSayMessage. communication is not allowed" );
         return;
     }
 
