@@ -70,7 +70,7 @@ PlayerConfig::setDefaultParam()
 {
     // basic setting
     M_team_name = "HELIOS_base";
-    M_version = 9.4;
+    M_version = 13;
     M_reconnect_number = 0;
     M_goalie = false;
 
@@ -134,13 +134,16 @@ PlayerConfig::setDefaultParam()
     M_debug_action = false;
     M_debug_intercept = false;
     M_debug_kick = false;
+    M_debug_hold = false;
     M_debug_dribble = false;
     M_debug_pass = false;
     M_debug_cross = false;
     M_debug_shoot = false;
     M_debug_clear = false;
-    M_debug_team = false;
+    M_debug_block = false;
+    M_debug_mark = false;
     M_debug_role = false;
+    M_debug_team = false;
 }
 
 /*-------------------------------------------------------------------*/
@@ -167,7 +170,7 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "normal_view_time_thr", "", &M_normal_view_time_thr )
 
         ( "host", "h", &M_rcssserver_host )
-        ( "port", "", &M_rcssserver_port )
+        ( "port", "p", &M_rcssserver_port )
 
         ( "compression", "", &M_compression )
 
@@ -191,7 +194,7 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "player_face_count_thr", "", &M_player_face_count_thr )
 
         ( "config_dir", "", &M_config_dir )
-        ( "player_number", "n",  &M_player_number )
+        ( "player_number", "n",  &M_player_number, "specifies the player's position number (not a uniform number)." )
 
         ( "debug_connect", "", BoolSwitch( &M_debug_connect ) )
         ( "debug_server_host", "", &M_debug_server_host )
@@ -209,13 +212,16 @@ PlayerConfig::createParamMap( ParamMap & param_map )
         ( "debug_action", "", BoolSwitch( &M_debug_action ) )
         ( "debug_intercept", "", BoolSwitch( &M_debug_intercept ) )
         ( "debug_kick", "", BoolSwitch( &M_debug_kick ) )
+        ( "debug_hold", "", BoolSwitch( &M_debug_hold ) )
         ( "debug_dribble", "", BoolSwitch( &M_debug_dribble ) )
         ( "debug_pass", "", BoolSwitch( &M_debug_pass ) )
         ( "debug_cross", "", BoolSwitch( &M_debug_cross ) )
         ( "debug_shoot", "", BoolSwitch( &M_debug_shoot ) )
         ( "debug_clear", "", BoolSwitch( &M_debug_clear ) )
-        ( "debug_team", "", BoolSwitch( &M_debug_team ) )
+        ( "debug_block", "", BoolSwitch( &M_debug_block ) )
+        ( "debug_mark", "", BoolSwitch( &M_debug_mark ) )
         ( "debug_role", "", BoolSwitch( &M_debug_role ) )
+        ( "debug_team", "", BoolSwitch( &M_debug_team ) )
         ;
 }
 
