@@ -62,7 +62,7 @@ private:
     double M_dist_from_self; //!< distance from self
     AngleDeg M_angle_from_self; //!< angle from self global
 
-    int M_ghost_count; //!< ghost flag
+    int M_ghost_count; //!< count since this object was recognized as a ghost object.
 
     Vector2D M_rpos; //!< relative coordinate
     int M_rpos_count; //!< relative coordinate accuracy counter
@@ -70,7 +70,11 @@ private:
     AngleDeg M_pointto_angle; //!< global pointing angle
     int M_pointto_count; //!< pointing angle accuracy
 
+    bool M_kicked; //!< kicking state
+
     int M_tackle_count; //!< tackle info accuracy
+
+    double M_heard_stamina; //!< heard stamina
 public:
 
     /*!
@@ -172,6 +176,15 @@ public:
           return M_pointto_count;
       }
 
+    /*!
+      \brief get kicking state information
+      \return true if player performed the kick.
+     */
+    bool kicked() const
+      {
+          return M_kicked;
+      }
+
     /*!
       \brief get tackling status accuracy
       \return count from last observation
@@ -188,6 +201,16 @@ public:
     bool isTackling() const;
 
 
+    /*!
+      \brief get the heard stamina inf
+      \return heard stamina value
+     */
+    const
+    double & heardStamina() const
+      {
+          return M_heard_stamina;
+      }
+
     /*!
       \brief velify global position accuracy
       \return true if position has enough accuracy
@@ -247,13 +270,28 @@ public:
     void update();
 
     /*!
-      \brief set ghost flag
+      \brief increment ghost count
      */
     void setGhost()
       {
           ++M_ghost_count;
       }
 
+    /*!
+      \brief set player's team info
+      \param side player's team side
+      \param unum player's uniform number
+      \param goalie goalie flag
+     */
+    void setTeam( const SideID side,
+                  const int unum,
+                  const bool goalie )
+      {
+          M_side = side;
+          M_unum = unum;
+          M_goalie = goalie;
+      }
+
     /*!
       \brief update status using localized player info
       \param side analyzed side info
@@ -274,24 +312,31 @@ public:
 
     /*!
       \brief update staus using heard info
+      \param heard_side heard side info
       \param heard_unum heard uniform number
       \param goalie update goalie info, only if this value is true.
       \param heard_pos heard global position
     */
-    void updateByHear( const int heard_unum,
+    void updateByHear( const SideID heard_side,
+                       const int heard_unum,
                        const bool goalie,
                        const Vector2D & heard_pos );
 
     /*!
       \brief update staus using heard info
+      \param heard_side heard side info
       \param heard_unum heard uniform number
       \param goalie update goalie info, only if this value is true.
       \param heard_pos heard global position
+      \param heard_body heard body angle
+      \param heard_stamina heard stamina value
     */
-    void updateByHear( const int heard_unum,
+    void updateByHear( const SideID heard_side,
+                       const int heard_unum,
                        const bool goalie,
                        const Vector2D & heard_pos,
-                       const AngleDeg & heard_body );
+                       const AngleDeg & heard_body,
+                       const double & heard_stamina );
 
     /*!
       \brief update status related to other objects
@@ -310,11 +355,16 @@ public:
 
     ///////////////////////////////////////////////////////////////
     /*!
+      \class UpdateOp
       \brief functor to update
     */
     class UpdateOp
         : public std::unary_function< PlayerObject, void > {
     public:
+        /*!
+          \brief operation function
+          \param player reference to the updated player
+         */
         result_type operator()( argument_type & player )
           {
               player.update();
@@ -322,11 +372,17 @@ public:
     };
 
     /*!
+      \class IsInvalidOp
       \brief functor to check if player has enough accuracy
     */
     class IsInvalidOp
         : public std::unary_function< PlayerObject, bool > {
     public:
+        /*!
+          \brief operation function
+          \param player reference to the updated player.
+          \return true if player's position accuracy is low.
+         */
         result_type operator()( const argument_type & player ) const
           {
               return ( ! player.posValid() );
@@ -335,11 +391,18 @@ public:
 
     ///////////////////////////////////////////////////////////////
     /*!
+      \class CountCmp
       \brief predicate functor to compare player's accuracy. reference version
     */
     class CountCmp
         : public std::binary_function< PlayerObject, PlayerObject, bool > {
     public:
+        /*!
+          \brief operation function
+          \param lhs left hand side variable
+          \param rhs right hand side variable
+          \return compared result
+         */
         result_type operator()( const first_argument_type & lhs,
                                 const second_argument_type & rhs ) const
           {
@@ -350,11 +413,18 @@ public:
     };
 
     /*!
+      \class PtrCountCmp
       \brief predicate functor to compare player's accuracy. pointer version
     */
     class PtrCountCmp
         : public std::binary_function< PlayerObject *, PlayerObject *, bool > {
     public:
+        /*!
+          \brief operation function
+          \param lhs left hand side variable
+          \param rhs right hand side variable
+          \return compared result
+         */
         result_type operator()( const first_argument_type lhs,
                                 const second_argument_type rhs ) const
           {
@@ -366,11 +436,18 @@ public:
 
     ///////////////////////////////////////////////////////////////
     /*!
+      \class PtrSelfDistCmp
       \brief predicate functor to compare player's distance from self
     */
     class PtrSelfDistCmp
         : public std::binary_function< PlayerObject *, PlayerObject *, bool > {
     public:
+        /*!
+          \brief operation function
+          \param lhs left hand side variable
+          \param rhs right hand side variable
+          \return compared result
+         */
         result_type operator()( const first_argument_type lhs,
                                 const second_argument_type rhs ) const
           {
@@ -379,11 +456,18 @@ public:
     };
 
     /*!
+      \class PtrBallDstCmp
       \brief predicate functor to compare player's distance from ball
     */
     class PtrBallDistCmp
         : public std::binary_function< PlayerObject *, PlayerObject *, bool > {
     public:
+        /*!
+          \brief operation function
+          \param lhs left hand side variable
+          \param rhs right hand side variable
+          \return compared result
+         */
         result_type operator()( const first_argument_type lhs,
                                 const second_argument_type rhs ) const
           {
