@@ -114,7 +114,7 @@ public:
     /*!
       \brief construct with raw information
       \param ball_pos ball position to be encoded
-      \param ball_pos ball velocity to be encoded
+      \param ball_vel ball velocity to be encoded
     */
     BallMessage( const Vector2D & ball_pos,
                  const Vector2D & ball_vel )
@@ -182,8 +182,10 @@ public:
 
     /*!
       \brief construct with raw information
-      \param receiver_number pass receiver's uniform number
+      \param receiver_unum pass receiver's uniform number
       \param receive_point desired pass receive point
+      \param ball_pos next ball position
+      \param ball_vel next ball velocity
     */
     PassMessage( const int receiver_unum,
                  const Vector2D & receive_point,
@@ -324,6 +326,7 @@ public:
       \brief construct with raw information
       \param goalie_unum goalie's uniform number
       \param goalie_pos goalie's global position
+      \param goalie_body goalie's body direction
     */
     GoalieMessage( const int goalie_unum,
                    const Vector2D & goalie_pos,
@@ -880,7 +883,7 @@ public:
   \brief one player info message encoder
 
   format:
-  "1<unum_pos_body:4>"
+  "P<unum_pos_body:4>"
   the length of message == 5
 */
 class OnePlayerMessage
@@ -943,6 +946,76 @@ public:
     bool toStr( std::string & to ) const;
 };
 
+
+/*-------------------------------------------------------------------*/
+/*!
+  \class SelfMessage
+  \brief self info message encoder
+
+  format:
+  "S<pos_body_stamina:4>"
+  the length of message == 5
+*/
+class SelfMessage
+    : public SayMessage {
+private:
+
+    Vector2D M_self_pos; //!< self position
+    AngleDeg M_self_body; //!< self body angle
+    double M_self_stamina; //!< self stamina rate [0,1]
+
+public:
+
+    /*!
+      \brief construct with raw information
+      \param self_pos self global position
+      \param self_body self body angle
+      \param self_stamina self stamina value
+    */
+    SelfMessage( const Vector2D & self_pos,
+                 const AngleDeg & self_body,
+                 const double & self_stamina )
+        : M_self_pos( self_pos )
+        , M_self_body( self_body )
+        , M_self_stamina( self_stamina )
+      { }
+
+    /*!
+      \brief get the header character of this message
+      \return header character of this message
+     */
+    char header() const
+      {
+          return SelfMessageParser::sheader();
+      }
+
+    /*!
+      \brief get the length of this message.
+      \return the length of encoded message
+    */
+    static
+    std::size_t slength()
+      {
+          return SelfMessageParser::slength();
+      }
+
+    /*!
+      \brief get the length of this message
+      \return the length of encoded message
+    */
+    std::size_t length() const
+      {
+          return slength();
+      }
+
+    /*!
+      \brief append this info to the audio message
+      \param to reference to the message string instance
+      \return result status of encoding
+    */
+    bool toStr( std::string & to ) const;
+};
+
 /*-------------------------------------------------------------------*/
 /*!
   \class BallPlayerMessage
