@@ -46,8 +46,8 @@
 #include <rcsc/soccer_math.h>
 #include <rcsc/math_util.h>
 
-//#define DEBUG1
-//#define DEBUG
+#define DEBUG1
+#define DEBUG
 
 namespace rcsc {
 
@@ -64,11 +64,13 @@ SelfIntercept::predict( const int max_cycle,
 {
 #ifdef DEBUG
     dlog.addText( Logger::INTERCEPT,
-                  "%s:%d: ------------- predict self ---------------"
-                  ,__FILE__, __LINE__ );
+                  __FILE__": ------------- predict self ---------------" );
 #endif
     predictOneStep( self_cache );
-    predictLongStep( max_cycle, self_cache );
+    predictLongStep( max_cycle, true, self_cache );
+#ifdef SELF_INTERCEPT_USE_NO_SAVE_RECEVERY
+    predictLongStep( max_cycle, false, self_cache );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -101,7 +103,6 @@ SelfIntercept::predictOneStep( std::vector< InterceptInfo > & self_cache ) const
         return;
     }
 
-
     // get next ball relative pos, and set angle relative to my body angle
     Vector2D next_ball_rel = ball_next;
     next_ball_rel -= M_world.self().pos();
@@ -118,7 +119,8 @@ SelfIntercept::predictOneStep( std::vector< InterceptInfo > & self_cache ) const
     ///////////////////////////////////////////////////////////
     // if Y difference is over control_area,
     // I never reach the ball even if max power dash
-    if ( next_ball_rel.absY() > control_area - 0.2 )
+    const double y_buf = ( goalie_mode ? 0.15 : 0.1 );
+    if ( next_ball_rel.absY() > control_area - y_buf )
     {
 #ifdef DEBUG
         dlog.addText( Logger::INTERCEPT,
@@ -154,9 +156,9 @@ SelfIntercept::predictNoDash( const bool goalie_mode,
                               const Vector2D & next_ball_rel,
                               std::vector< InterceptInfo > & self_cache ) const
 {
-    const double next_ball_dist = next_ball_rel.r();
+    double next_ball_dist = next_ball_rel.r();
 
-    double ball_noise = M_world.ball().vel().r() * ServerParam::i().ballRand() * 1.4;
+    const double ball_noise = M_world.ball().vel().r() * ServerParam::i().ballRand();
 
     // over control area
     if ( next_ball_dist > control_area - 0.15 - ball_noise )
@@ -176,32 +178,40 @@ SelfIntercept::predictNoDash( const bool goalie_mode,
         return true;
     }
 
-    // normal player
+    const double collide_dist
+        = M_world.self().playerType().playerSize()
+        + ServerParam::i().ballSize();
 
-    // check collision.
-    if ( next_ball_dist < ( M_world.self().playerType().playerSize()
-                            + ServerParam::i().ballSize()
-                            + 0.15 ) )
-    {
-        // maybe collision ...
-        dlog.addText( Logger::INTERCEPT,
-                      "____No dash kickable: but may cause collision next_dist = %f",
-                      next_ball_dist );
-        return false;
-    }
+//     // check collision.
+//     if ( next_ball_dist < collide_dist + 0.15 )
+//     {
+//         // maybe collision ...
+//         dlog.addText( Logger::INTERCEPT,
+//                       "____No dash kickable: but may cause collision next_dist = %f",
+//                       next_ball_dist );
+//         return false;
+//     }
 
     // check kick rate
     Vector2D next_ball_vel
         = M_world.ball().vel()
         * ServerParam::i().ballDecay();
+    Vector2D ball_rel = next_ball_rel;
 
-    double kickrate
+    if ( next_ball_dist < collide_dist )
+    {
+        next_ball_dist = std::max( collide_dist, next_ball_dist );
+        ball_rel.setLength( next_ball_dist );
+        next_ball_vel *= -0.1;
+    }
+
+    double kick_rate
         = M_world.self().playerType().kickRate( ServerParam::i(),
                                                 next_ball_dist,
-                                                next_ball_rel.th().degree() );
+                                                ball_rel.th().degree() );
 
-    if ( ServerParam::i().maxPower() * kickrate
-         <= next_ball_vel.r() * ServerParam::i().ballDecay() * 1.01 )
+    if ( ServerParam::i().maxPower() * kick_rate
+         <= next_ball_vel.r() * ServerParam::i().ballDecay() * 1.1 )
     {
         // it has possibility that player cannot stop the ball
         dlog.addText( Logger::INTERCEPT,
@@ -232,26 +242,26 @@ SelfIntercept::predictOneDash( const double & control_area,
 {
     // check possible max accel
     // possible forward dash accel distance
-    double max_forward_accel_x
-        = ServerParam::i().maxPower()
-        * M_world.self().dashRate();
+    double max_forward_accel_x = ServerParam::i().maxDashPower() * M_world.self().dashRate();
     M_world.self().playerType().normalizeAccel( M_world.self().vel(),
                                                 M_world.self().body(),
                                                 &max_forward_accel_x );
 
-    if ( max_forward_accel_x + control_area - 0.2 < next_ball_rel.x )
+    if ( max_forward_accel_x + control_area - 0.15 < next_ball_rel.x )
     {
 #ifdef DEBUG
         dlog.addText( Logger::INTERCEPT,
                       "____1 dash: ball is front but over dash capacity."
-                      "  max_forward_accel = %f",
-                      max_forward_accel_x );
+                      "  max_forward_accel=%.3f control_buf=%.3f ball_x=%.3f",
+                      max_forward_accel_x,
+                      max_forward_accel_x + control_area - 0.15,
+                      next_ball_rel.x );
 #endif
         return;
     }
 
     // possible backward dash accel distance
-    double max_back_accel_x = max_forward_accel_x;
+    double max_back_accel_x = std::fabs( ServerParam::i().minDashPower() * M_world.self().dashRate() );
     // normalize accel magnitude
     M_world.self().playerType().normalizeAccel( M_world.self().vel(),
                                                 M_world.self().body() + 180.0,
@@ -259,34 +269,32 @@ SelfIntercept::predictOneDash( const double & control_area,
     max_back_accel_x *= -1.0;
 
     ///////////////////////////////////////////////////////////
-    if ( next_ball_rel.x < max_back_accel_x - control_area - 0.2 )
+    if ( next_ball_rel.x < max_back_accel_x - control_area - 0.15 )
 
     {
         // over the reachable distance
 #ifdef DEBUG
         dlog.addText( Logger::INTERCEPT,
                       "____1 dash: ball is back but over dash capacity."
-                      "  max_back_accel = %f",
-                      max_back_accel_x );
+                      "  max_back_accel=%.3f control_buf=%.3f ball_x=%.3f",
+                      max_back_accel_x,
+                      max_back_accel_x - control_area - 0.15,
+                      next_ball_rel.x );
 #endif
         return;
     }
 
     const double safety_power_forward
         = min_max( 0.0,
-                   M_world.self().stamina()
-                   - ServerParam::i().recoverDecThrValue()
-                   - 5.0,
-                   ServerParam::i().maxPower() );
+                   M_world.self().stamina() - ServerParam::i().recoverDecThrValue() - 5.0,
+                   ServerParam::i().maxDashPower() );
     double safety_forward_accel_x
         = std::min( max_forward_accel_x,
                     safety_power_forward * M_world.self().dashRate() );
     const double safety_power_back
         = min_max( 0.0,
-                   M_world.self().stamina()
-                   - ServerParam::i().recoverDecThrValue()
-                   - 5.0,
-                   2.0 * ServerParam::i().maxPower() );
+                   M_world.self().stamina() - ServerParam::i().recoverDecThrValue() - 5.0,
+                   2.0 * - ServerParam::i().minDashPower() );
     double safety_back_accel_x
         = std::max( max_back_accel_x,
                     -0.5 * safety_power_back * M_world.self().dashRate() );
@@ -316,7 +324,7 @@ SelfIntercept::predictOneDash( const double & control_area,
         return;
     }
 
-    // second, check maximal accel dash
+    // second, check max accel dash
     if ( predictOneDashAdjustX( control_area,
                                 next_ball_rel,
                                 max_back_accel_x,
@@ -373,8 +381,9 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
                       dash_power );
 #endif
     }
+
     /////////////////////////////////////////////////////
-    // x difference is longer than acccelx. needed to check ball dist after dash
+    // x difference is bigger than acccelx. need to check ball dist after dash
     // ball is front
     if ( dash_power < -999.0
          && forward_accel_x < next_ball_rel.x )
@@ -393,7 +402,8 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
 #endif
         }
     }
-    // x difference is longer than acccelx. needed to check ball dist after dash
+
+    // x difference is bigger than acccelx. need to check ball dist after dash
     // ball is back
     if ( dash_power < -999.0
          && next_ball_rel.x < back_accel_x )
@@ -460,25 +470,31 @@ SelfIntercept::predictOneDashAdjustX( const double & control_area,
 
 */
 double
-SelfIntercept::getOneStepBestTrapPower( const double & control_area,
+SelfIntercept::getOneStepBestTrapPower( const double & /*control_area*/,
                                         const Vector2D & next_ball_rel,
                                         const double & max_back_accel_x,
                                         const double & max_forward_accel_x ) const
 {
-    const double best_ctrl_dist
+    const double best_ctrl_dist_forward
         = M_world.self().playerType().playerSize()
-        + 0.5 * ( M_world.self().playerType().kickableMargin()
-                  + ServerParam::i().ballSize() );
+        + 0.5 * M_world.self().playerType().kickableMargin()
+        + ServerParam::i().ballSize();
+    const double best_ctrl_dist_backward
+        = M_world.self().playerType().playerSize()
+        + 0.3 * M_world.self().playerType().kickableMargin()
+        + ServerParam::i().ballSize();
 
 #ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
-                  "________1 dash Best Trap. BestCtrlDist = %.3f, next_ball_y = %.3f",
-                  best_ctrl_dist, next_ball_rel.y );
+                  "________1 dash Best Trap. best_ctrl_dist_f=%.3f best_ctrl_dist_b=%.3f next_ball_y=%.3f",
+                  best_ctrl_dist_forward,
+                  best_ctrl_dist_backward,
+                  next_ball_rel.y );
 #endif
 
     // Y diff is longer than best distance.
     // just put the ball on player's side
-    if ( next_ball_rel.absY() > best_ctrl_dist )
+    if ( next_ball_rel.absY() > best_ctrl_dist_forward )
     {
 #ifdef DEBUG1
         dlog.addText( Logger::INTERCEPT,
@@ -487,16 +503,64 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
         return next_ball_rel.x / M_world.self().dashRate();
     }
 
+#if 1
+
+    const double forward_trap_accel_x
+        = next_ball_rel.x
+        - std::sqrt( std::pow( best_ctrl_dist_forward, 2.0 )
+                     - std::pow( next_ball_rel.y, 2.0 ) );
+    const double backward_trap_accel_x
+        = next_ball_rel.x
+        + std::sqrt( std::pow( best_ctrl_dist_backward, 2.0 )
+                     - std::pow( next_ball_rel.y, 2.0 ) );
+
+    double best_accel_x = 10000.0;
+    double min_power = 10000.0;
+
+    const double x_step = ( backward_trap_accel_x - forward_trap_accel_x ) / 5.0;
+    for ( double accel_x = forward_trap_accel_x;
+          accel_x < backward_trap_accel_x + 0.01;
+          accel_x += x_step )
+    {
+        if ( ( accel_x >= 0.0
+               && max_forward_accel_x > accel_x )
+             || ( accel_x < 0.0
+                  && max_back_accel_x < accel_x )
+             )
+        {
+            double power = accel_x / M_world.self().dashRate();
+            if ( std::fabs( power ) < std::fabs( min_power ) )
+            {
+                best_accel_x = accel_x;
+                min_power = power;
+            }
+        }
+    }
+
+    if ( min_power < 1000.0 )
+    {
+#ifdef DEBUG1
+        dlog.addText( Logger::INTERCEPT,
+                      "________1 dash Best Trap.  best trap."
+                      " accel_x=%.3f power==%.3f",
+                      best_accel_x,
+                      min_power );
+#endif
+        return min_power;
+    }
+
+#else
+
     double required_accel_x[2];
     // case  put ball front
     required_accel_x[0]
         = next_ball_rel.x
-        - std::sqrt( std::pow( best_ctrl_dist, 2.0 )
+        - std::sqrt( std::pow( best_ctrl_dist_forward, 2.0 )
                      - std::pow( next_ball_rel.y, 2.0 ) );
     // case put ball back
     required_accel_x[1]
         = next_ball_rel.x
-        + std::sqrt( std::pow( best_ctrl_dist - 0.1, 2.0 )
+        + std::sqrt( std::pow( best_ctrl_dist_backward, 2.0 )
                      - std::pow( next_ball_rel.y, 2.0 ) );
 
     if ( M_world.self().body().abs() < 45.0 )
@@ -545,6 +609,9 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
             return required_accel_x[i] / M_world.self().dashRate();
         }
     }
+
+#endif
+
 #ifdef DEBUG1
     dlog.addText( Logger::INTERCEPT,
                   "________1 dash Best Trap. Failed" );
@@ -559,8 +626,11 @@ SelfIntercept::getOneStepBestTrapPower( const double & control_area,
 */
 void
 SelfIntercept::predictLongStep( const int max_cycle,
+                                const bool save_recovery,
                                 std::vector< InterceptInfo > & self_cache ) const
 {
+    const ServerParam & param = ServerParam::i();
+
     // calc Y distance from ball line
     Vector2D ball_to_self = M_world.self().pos() - M_world.ball().pos();
     ball_to_self.rotate( - M_world.ball().vel().th() );
@@ -586,16 +656,16 @@ SelfIntercept::predictLongStep( const int max_cycle,
         = inertia_n_step_point( M_world.ball().pos(),
                                 M_world.ball().vel(),
                                 min_cycle - 1,
-                                ServerParam::i().ballDecay() );
+                                param.ballDecay() );
     Vector2D ball_vel = M_world.ball().vel()
-        * std::pow( ServerParam::i().ballDecay(), min_cycle - 1 );
+        * std::pow( param.ballDecay(), min_cycle - 1 );
     bool found = false;
 
     int max_loop = max_cycle;
     for ( int cycle = min_cycle; cycle < max_loop; ++cycle )
     {
         ball_pos += ball_vel;
-        ball_vel *= ServerParam::i().ballDecay();
+        ball_vel *= param.ballDecay();
 #ifdef DEBUG
         dlog.addText( Logger::INTERCEPT,
                       "__cycle %d: bpos(%.3f, %.3f) bvel(%.3f, %.3f)",
@@ -604,17 +674,18 @@ SelfIntercept::predictLongStep( const int max_cycle,
                       ball_vel.x, ball_vel.y );
 
 #endif
-        // ball is stopped
-        if ( ball_vel.r2() < 0.01 )
-        {
-#ifdef DEBUG
-            dlog.addText( Logger::INTERCEPT,
-                          "____ball speed reach ZERO" );
-#endif
-            break;;
-        }
-        if ( ball_pos.absX() > ServerParam::i().pitchHalfLength() + 10.0
-             || ball_pos.absY() > ServerParam::i().pitchHalfWidth() + 10.0
+//         // ball is stopped
+//         if ( ball_vel.r2() < 0.0001 )
+//         {
+// #ifdef DEBUG
+//             dlog.addText( Logger::INTERCEPT,
+//                           "____ball speed reach ZERO" );
+// #endif
+//             break;
+//         }
+
+        if ( ball_pos.absX() > param.pitchHalfLength() + 10.0
+             || ball_pos.absY() > param.pitchHalfWidth() + 10.0
              )
         {
 #ifdef DEBUG
@@ -623,13 +694,14 @@ SelfIntercept::predictLongStep( const int max_cycle,
 #endif
             break;
         }
-        const bool goalie_mode =
-            ( M_world.self().goalie()
-              && ball_pos.x < ServerParam::i().ourPenaltyAreaLineX()
-              && ball_pos.absY() < ServerParam::i().penaltyAreaHalfWidth()
-              );
+
+        const bool goalie_mode
+            = ( M_world.self().goalie()
+                && ball_pos.x < param.ourPenaltyAreaLineX()
+                && ball_pos.absY() < param.penaltyAreaHalfWidth()
+                );
         const double control_area = ( goalie_mode
-                                      ? ServerParam::i().catchableArea()
+                                      ? param.catchableArea()
                                       : M_world.self().playerType().kickableArea() );
 
         ///////////////////////////////////////////////////////////
@@ -644,29 +716,30 @@ SelfIntercept::predictLongStep( const int max_cycle,
 #endif
             continue;
         }
+
         ///////////////////////////////////////////////////////////
         bool back_dash = false;
-        bool save_recovery = true;
         int n_turn = 0;
+        double result_recovery = 0.0;
         if ( canReachAfterTurnDash( cycle,
                                     ball_pos,
                                     control_area,
-                                    &n_turn, &back_dash, &save_recovery ) )
+                                    save_recovery, // save recovery
+                                    &n_turn, &back_dash, &result_recovery ) )
         {
 #ifdef DEBUG
             dlog.addText( Logger::INTERCEPT,
-                          " --> can reach. cycle = %d, turn = %d",
-                          cycle, n_turn);
-            if ( ! save_recovery )
-            {
-                dlog.addText( Logger::INTERCEPT,
-                              "____But  cannot save recovery" );
-
-            }
+                          " --> can reach. cycle=%d, turn=%d, back=%c recovery=%f",
+                          cycle, n_turn,
+                          ( back_dash ? 'b' : 'f' ),
+                          result_recovery );
 #endif
-            double dash_power = ServerParam::i().maxPower();
-            if ( back_dash ) dash_power = -dash_power;
-            self_cache.push_back( InterceptInfo( ( save_recovery
+            //double dash_power = ServerParam::i().maxDashPower();
+            //if ( back_dash ) dash_power = -dash_power;
+            double dash_power = ( back_dash
+                                  ? ServerParam::i().minDashPower()
+                                  : ServerParam::i().maxDashPower() );
+            self_cache.push_back( InterceptInfo( ( result_recovery >= M_world.self().recovery()
                                                    ? InterceptInfo::NORMAL
                                                    : InterceptInfo::EXHAUST ),
                                                  n_turn,
@@ -682,12 +755,12 @@ SelfIntercept::predictLongStep( const int max_cycle,
     }
 
     // not registered any interception
-    if ( ! found )
+    if ( ! found
+         && save_recovery )
     {
         dlog.addText( Logger::INTERCEPT,
-                      "%s: SelfIntercept. failed to predict? register ball final point",
-                      __FILE__ );
-        predictFinal( self_cache );
+                      __FILE__": SelfIntercept. failed to predict? register ball final point" );
+        predictFinal( max_cycle, self_cache );
     }
 }
 
@@ -696,13 +769,12 @@ SelfIntercept::predictLongStep( const int max_cycle,
 
 */
 void
-SelfIntercept::predictFinal( std::vector< InterceptInfo > & self_cache ) const
+SelfIntercept::predictFinal( const int max_cycle,
+                             std::vector< InterceptInfo > & self_cache ) const
 
 {
-    Vector2D ball_final_pos
-        = inertia_final_point( M_world.ball().pos(),
-                               M_world.ball().vel(),
-                               ServerParam::i().ballDecay() );
+    const Vector2D my_final_pos = M_world.self().inertiaPoint( 100 );
+    const Vector2D ball_final_pos = M_world.ball().inertiaPoint( 100 );
     const bool goalie_mode =
         ( M_world.self().goalie()
           && ball_final_pos.x < ServerParam::i().ourPenaltyAreaLineX()
@@ -713,22 +785,34 @@ SelfIntercept::predictFinal( std::vector< InterceptInfo > & self_cache ) const
                                   : M_world.self().playerType().kickableArea() );
 
     AngleDeg dash_angle = M_world.self().body();
-    bool back_dash; // dummy
+    bool back_dash = false; // dummy
     int n_turn = predictTurnCycle( 100,
                                    ball_final_pos,
                                    control_area,
                                    &dash_angle, &back_dash );
-    double dash_dist = M_world.self().pos().dist( ball_final_pos );
+    double dash_dist = my_final_pos.dist( ball_final_pos );
+    dash_dist -= control_area;
     int n_dash = M_world.self().playerType().cyclesToReachDistance( dash_dist );
 
 #ifdef DEBUG
     dlog.addText( Logger::INTERCEPT,
-                  "__Final. register ball final point. turn=%d, dash=%d",
+                  "__Final. register ball final point. max_cycle=%d, turn=%d, dash=%d",
+                  max_cycle,
                   n_turn, n_dash );
 #endif
+    if ( max_cycle > n_turn + n_dash )
+    {
+        n_dash = max_cycle - n_turn;
+#ifdef DEBUG
+        dlog.addText( Logger::INTERCEPT,
+                      "__Final(2) dash step is changed by max_cycle. max=%d turn=%d dash=%d",
+                      max_cycle,
+                      n_turn, n_dash );
+#endif
+    }
     self_cache.push_back( InterceptInfo( InterceptInfo::NORMAL,
                                          n_turn, n_dash,
-                                         ServerParam::i().maxPower() ) );
+                                         ServerParam::i().maxDashPower() ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -739,12 +823,12 @@ bool
 SelfIntercept::canReachAfterTurnDash( const int cycle,
                                       const Vector2D & ball_pos,
                                       const double & control_area,
+                                      const bool save_recovery,
                                       int * n_turn,
                                       bool * back_dash,
-                                      bool * save_recovery ) const
+                                      double * result_recovery ) const
 {
     AngleDeg dash_angle = M_world.self().body();
-
     *n_turn = predictTurnCycle( cycle,
                                 ball_pos,
                                 control_area,
@@ -754,9 +838,55 @@ SelfIntercept::canReachAfterTurnDash( const int cycle,
         return false;
     }
 
+    //
+    // backward dash
+    //
+
+    if ( *back_dash )
+    {
+#ifdef DEBUG
+        dlog.addText( Logger::INTERCEPT,
+                      "____ check back dash" );
+#endif
+        if ( canReachAfterDash( *n_turn, std::max( 0, cycle - (*n_turn) ),
+                                ball_pos, control_area,
+                                save_recovery,
+                                dash_angle, *back_dash,
+                                result_recovery ) )
+        {
+#ifdef DEBUG
+            dlog.addText( Logger::INTERCEPT,
+                          "____ back dash succeeded" );
+#endif
+            return true;
+        }
+        else
+        {
+#ifdef DEBUG
+            dlog.addText( Logger::INTERCEPT,
+                          "____ back dash failed!" );
+#endif
+            *back_dash = false;
+            dash_angle = M_world.self().body();
+            *n_turn = predictTurnCycle( cycle,
+                                        ball_pos,
+                                        control_area,
+                                        &dash_angle,
+                                        static_cast< bool * >( 0 ) );
+        }
+    }
+
+
+    //
+    // forward dash
+    //
+
     return canReachAfterDash( *n_turn, std::max( 0, cycle - (*n_turn) ),
                               ball_pos, control_area,
-                              dash_angle, *back_dash, save_recovery );
+                              save_recovery,
+                              dash_angle,
+                              false, // no back dash
+                              result_recovery );
 }
 
 /*-------------------------------------------------------------------*/
@@ -822,7 +952,8 @@ SelfIntercept::predictTurnCycle( const int cycle,
     */
     ///////////////////////////////////////////////////
     // check back dash possibility
-    if ( canBackDashChase( cycle, target_dist, angle_diff ) )
+    if ( back_dash
+         && canBackDashChase( cycle, target_dist, angle_diff ) )
     {
         *back_dash = true;
         *dash_angle += 180.0;
@@ -869,7 +1000,7 @@ SelfIntercept::predictTurnCycle( const int cycle,
 */
 bool
 SelfIntercept::canBackDashChase( const int cycle,
-                                 const double & target_dist,
+                                 const double & /*target_dist*/,
                                  const double & angle_diff ) const
 {
     ///////////////////////////////////////////////
@@ -892,16 +1023,27 @@ SelfIntercept::canBackDashChase( const int cycle,
         }
     }
 #else
-    if ( ! M_world.self().goalie() && cycle - 1 >= 6 )
+    if ( ! M_world.self().goalie() && cycle >= 5 )
     {
         return false;
     }
+
+    if ( M_world.self().goalie()
+         && cycle >= 5 )
+    {
+        Vector2D goal( - ServerParam::i().pitchHalfLength(), 0.0 );
+        Vector2D bpos = M_world.ball().inertiaPoint( cycle );
+        if ( goal.dist( bpos ) > 21.0 )
+        {
+            return false;
+        }
+    }
 #endif
 
     ///////////////////////////////////////////////
     // check stamina threshold
     // consumed stamina by one step
-    double total_consume = ServerParam::i().maxPower() * 2.0 * cycle;
+    double total_consume = - ServerParam::i().minDashPower() * 2.0 * cycle;
     double total_recover
         = M_world.self().playerType().staminaIncMax() * M_world.self().recovery()
         * ( cycle - 1 );
@@ -939,23 +1081,28 @@ SelfIntercept::canReachAfterDash( const int n_turn,
                                   const int n_dash,
                                   const Vector2D & ball_pos,
                                   const double & control_area,
+                                  const bool save_recovery,
                                   const AngleDeg & dash_angle,
                                   const bool back_dash,
-                                  bool * save_recovery ) const
+                                  double * result_recovery ) const
 {
     static const double PLAYER_NOISE_RATE
         = ( 1.0 - ServerParam::i().playerRand() * 0.25 );
-    static const double MAX_POWER = ServerParam::i().maxPower();
+    static const double MAX_POWER = ( back_dash
+                                      ? std::fabs( ServerParam::i().minDashPower() )
+                                      : std::fabs( ServerParam::i().maxDashPower() ) );
 
+    const ServerParam & param = ServerParam::i();
     const PlayerType & my_type = M_world.self().playerType();
 
+    const double recover_dec_thr = param.recoverDecThr() * param.staminaMax();
     //////////////////////////////////////////////////////////////
     const AngleDeg dash_angle_minus = -dash_angle;
     const Vector2D ball_rel
         = ( ball_pos - M_world.self().pos() ).rotatedVector( dash_angle_minus );
     const double ball_noise
         = M_world.ball().pos().dist( ball_pos )
-        * ServerParam::i().ballRand()
+        * param.ballRand()
         * 0.5;
     const double noised_ball_x = ball_rel.x + ball_noise;
 
@@ -976,7 +1123,7 @@ SelfIntercept::canReachAfterDash( const int n_turn,
     double tmp_stamina = M_world.self().stamina();
     double tmp_effort = M_world.self().effort();
     double tmp_recovery = M_world.self().recovery();
-    my_type.predictStaminaAfterWait( ServerParam::i(),
+    my_type.predictStaminaAfterWait( param,
                                      n_turn,
                                      &tmp_stamina,
                                      &tmp_effort,
@@ -992,9 +1139,11 @@ SelfIntercept::canReachAfterDash( const int n_turn,
         * my_type.dashRate( tmp_effort );
 #ifdef DEBUG
     dlog.addText( Logger::INTERCEPT,
-                  "______Try %d turn: %d dash:"
+                  "______Try: turn=%d, dash=%d :"
+                  " (%s)power=%.1f "
                   " angle=%.1f first_accel=%.2f first_vel=(%.2f %.2f)",
                   n_turn, n_dash,
+                  ( back_dash ? "backward" : "forward" ), MAX_POWER,
                   dash_angle.degree(), dash_accel_x, tmp_vel.x, tmp_vel.y );
 #endif
 
@@ -1005,7 +1154,9 @@ SelfIntercept::canReachAfterDash( const int n_turn,
     {
         /////////////////////////////////////////////////////////
         // update dash power & accel
-        double available_power = tmp_stamina + my_type.extraStamina();
+        double available_power = ( save_recovery
+                                   ? std::max( 0.0, tmp_stamina - recover_dec_thr )
+                                   : tmp_stamina + my_type.extraStamina() );
         if ( back_dash ) available_power *= 0.5;
         available_power = min_max( 0.0, available_power, MAX_POWER );
 
@@ -1082,7 +1233,7 @@ SelfIntercept::canReachAfterDash( const int n_turn,
             double real_power = dash_power_abs;
             if ( back_dash ) real_power *= -1.0;
             int n_safety_dash
-                = my_type.getMaxDashCyclesSavingStamina( ServerParam::i(),
+                = my_type.getMaxDashCyclesSavingStamina( param,
                                                          real_power,
                                                          tmp_stamina,
                                                          tmp_recovery );
@@ -1108,44 +1259,38 @@ SelfIntercept::canReachAfterDash( const int n_turn,
         tmp_vel *= my_type.playerDecay();
         /////////////////////////////////////////////////////////
         // update stamina
-        my_type.predictStaminaAfterDash( ServerParam::i(),
-                                         dash_power_abs * (back_dash ? -1.0 : 1.0),
-                                         &tmp_stamina, &tmp_effort, &tmp_recovery );
+
+        my_type.predictStaminaAfterOneDash( param,
+                                            dash_power_abs * ( back_dash ? -1.0 : 1.0 ),
+                                            &tmp_stamina, &tmp_effort, &tmp_recovery );
 
         /////////////////////////////////////////////////////////
         // check run over
         // it is not necessary to consider about Y difference,
         // because dash_angle is corrected for ball_reach_point
-        if ( tmp_pos.x * PLAYER_NOISE_RATE > noised_ball_x )
+        //if ( tmp_pos.x * PLAYER_NOISE_RATE > noised_ball_x )
+        if ( tmp_pos.x * PLAYER_NOISE_RATE + 0.1 > noised_ball_x )
         {
 #ifdef DEBUG
             dlog.addText( Logger::INTERCEPT,
                           "____dash %d/%d: can run over. rel_move_pos=(%.2f, %.2f)"
-                          " ball_x=%f over=%f y_diff=%f",
+                          " ball_x=%.3f over=%.3f y_diff=%.3f",
                           i, n_dash,
                           tmp_pos.x, tmp_pos.y,
                           noised_ball_x,
-                          tmp_pos.x * PLAYER_NOISE_RATE - noised_ball_x,
+                          //tmp_pos.x * PLAYER_NOISE_RATE - noised_ball_x,
+                          tmp_pos.x * PLAYER_NOISE_RATE - 0.1 - noised_ball_x,
                           std::fabs( tmp_pos.y - ball_rel.y ) );
 #endif
-            if ( tmp_recovery < M_world.self().recovery() )
-            {
-#ifdef DEBUG
-                dlog.addText( Logger::INTERCEPT,
-                              "________stamina exhaust. stamina = %f recover = %f",
-                              tmp_stamina,
-                              tmp_recovery );
-#endif
-                *save_recovery = false;
-            }
+            *result_recovery = tmp_recovery;
             return true;
         }
     }
 
 #ifdef DEBUG
     dlog.addText( Logger::INTERCEPT,
-                  "______dash %d: no run over. pmove=(%.2f(noised=%.2f), %.2f)  ball_x=%f"
-                  "  x_diff=%f y_diff=%f",
+                  "______dash %d: no run over. pmove=(%.2f(noised=%.2f), %.2f)  ball_x=%.3f"
+                  " x_diff=%.3f y_diff=%.3f",
                   n_dash,
                   tmp_pos.x, tmp_pos.x * PLAYER_NOISE_RATE, tmp_pos.y,
                   noised_ball_x,
@@ -1155,41 +1300,46 @@ SelfIntercept::canReachAfterDash( const int n_turn,
 
     //////////////////////////////////////////////////////////
     // when cycle is small, do strict check
-    if ( n_turn + n_dash <= 4 )
+    if ( n_turn + n_dash <= 6 )
     {
         // tmp_pos is relative to playerPos() --> tmp_pos.r() == player_travel
         double player_travel = tmp_pos.r();
-        double player_noise = player_travel * ServerParam::i().playerRand() * 0.5;
+        double player_noise = player_travel * param.playerRand() * 0.5;
         double last_ball_dist = ball_rel.dist( tmp_pos );
-        double buf = 0.2;
-        if ( n_turn > 0 ) buf += 0.3;
+        double buf = 0.15;
+        //if ( n_turn > 0 ) buf += 0.3;
         buf += player_noise;
         buf += ball_noise;
         if ( last_ball_dist < control_area - buf )
+//         if ( std::fabs( tmp_pos.x - ball_rel.x ) < control_area - 0.15
+//              && std::fabs( tmp_pos.y - ball_rel.y ) < control_area - buf
+//              && last_ball_dist < control_area )
         {
 #ifdef DEBUG
             dlog.addText( Logger::INTERCEPT,
-                          "____dash %d: can reach. last ball dist=%.3f. mod_ctrl_area=%.3f",
-                          n_dash, last_ball_dist, control_area - buf );
+                          "____dash %d: can reach. last ball dist=%.3f. noised_ctrl_area=%.3f/%.3f",
+                          n_dash,
+                          last_ball_dist,
+                          control_area - buf,
+                          control_area );
 #endif
-            if ( save_recovery && tmp_recovery < M_world.self().recovery() )
-            {
-#ifdef DEBUG
-                dlog.addText( Logger::INTERCEPT,
-                              "______dash %d: stamina exhaust",
-                              n_dash );
-#endif
-                *save_recovery = false;
-            }
+            *result_recovery = tmp_recovery;
             return true;
         }
+
 #ifdef DEBUG
         dlog.addText( Logger::INTERCEPT,
-                      "____%d Near Dash: failed. last_ball_dist=%.3f player_rel=(%.3f %.3f)  ball_rel=(%.3f %.3f)",
+                      "______dash %d: failed. last_ball_dist=%.3f noised_ctrl_area=%.3f/%.3f",
                       n_dash,
                       last_ball_dist,
+                      control_area - buf,
+                      control_area );
+        dlog.addText( Logger::INTERCEPT,
+                      "______player_rel=(%.3f %.3f) ball_rel=(%.3f %.3f)  p_noise=%.3f b_noise=%.3f",
                       tmp_pos.x, tmp_pos.y,
-                      ball_rel.x, ball_rel.y );
+                      ball_rel.x, ball_rel.y,
+                      player_noise,
+                      ball_noise );
         dlog.addText( Logger::INTERCEPT,
                       "______noised_ball_x=%.3f  bnoise=%.3f  pnoise=%.3f",
                       noised_ball_x,
