@@ -87,6 +87,7 @@ SelfObject::SelfObject()
     , M_kick_rate( 0.0 )
     , M_tackle_probability( 0.0 )
 {
+    M_unum_count = 0;
     M_type = Hetero_Default;
     M_player_type = PlayerTypeSet::i().get( Hetero_Default );
 
@@ -326,6 +327,7 @@ SelfObject::update( const ActionEffector & act,
     ++M_pos_count;
     ++M_seen_pos_count;
     ++M_vel_count;
+    ++M_seen_vel_count;
     ++M_body_count;
     ++M_face_count;
 
@@ -429,9 +431,9 @@ SelfObject::updateWindEffect()
 
 */
 void
-SelfObject::updateAfterSense( const BodySensor & sense,
-                              const ActionEffector & act,
-                              const GameTime & current )
+SelfObject::updateAfterSenseBody( const BodySensor & sense,
+                                  const ActionEffector & act,
+                                  const GameTime & current )
 {
     if ( M_sense_body_time == current )
     {
@@ -457,9 +459,7 @@ SelfObject::updateAfterSense( const BodySensor & sense,
 
     ////////////////////////////////////////////////////
     // stamina
-    M_stamina.updateAfterSense( playerType(),
-                                sense.stamina(), sense.effort(),
-                                current );
+    M_stamina.updateAfterSense( playerType(), sense, current );
 
     ////////////////////////////////////////////////////
     // neck relative
@@ -681,6 +681,8 @@ SelfObject::updateAfterFullstate( const FullstateSensor::PlayerT & my_state,
     M_vel = my_state.vel_;
     M_vel_error.assign( 0.0, 0.0 );
     M_vel_count = 0;
+    M_seen_vel = M_vel;
+    M_seen_vel_count = 0;
 
     M_body = my_state.body_;
     M_body_count = 0;
@@ -689,9 +691,7 @@ SelfObject::updateAfterFullstate( const FullstateSensor::PlayerT & my_state,
     M_face_error = 0.0;
     M_face_count = 0;
 
-    M_stamina.updateAfterFullstate( my_state.stamina_,
-                                    my_state.effort_,
-                                    my_state.recovery_ );
+    M_stamina.updateAfterFullstate( my_state );
 
     M_pointto_pos
         = pos()
@@ -706,11 +706,11 @@ SelfObject::updateAfterFullstate( const FullstateSensor::PlayerT & my_state,
 
 */
 void
-SelfObject::updatePos( const Vector2D & pos,
-                       const Vector2D & pos_err,
-                       const double & face,
-                       const double & face_err,
-                       const GameTime & current)
+SelfObject::updatePosBySee( const Vector2D & pos,
+                            const Vector2D & pos_err,
+                            const double & face,
+                            const double & face_err,
+                            const GameTime & current)
 {
     // other param is updated in update or update_after_sense
     M_time = current;
@@ -728,7 +728,7 @@ SelfObject::updatePos( const Vector2D & pos,
                 + ( M_pos.x - pos.x ) * ( pos_err.x / ( M_pos_error.x + pos_err.x ) );
             new_error.x = ( M_pos_error.x + pos_err.x ) * 0.5;
             dlog.addText( Logger::WORLD,
-                          __FILE__": updatePos. X modified [%f] < [%f]",
+                          __FILE__": updatePosBySee. X modified [%f] < [%f]",
                           M_pos_error.x, pos_err.x );
         }
         if  ( M_pos_error.y < pos_err.y )
@@ -738,14 +738,14 @@ SelfObject::updatePos( const Vector2D & pos,
                 + ( M_pos.y - pos.y ) * ( pos_err.y / ( M_pos_error.y + pos_err.y ) );
             new_error.y = ( M_pos_error.y + pos_err.y ) * 0.5;
             dlog.addText( Logger::WORLD,
-                          __FILE__": updatePos. Y modified [%f] < [%f]",
+                          __FILE__": updatePosBySee. Y modified [%f] < [%f]",
                           M_pos_error.y, pos_err.y );
         }
         M_pos = new_pos;
         M_pos_error = new_error;
 
         dlog.addText( Logger::WORLD,
-                      __FILE__": updatePos. seen_pos=(%.3f, %.3f) average_pos=(%.3f, %.3f) "
+                      __FILE__": updatePosBySee. seen_pos=(%.3f, %.3f) average_pos=(%.3f, %.3f) "
                       " error=(%.4f, %.4f)",
                       pos.x, pos.y, new_pos.x, new_pos.y,
                       M_pos_error.x, M_pos_error.y );
@@ -784,9 +784,9 @@ SelfObject::updateByCollision( const Vector2D & pos,
 
 */
 void
-SelfObject::updateAngle( const double & face,
-                         const double & face_err,
-                         const GameTime & current )
+SelfObject::updateAngleBySee( const double & face,
+                              const double & face_err,
+                              const GameTime & current )
 {
     M_time = current;
     M_face = face;
@@ -836,6 +836,8 @@ SelfObject::updateVelDirAfterSee( const BodySensor & sense,
 
         M_vel.setPolar( sense.speedMag(), vel_ang );
         M_vel_count = 0;
+        M_seen_vel = M_vel;
+        M_seen_vel_count = 0;
 
         // vel error
         double mincos, maxcos, minsin, maxsin;
@@ -885,7 +887,7 @@ SelfObject::updateBallInfo( const BallObject & ball )
     //------------------------------------------------------
     // check kickable
 
-    if ( ball.distFromSelf() <= this->kickableArea() - 0.06 )
+    if ( ball.distFromSelf() <= this->kickableArea() - 0.055 )
     {
         M_kickable = true;
 
@@ -913,12 +915,14 @@ SelfObject::updateBallInfo( const BallObject & ball )
         tackle_dist = ServerParam::i().tackleBackDist();
     }
 
-    double fail_prob
-        = ( std::pow( std::fabs( player2ball.x ) / tackle_dist,
-                      ServerParam::i().tackleExponent() )
-            + std::pow( std::fabs( player2ball.y )
-                        / ServerParam::i().tackleWidth(),
-                        ServerParam::i().tackleExponent() ) );
+    double fail_prob = 1.0;
+    if ( tackle_dist > 1.0e-5 )
+    {
+        fail_prob = ( std::pow( player2ball.absX() / tackle_dist,
+                                ServerParam::i().tackleExponent() )
+                      + std::pow( player2ball.absY() / ServerParam::i().tackleWidth(),
+                                  ServerParam::i().tackleExponent() ) );
+    }
 
     if ( fail_prob < 1.0 )
     {
@@ -941,48 +945,51 @@ SelfObject::updateBallInfo( const BallObject & ball )
 double
 SelfObject::getSafetyDashPower( const double & dash_power ) const
 {
+    double normalized_power = ServerParam::i().normalizeDashPower( dash_power );
+
     // required stamina value
-    double required_stamina = ( dash_power > 0.0
-                                ? dash_power
-                                : dash_power * -2.0 );
+    double required_stamina = ( normalized_power > 0.0
+                                ? normalized_power
+                                : normalized_power * -2.0 );
 
     if ( required_stamina < 0.0 )
     {
         std::cerr << "required stamina should be positive now "
                   << " input dash power = " << dash_power
+                  << " normalized power=" << normalized_power
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                      __FILE__": getSafetyDashPower. required stamina should be positive now" );
+                      __FILE__": getSafetyDashPower. input_dash_power=%.2f normalized_power=%.1f required_stamina=%.1f",
+                      dash_power,
+                      normalized_power,
+                      required_stamina );
     }
 
     // check rest stamina
 
     const double safety_stamina
-        = stamina() - ServerParam::i().recoverDecThrValue() - 3.0;
+        = stamina() - ServerParam::i().recoverDecThrValue() - 1.0;
 
     double available_stamina = std::max( 0.0, safety_stamina );
 
     double result_power = std::min( required_stamina, available_stamina );
     // backward dash case
-    if ( dash_power < 0.0 )
+    if ( normalized_power < 0.0 )
     {
         result_power *= -0.5;
     }
 
-    if ( std::fabs( result_power ) > std::fabs( dash_power ) )
+    if ( std::fabs( result_power ) > std::fabs( normalized_power ) )
     {
-        return dash_power;
+        return normalized_power;
     }
 
     dlog.addText( Logger::ACTION,
                   __FILE__": getSafetyDashPower. cur_stamina=%.1f, required=%.1f, result=%.2f",
                   stamina(), required_stamina, result_power );
-
-
     return result_power;
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
