@@ -35,6 +35,7 @@
 #include <rcsc/player/self_object.h>
 #include <rcsc/player/ball_object.h>
 #include <rcsc/player/player_object.h>
+#include <rcsc/player/view_area.h>
 
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/game_mode.h>
@@ -43,8 +44,6 @@
 
 #include <boost/shared_ptr.hpp>
 
-#include <list>
-#include <vector>
 #include <string>
 
 namespace rcsc {
@@ -59,7 +58,6 @@ class PlayerPredicate;
 class PlayerType;
 class PenaltyKickState;
 class VisualSensor;
-struct ViewArea;
 
 /*!
   \class WorldModel
@@ -72,13 +70,14 @@ public:
         DIR_CONF_DIVS = 72
     };
 
-    static const double DIR_STEP;
+    static const std::size_t MAX_RECORD; //!< max record size
+    static const double DIR_STEP; //!< the angle steps for dir confidence
 
 private:
 
     Localization * M_localize; //!< localization module
     InterceptTable * M_intercept_table; //!< interception info table
-    boost::shared_ptr< AudioMemory > M_audio_memory; //!< heard info memory
+    boost::shared_ptr< AudioMemory > M_audio_memory; //!< heard deqinfo memory
     PenaltyKickState * M_penalty_kick_state; //!< penalty kick mode status
 
     //////////////////////////////////////////////////
@@ -100,6 +99,8 @@ private:
 
     GameMode M_game_mode; //!< playmode and scores
 
+    bool M_valid;
+
     //////////////////////////////////////////////////
     // objects
     SelfObject M_self; //!< self object
@@ -124,6 +125,8 @@ private:
     //! sorted by distance from ball. include unknown players
     PlayerPtrCont M_opponents_from_ball;
 
+    //! uniform number of teammate goalie
+    int M_teammate_goalie_unum;
     //! uniform number of opponent goalie
     int M_opponent_goalie_unum;
 
@@ -158,6 +161,9 @@ private:
     //! array of direction confidence count
     int M_dir_count[DIR_CONF_DIVS];
 
+    //! view area history
+    ViewAreaCont M_view_area_cont;
+
     //////////////////////////////////////////////////
 
     //! not used
@@ -176,6 +182,18 @@ public:
     */
     ~WorldModel();
 
+    /*!
+      \brief get this world mode is valid or not
+      \return true if this world model is valid
+    */
+    bool valid() const;
+
+    /*!
+      \brief set this world mode is valid or not
+      \param is_valid value to be set
+    */
+    void setValid( bool is_valid );
+
     /*!
       \brief get intercept table
       \return const pointer to the intercept table instance
@@ -258,9 +276,9 @@ public:
 
       This method is called just after sense_body message receive
     */
-    void updateAfterSense( const BodySensor & sense,
-                           const ActionEffector & act,
-                           const GameTime & current );
+    void updateAfterSenseBody( const BodySensor & sense,
+                               const ActionEffector & act,
+                               const GameTime & current );
 
     /*!
       \brief update by see info
@@ -426,6 +444,11 @@ private:
     */
     void updateDirCount( const ViewArea & varea );
 
+    /*!
+      \brief update ball by heard info
+    */
+    void updateBallByHear( const ActionEffector & act );
+
     /*!
       \brief update opponent goalie by heard info
     */
@@ -441,13 +464,6 @@ private:
      */
     void updatePlayerType();
 
-    /*!
-      \brief update object relation
-
-      This method makes special player container
-    */
-    void updateObjectRelation();
-
     /*!
       \brief update player info. relation with ball and self.
      */
@@ -531,6 +547,16 @@ public:
           return M_time;
       }
 
+    /*!
+      \brief get last time updated by sense_body
+      \return const reference to the game time object
+    */
+    const
+    GameTime & senseBodyTime() const
+      {
+          return M_sense_body_time;
+      }
+
     /*!
       \brief get last time updated by see
       \return const reference to the game time object
@@ -677,6 +703,15 @@ public:
           return M_opponents_from_ball;
       }
 
+    /*!
+      \brief get the uniform number of teammate goalie
+      \return uniform number value or Unum_Unknown
+     */
+    int teammateGoalieUnum() const
+      {
+          return M_teammate_goalie_unum;
+      }
+
     /*!
       \brief get the uniform number of opponent goalie
       \return uniform number value or Unum_Unknown
@@ -778,20 +813,30 @@ public:
       \param first first iterator of PlayerObject pointer container
       \param last last iterator of PlayerObject pointer container
       \param count_thr accuracy count threshold
+      \param with_goalie if this value is false, goalie is ignored.
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
     PlayerObject * getFirstPlayer( PlayerPtrCont::const_iterator first,
                                    PlayerPtrCont::const_iterator last,
-                                   const int count_thr ) const
+                                   const int count_thr,
+                                   const bool with_goalie ) const
       {
           while ( first != last )
           {
+              if ( ! with_goalie
+                   && (*first)->goalie() )
+              {
+                  ++first;
+                  continue;
+              }
+
               if ( ! (*first)->isGhost()
                    && (*first)->posCount() <= count_thr )
               {
                   return *first;
               }
+
               ++first;
           }
           return static_cast< PlayerObject * >( 0 );
@@ -800,96 +845,116 @@ public:
     /*!
       \brief get teammate nearest to self with confidence count check
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
-    PlayerObject * getTeammateNearestToSelf( const int count_thr ) const
+    PlayerObject * getTeammateNearestToSelf( const int count_thr,
+                                             const bool with_goalie = true ) const
       {
           return getFirstPlayer( teammatesFromSelf().begin(),
                                  teammatesFromSelf().end(),
-                                 count_thr );
+                                 count_thr,
+                                 with_goalie );
       }
 
     /*!
       \brief get opponent nearest to self with accuracy count check
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
-    PlayerObject * getOpponentNearestToSelf( const int count_thr ) const
+    PlayerObject * getOpponentNearestToSelf( const int count_thr,
+                                             const bool with_goalie = true ) const
       {
           return getFirstPlayer( opponentsFromSelf().begin(),
                                  opponentsFromSelf().end(),
-                                 count_thr );
+                                 count_thr,
+                                 with_goalie );
       }
 
     /*!
       \brief get the distance to teammate nearest to self wtth accuracy count
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return distance to the matched opponent. if not found, a big value is returned.
      */
-    double getDistTeammateNearestToSelf( const int count_thr ) const
+    double getDistTeammateNearestToSelf( const int count_thr,
+                                         const bool with_goalie = true ) const
       {
-          const PlayerObject * p = getTeammateNearestToSelf( count_thr );
+          const PlayerObject * p = getTeammateNearestToSelf( count_thr, with_goalie );
           return ( p ? p->distFromSelf() : 65535.0 );
       }
 
     /*!
       \brief get the distance to opponent nearest to self wtth accuracy count
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return distance to the matched opponent. if not found, a big value is returned.
      */
-    double getDistOpponentNearestToSelf( const int count_thr ) const
+    double getDistOpponentNearestToSelf( const int count_thr,
+                                         const bool with_goalie = true) const
       {
-          const PlayerObject * p = getOpponentNearestToSelf( count_thr );
+          const PlayerObject * p = getOpponentNearestToSelf( count_thr, with_goalie );
           return ( p ? p->distFromSelf() : 65535.0 );
       }
 
     /*!
       \brief get teammate nearest to with with confidence count check
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
-    PlayerObject * getTeammateNearestToBall( const int count_thr ) const
+    PlayerObject * getTeammateNearestToBall( const int count_thr,
+                                             const bool with_goalie = true ) const
       {
           return getFirstPlayer( teammatesFromBall().begin(),
                                  teammatesFromBall().end(),
-                                 count_thr );
+                                 count_thr,
+                                 with_goalie );
       }
 
     /*!
       \brief get opponent nearest to ball with confidence count check
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return if found, const pointer to the PlayerOjbect. else NULL
     */
     const
-    PlayerObject * getOpponentNearestToBall( const int count_thr ) const
+    PlayerObject * getOpponentNearestToBall( const int count_thr,
+                                             const bool with_goalie = true ) const
       {
           return getFirstPlayer( opponentsFromBall().begin(),
                                  opponentsFromBall().end(),
-                                 count_thr );
+                                 count_thr,
+                                 with_goalie );
       }
 
     /*!
       \brief get the distance to teammate nearest to ball wtth accuracy count
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return distance to the matched opponent. if not found, a big value is returned.
      */
-    double getDistTeammateNearestToBall( const int count_thr ) const
+    double getDistTeammateNearestToBall( const int count_thr,
+                                         const bool with_goalie ) const
       {
-          const PlayerObject * p = getTeammateNearestToSelf( count_thr );
+          const PlayerObject * p = getTeammateNearestToSelf( count_thr, with_goalie );
           return ( p ? p->distFromBall() : 65535.0 );
       }
 
     /*!
       \brief get the distance to opponent nearest to ball wtth accuracy count
       \param count_thr accuracy count threshold
+      \param with_goalie include goalie if true
       \return distance to the matched opponent. if not found, a big value is returned.
      */
-    double getDistOpponentNearestToBall( const int count_thr ) const
+    double getDistOpponentNearestToBall( const int count_thr,
+                                         const bool with_goalie = true ) const
       {
-          const PlayerObject * p = getOpponentNearestToBall( count_thr );
+          const PlayerObject * p = getOpponentNearestToBall( count_thr, with_goalie );
           return ( p ? p->distFromBall() : 65535.0 );
       }
 
@@ -1025,6 +1090,25 @@ public:
                        int * sum_count,
                        int * ave_count ) const;
 
+    /*!
+      \brief get view area history container
+      \return const refrence to the view area container.
+     */
+    const
+    ViewAreaCont & viewAreaCont() const
+      {
+          return M_view_area_cont;
+      }
+
+    /*!
+      \brief get the specific point accuracy count
+      \param point global cooridinate value of checked point
+      \param dir_thr direction threshold for view area
+      \return accuracy count. if player has not seen the point, returnthe big value (e.g. 1000).
+     */
+    int getPointCount( const Vector2D & point,
+                       const double & dir_thr ) const;
+
     //
     // interfaces to player objects
     //
@@ -1044,10 +1128,16 @@ public:
     void getPlayerCont( AbstractPlayerCont & cont,
                         const PlayerPredicate * predicate ) const;
 
+    /*!
+      \brief get teammate goalie pointer
+      \return if found pointer to goalie object, otherwise NULL
+    */
+    const
+    PlayerObject * getTeammateGoalie() const;
 
     /*!
       \brief get opponent goalie pointer
-      \return if found pointer to goalie object, othewise NULL
+      \return if found pointer to goalie object, otherwise NULL
     */
     const
     PlayerObject * getOpponentGoalie() const;
