@@ -202,7 +202,7 @@ public:
     virtual
     bool handleMsg( const int time,
                     const int board,
-                    const char * msg ) = 0;
+                    const std::string & msg ) = 0;
 
     /*!
       \brief handle playmode
