@@ -32,6 +32,8 @@
 #ifndef RCSC_RCG_PARSER_H
 #define RCSC_RCG_PARSER_H
 
+#include <rcsc/factory.h>
+
 #include <boost/shared_ptr.hpp>
 
 #include <istream>
@@ -48,6 +50,29 @@ class Handler;
   \brief rcg stream parser interface class
 */
 class Parser {
+public:
+
+    typedef boost::shared_ptr< Parser > Ptr; //!< rcg parser pointer type
+    typedef Ptr (*Creator)(); //!< rcg parser creator function
+    typedef rcss::Factory< Creator, int > FactoryHolder; //!< creator function holder
+
+    /*!
+      \brief factory holder singleton
+      \return reference to the factory holder instance
+     */
+    static
+    FactoryHolder & factoryHolder();
+
+    /*!
+      \brief create a suitable version parser instance depending on the input stream.
+      \param is reference to the imput stream.
+      \return smart pointer to the rcg parser instance
+
+      poionted index of istream becomes 4.
+     */
+    static
+    Ptr create( std::istream & is );
+
 protected:
 
     /*!
@@ -84,10 +109,6 @@ public:
                 Handler & handler ) const = 0;
 };
 
-
-//! typedef of the rcg parser pointer.
-typedef boost::shared_ptr< Parser > ParserPtr;
-
 } // end of namespace
 } // end of namespace
 
