@@ -190,5 +190,23 @@ ParserV2::parseMsgInfo( std::istream & is,
     return result;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Parser::Ptr
+create_v2()
+{
+    Parser::Ptr ptr( new ParserV2() );
+    return ptr;
+}
+
+rcss::RegHolder v2 = Parser::factoryHolder().autoReg( &create_v2, REC_VERSION_2 );
+
+}
+
 } // end of namespace
 } // end of namespace
