@@ -304,5 +304,23 @@ SerializerV2::serialize( std::ostream & os,
     return serialize( os, disp.show_ );
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Serializer::Ptr
+create_v2()
+{
+    Serializer::Ptr ptr( new SerializerV2() );
+    return ptr;
+}
+
+rcss::RegHolder v2 = Serializer::factoryHolder().autoReg( &create_v2, REC_VERSION_2 );
+
+}
+
 } // end of namespace
 } // end of namespace
