@@ -44,8 +44,8 @@
 
 #include "util.h"
 
-#include <cmath>
 #include <cstring>
+#include <cmath>
 
 namespace {
 inline
@@ -434,7 +434,7 @@ SerializerV4::serialize( std::ostream & os,
  */
 std::ostream &
 SerializerV4::serialize( std::ostream & os,
-                         const drawinfo_t & draw )
+                         const drawinfo_t & )
 {
     // TODO
     return os;
@@ -520,9 +520,16 @@ SerializerV4::serialize( std::ostream & os,
     // ball
 
     os << " ((b)"
-       << ' ' << show.ball_.x_ << ' ' << show.ball_.y_
-       << ' ' << show.ball_.vx_ << ' ' << show.ball_.vy_
-       << ')';
+       << ' ' << show.ball_.x_ << ' ' << show.ball_.y_;
+    if ( show.ball_.hasVelocity() )
+    {
+        os << ' ' << show.ball_.vx_ << ' ' << show.ball_.vy_;
+    }
+    else
+    {
+        os << " 0 0";
+    }
+    os << ')';
 
     // players
 
@@ -536,18 +543,43 @@ SerializerV4::serialize( std::ostream & os,
            << p.state_
            << std::dec << std::noshowbase;
 
-        os << ' ' << p.x_ << ' ' << p.y_
-           << ' ' << p.vx_ << ' ' << p.vy_
-           << ' ' << p.body_ << ' ' << p.neck_;
-        if ( p.point_x_ != SHOWINFO_SCALE2F
-             && p.point_y_ != SHOWINFO_SCALE2F )
+        os << ' ' << p.x_ << ' ' << p.y_;
+        if ( p.hasVelocity() )
+        {
+            os << ' ' << p.vx_ << ' ' << p.vy_;
+        }
+        else
+        {
+            os << " 0 0";
+        }
+        os << ' ' << p.body_
+           << ' ' << ( p.hasNeck() ? p.neck_ : 0.0f );
+
+        if ( p.isPointing() )
         {
             os << ' ' << p.point_x_ << ' ' << p.point_y_;
         }
 
-        os << " (v " << p.view_quality_ << ' ' << p.view_width_ << ')';
+        if ( p.hasView() )
+        {
+            os << " (v " << p.view_quality_ << ' ' << p.view_width_ << ')';
+        }
+        else
+        {
+            os << " (v h 90)";
+        }
 
-        os << " (s " << p.stamina_ << ' ' << p.effort_ << ' ' << p.recovery_ << ')';
+        if ( p.hasStamina() )
+        {
+            os << " (s " << p.stamina_
+               << ' ' << p.effort_
+               << ' ' << p.recovery_
+               << ')';
+        }
+        else
+        {
+            os << " (s 4000 1 1)";
+        }
 
         if ( p.focus_side_ != 'n' )
         {
@@ -607,5 +639,22 @@ SerializerV4::serialize( std::ostream & os,
 }
 
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Serializer::Ptr
+create_v4()
+{
+    Serializer::Ptr ptr( new SerializerV4() );
+    return ptr;
+}
+
+rcss::RegHolder v4 = Serializer::factoryHolder().autoReg( &create_v4, REC_VERSION_4 );
+
+}
+
 } // end of namespace
 } // end of namespace
