@@ -89,23 +89,24 @@ enum MsgInfoMode {
   \brief player status bit mask.
  */
 enum PlayerStatus {
-    DISABLE         = 0x0000,
-    STAND           = 0x0001,
-    KICK            = 0x0002,
-    KICK_FAULT      = 0x0004,
-    GOALIE          = 0x0008,
-    CATCH           = 0x0010,
-    CATCH_FAULT     = 0x0020,
-    BALL_TO_PLAYER  = 0x0040,
-    PLAYER_TO_BALL  = 0x0080,
-    DISCARD         = 0x0100,
-    LOST            = 0x0200,
-    BALL_COLLIDE    = 0x0400,
-    PLAYER_COLLIDE  = 0x0800,
-    TACKLE          = 0x1000,
-    TACKLE_FAULT    = 0x2000,
-    BACK_PASS       = 0x4000,
-    FREE_KICK_FAULT = 0x8000
+    DISABLE         = 0x00000000,
+    STAND           = 0x00000001,
+    KICK            = 0x00000002,
+    KICK_FAULT      = 0x00000004,
+    GOALIE          = 0x00000008,
+    CATCH           = 0x00000010,
+    CATCH_FAULT     = 0x00000020,
+    BALL_TO_PLAYER  = 0x00000040,
+    PLAYER_TO_BALL  = 0x00000080,
+    DISCARD         = 0x00000100,
+    LOST            = 0x00000200,
+    BALL_COLLIDE    = 0x00000400,
+    PLAYER_COLLIDE  = 0x00000800,
+    TACKLE          = 0x00001000,
+    TACKLE_FAULT    = 0x00002000,
+    BACK_PASS       = 0x00004000,
+    FREE_KICK_FAULT = 0x00008000,
+    POST_COLLIDE    = 0x00010000, // player collided with goal posts
 };
 
 
@@ -113,8 +114,8 @@ enum PlayerStatus {
 // monitor protocol version 1
 // game log format versin 1 & version 2
 
-const double SHOWINFO_SCALE = 16.0; //!< scaling parameter
-const float SHOWINFO_SCALEF = 16.0f;
+const double SHOWINFO_SCALE = 16.0; //!< scaling factor
+const float SHOWINFO_SCALEF = 16.0f; //!< scaling factor
 const int REC_OLD_VERSION = 1; //!< version number of rcg v1
 const int REC_VERSION_2 = 2; //!< recorded value of rcg v2
 
@@ -552,9 +553,19 @@ struct BallT {
     BallT()
         : x_( 0.0 )
         , y_( 0.0 )
-        , vx_( 0.0 )
-        , vy_( 0.0 )
+        , vx_( SHOWINFO_SCALE2F )
+        , vy_( SHOWINFO_SCALE2F )
       { }
+
+    /*!
+      \brief check if this object has velocity info
+      \return true if this object has velocity info
+     */
+    bool hasVelocity() const
+      {
+          return vx_ != SHOWINFO_SCALE2F
+              && vy_ != SHOWINFO_SCALE2F;
+      }
 };
 
 /*!
@@ -587,6 +598,7 @@ struct PlayerT {
     float stamina_; //!< satamina value
     float effort_; //!< effort value
     float recovery_; //!< recovery value
+    float stamina_capacity_; //!< stamina capacity value
 
     UInt16 kick_count_; //!< kick command count
     UInt16 dash_count_; //!< dash command count
@@ -613,16 +625,17 @@ struct PlayerT {
         , state_( 0 )
         , x_( 0.0f )
         , y_( 0.0f )
-        , vx_( 0.0f )
-        , vy_( 0.0f )
+        , vx_( SHOWINFO_SCALE2F )
+        , vy_( SHOWINFO_SCALE2F )
         , body_( 0.0f )
-        , neck_( 0.0f )
+        , neck_( SHOWINFO_SCALE2F )
         , point_x_( SHOWINFO_SCALE2F )
         , point_y_( SHOWINFO_SCALE2F )
         , view_width_( 0.0f )
-        , stamina_( -1.0f )
-        , effort_( 0.0f )
-        , recovery_( 0.0f )
+        , stamina_( SHOWINFO_SCALE2F )
+        , effort_( SHOWINFO_SCALE2F )
+        , recovery_( SHOWINFO_SCALE2F )
+        , stamina_capacity_( -1.0f )
         , kick_count_( 0 )
         , dash_count_( 0 )
         , turn_count_( 0 )
@@ -647,6 +660,10 @@ struct PlayerT {
                    : NEUTRAL );
       }
 
+    /*!
+      \brief check if view quality is high.
+      \return true if view quality is high.
+     */
     bool highQuality() const
       {
           return ( view_quality_ == 'h' );
@@ -662,6 +679,162 @@ struct PlayerT {
                    : side_ == 'r' ? RIGHT
                    : NEUTRAL );
       }
+
+    /*!
+      \brief check if this object has velocity info
+      \return true if this object has velocity info
+     */
+    bool hasVelocity() const
+      {
+          return vx_ != SHOWINFO_SCALE2F
+              && vy_ != SHOWINFO_SCALE2F;
+      }
+
+    /*!
+      \brief check if this object has neck info
+      \return true if this object has neck info
+     */
+    bool hasNeck() const
+      {
+          return neck_ != SHOWINFO_SCALE2F;
+      }
+
+    /*!
+      \brief check if this object has view width info
+      \return true if this object has view width info
+     */
+    bool hasView() const
+      {
+          return view_width_ != SHOWINFO_SCALE2F;
+      }
+
+    /*!
+      \brief check if this object has stamina info
+      \return true if this object has stamina info
+     */
+    bool hasStamina() const
+      {
+          return stamina_ != SHOWINFO_SCALE2F;
+      }
+
+    /*!
+      \brief check if this object has stamina capacity info
+      \return true if this object has stamina capacity info
+     */
+    bool hasStaminaCapacity() const
+      {
+          return stamina_capacity_ >= 0;
+      }
+
+    /*!
+      \brief check if this object is enabled.
+      \return true if this object is enabled.
+     */
+    bool isAlive() const
+      {
+          return state_ != 0;
+      }
+
+    /*!
+      \brief check if this object is kicking.
+      \return true if this object is kicking.
+     */
+    bool isKicking() const
+      {
+          return state_ & KICK;
+      }
+
+    /*!
+      \brief check if this object is kicking fault.
+      \return true if this object is kicking fault.
+     */
+    bool isKickingFault() const
+      {
+          return state_ & KICK_FAULT;
+      }
+
+    /*!
+      \brief check if this object is goalie.
+      \return true if this object is goalie.
+     */
+    bool isGoalie() const
+      {
+          return state_ & GOALIE;
+      }
+
+    /*!
+      \brief check if this object is catching.
+      \return true if this object is catching.
+     */
+    bool isCatching() const
+      {
+          return state_ & CATCH;
+      }
+
+    /*!
+      \brief check if this object is catching fault.
+      \return true if this object is catching fault.
+     */
+    bool isCatchingFault() const
+      {
+          return state_ & CATCH_FAULT;
+      }
+
+    /*!
+      \brief check if this object is tackling
+      \return true if this object is tackling
+     */
+    bool isTackling() const
+      {
+          return state_ & TACKLE;
+      }
+
+    /*!
+      \brief check if this object is tackling fault.
+      \return true if this object is tackling fault.
+     */
+    bool isTacklingFault() const
+      {
+          return state_ & TACKLE_FAULT;
+      }
+
+    /*!
+      \brief check if this object is pointing to somewhere.
+      \return true if this object is pointing to somewhere.
+     */
+    bool isPointing() const
+      {
+          return point_x_ != SHOWINFO_SCALE2F
+              && point_y_ != SHOWINFO_SCALE2F;
+      }
+
+    /*!
+      \brief check if this object is focusing to someone.
+      \return true if this object is focusing to someone.
+     */
+    bool isFocusing() const
+      {
+          return side_ != 'n';
+      }
+
+    /*!
+      \brief check if this object is collided with ball.
+      \return true if this object is collided with ball.
+     */
+    bool isCollidedBall() const
+      {
+          return state_ & BALL_COLLIDE;
+      }
+
+    /*!
+      \brief check if this object is collided with player.
+      \return true if this object is collided with player.
+     */
+    bool isCollidedPlayer() const
+      {
+          return state_ & PLAYER_COLLIDE;
+      }
+
 };
 
 /*!
@@ -740,11 +913,11 @@ struct TeamT {
   \brief show information
  */
 struct ShowInfoT {
-    UInt32 time_;
+    UInt32 time_; //!< game time
     //    PlayMode pmode_;
     //    TeamT team_[2];
-    BallT ball_;
-    PlayerT player_[MAX_PLAYER * 2];
+    BallT ball_; //!< ball data
+    PlayerT player_[MAX_PLAYER * 2]; //!< player data
 };
 
 
@@ -753,17 +926,20 @@ struct ShowInfoT {
   \brief display information
  */
 struct DispInfoT {
-    PlayMode pmode_;
-    TeamT team_[2];
-    ShowInfoT show_;
+    PlayMode pmode_; //!< playmode id
+    TeamT team_[2]; //!< team data
+    ShowInfoT show_; //!< positional data
 };
 
 
 //! recorded value of rcg v4
 const int REC_VERSION_4 = 4;
 
+//! recorded value of rcg v5
+const int REC_VERSION_5 = 5;
+
 //! default rcg version
-const int DEFAULT_LOG_VERSION = REC_VERSION_4;
+const int DEFAULT_LOG_VERSION = REC_VERSION_5;
 
 } // end namespace
 } // end namespace
